<?php

namespace App\Http\Controllers;

use App\Jobs\AnnouncementAlert;
use App\Models\Announcement;
use App\Models\AnnouncementsRead;
use App\Models\GeneralNotification;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\Storage;

class AnnouncementsController extends Controller
{
    //
    public function index(Request $request)
    {

        $posts = Announcement::with(['addedBy'])->where('instance_id','=',getActiveInstance())->orderBy('publish_date','desc');

        if ($request->title) {
            $posts = $posts->where('title','like','%'.$request->title.'%');
        }

        if ($request->status) {
            $posts = $posts->where('status','=',$request->status);
        }

        if ( isset($request->date_range)) {
            $dates = getDateTRange($request->date_range);
            $posts = $posts->where('publish_date','>=',$dates['from'])->where('publish_date','<=',$dates['to']);
        }

        $posts = $posts->paginate(100);

        return view('announcement.list', compact('posts'));
    }

    public function create()
    {
        $announcement = [];
        $announcement_id = null;
        $action = 'new';
        return view('announcement.create', compact('announcement_id','action','announcement'));
    }

    public function edit($announcement_id)
    {
        $announcement = Announcement::find(Crypt::decrypt($announcement_id));
        $action = 'edit';
        return view('announcement.create', compact('announcement_id','action','announcement'));
    }

    public function view($announcement_id)
    {
        $announcement = Announcement::find(Crypt::decrypt($announcement_id));
        $action = 'view';
        $can_edit = (canDo('administrator','edit-announcement'));
        $announcements_route = route('announcement.list');
        return view('announcement.view', compact('announcement_id','action',
            'announcement','announcements_route','can_edit'));
    }

    public function save(Request $request)
    {
        $request->validate([
            'title' =>  'required|string|min:3',
            'announcement_content' =>  'required|string|min:10',
            'status' =>  'required|string',
        ]);

        $deployment = getActiveDeployment();
        $applicants = $deployment->applicants->pluck('regID')->toArray();
        $receivers = null;
        if (!is_null($applicants)) {
            $receivers = User::whereIn('applicant_number',$applicants)->pluck('id')->toArray();
        }

        if (!$request->announcement_id) {
            $announcement = new Announcement();
            $announcement->instance_id = getActiveInstance();
        }
        else{
            $announcement = Announcement::find(Crypt::decrypt($request->announcement_id));

            if (!$announcement) {
                $message = "Announcement Post does not exist.";
                session()->flash('status', 'error');
                session()->flash('message', $message);
                return back()->withInput();
            }
        }

        if($request->file('featured_image')) {
            $announcement->featured_image = $this->uploadDoc();
        }

        if(isset($request->send_email)) {
            $announcement->pending_alert = true;
        }

        $visibility = false;
        if(isset($request->visibility)) {
            $visibility = true;
        }

        if ($request->schedule_date) {
            $publish_date = date('Y-m-d H:i:s',strtotime($request->schedule_date.' '.$request->schedule_time));
        }

        $announcement->title = $request->title;
        $announcement->content = $request->announcement_content;
        $announcement->status = $request->status;
        $announcement->visibility = $visibility;
        $announcement->publish_date = $publish_date;
        $announcement->user_id = auth()->id();

        if($announcement->save()) {

            AnnouncementAlert::dispatch($announcement);

            GeneralNotification::create([
                'user_id'   =>  auth()->id(),
                'receivers'   =>  $receivers,
                'subject_id'   =>  $announcement->id,
                'table'   =>  $announcement->getModel()->getTable(),
                'instance_id'   =>  $announcement->instance_id,
                'message'   =>  '<strong>New Announcement:</strong> '.content_excerpt($announcement->title,50),
                'route'   =>  route('customer.customerAnnouncementView',[cryptThis($announcement->id)]),
            ]);

            $message = "Announcement has been saved.";
            session()->flash('status', 'success');
            session()->flash('message', $message);
        }
        else{
            $message = "Sorry! Something went wrong. Please try again";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }

        return redirect()->route('announcement.list');
    }

    public function uploadDoc($key = 'featured_image', $validation = 'image|mimes:jpeg,png,jpg,gif|max:20048|sometimes')
    {
        $folder = getActiveInstance().'/announcement/featured-images';
        request()->validate([$key => $validation]);

        $file = null;
        if (request()->hasFile($key)) {
            $file = request()->file($key);
            $name = $file->hashName();
            $file = Storage::disk('public')->putFileAs($folder, request()->file($key), $name);
        }

        return 'storage/'.$file;
    }

    public function delete(Request $request)
    {
        if (Announcement::destroy(Crypt::decrypt($request->id))) {
            $message = "Announcement Post has been deleted.";
            session()->flash('status', 'success');
            session()->flash('message', $message);
        }
        else{
            $message = "Invalid Announcement";
            session()->flash('status', 'error');
            session()->flash('message', $message);
        }
        return redirect()->route('announcement.list');
    }

    //Customer Section
    public function customerAnnouncements()
    {
        $subscriptions = auth()->user()->deployments()->pluck('id')->toArray();
        $announcements = Announcement::whereIn('instance_id',$subscriptions)->orderBy('updated_at','desc')->paginate(50);
        return view('customers.announcements', compact('announcements'));
    }

    public function customerAnnouncementView($id)
    {
        $announcement = Announcement::findOrFail(Crypt::decrypt($id));

        updateNotification($announcement->id,$announcement->getTable());

        if(!hasReadAnnouncement(Crypt::decrypt($id))){
            AnnouncementsRead::create([
                'user_id'   =>  auth()->id(),
                'announcement_id'   =>  Crypt::decrypt($id),
                'read_at'   =>  now(),
            ]);
        }

        $announcements_route = route('customer.customerAnnouncements');
        return view('customers.announcement-view', compact('announcement','announcements_route'));
    }
}
