<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Traits\HttpResponses;
use App\Models\Area;
use App\Models\City;
use App\Models\Country;
use App\Models\County;
use App\Models\Region;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Str;




class LocationApiController extends Controller
{
    use HttpResponses;


    /**
     * @OA\Get(
     *      path="/api/get-nationalities",
     *      operationId="getNationalities",
     *      tags={"1b. Geolocations"},
     *      summary="Get List of All Nationalities",
     *      description="Returns all active nationalities",
     *      @OA\Response(
     *          response=200,
     *          description="successful operation"
     *       ),
     *       @OA\Response(response=400, description="Bad request"),
     *
     *     )
     *
     * Returns List of All Active Nationalities
     */
    public function getNationalities()
    {
        $countries = Country::active()
            ->orderBy('nationality')->pluck('nationality','id');
        return $this->successResponse($countries,'All available nationalities');
    }

    /**
     * @OA\Get(
     *      path="/api/get-country/{id}",
     *      operationId="getCountry",
     *      tags={"1b. Geolocations"},
     *      summary="Get details of a country",
     *      description="Returns details of a country selected",
     *      @OA\Parameter(
     *          name="id",
     *          description="Country ID unique identity",
     *          required=true,
     *          in="path",
     *          @OA\Schema(
     *              type="integer",
     *          ),
     *      ),
     *      @OA\Response(
     *          response=200,
     *          description="successful operation"
     *       ),
     *       @OA\Response(response=400, description="Bad request"),
     *
     *     )
     *
     * Returns Details of a specific country
     */
    public function getCountry($id)
    {
        $country = Country::find($id);
        if ($country) return $this->successResponse($country,'Country found');
        return $this->errorResponse(null,'Invalid country',204);
    }

    /**
     * @OA\Get(
     *      path="/api/get-countries",
     *      operationId="getCountries",
     *      tags={"1b. Geolocations"},
         *      summary="Get List of All Countries",
     *      description="Returns all active countries",
     *      @OA\Response(
     *          response=200,
     *          description="successful operation"
     *       ),
     *       @OA\Response(response=400, description="Bad request"),
     *
     *     )
     *
     * Returns List of All Active Countries
     */
    public function getCountries()
    {
        $countries = Country::has('regions', '>' , 0)->active()
            ->select(['id','country_code','country_name','system','phone_code','currency_name','currency_code','currency_symbol','currency_coins_name','coins_code'])
            ->orderBy('country_name')->get();
        return $this->successResponse($countries->toArray(),'All available countries');
    }

    /**
     * @OA\Get(
     *      path="/api/get-state/{id}",
     *      operationId="getState",
     *      tags={"1b. Geolocations"},
     *      summary="Get details of the state",
     *      description="Returns details of state selected",
     *      @OA\Parameter(
     *          name="id",
     *          description="state ID unique identity",
     *          required=true,
     *          in="path",
     *          @OA\Schema(
     *              type="integer",
     *          ),
     *      ),
     *      @OA\Response(
     *          response=200,
     *          description="successful operation"
     *       ),
     *       @OA\Response(response=400, description="Bad request"),
     *       @OA\Response(response=204, description="Invalid state"),
     *     )
     *
     * Returns Details of a specific state
     */
    public function getState($id)
    {
        $region = Region::find($id);
        if ($region) return $this->successResponse($region,'State found');
        return $this->errorResponse(null,'Invalid state',204);
    }

    /**
     * @OA\Get(
     *      path="/api/get-country-states/{country_id}",
     *      operationId="getStatesOrRegions",
     *      tags={"1b. Geolocations"},
     *      summary="Get details of the state or region based on country id",
     *      description="Returns details of state based on country id",
     *      @OA\Parameter(
     *          name="country_id",
     *          description="country ID unique identity",
     *          required=true,
     *          in="path",
     *          @OA\Schema(
     *              type="integer",
     *          ),
     *      ),
     *      @OA\Response(
     *          response=200,
     *          description="successful operation"
     *       ),
     *       @OA\Response(response=400, description="Bad request"),
     *       @OA\Response(response=204, description="Invalid state"),
     *     )
     *
     * Returns Details of a specific state based on country ID
     */
    public function getStatesOrRegions($country_id)
    {
        $country = Country::where('id','=',$country_id)
            ->select(['id','country_code','country_name','system'])
            ->first()->toArray();

        $regions = Region::has('counties','>',0)->active()
            ->select(['id','name','system'])
            ->where('country_id','=',$country_id)
            ->orderBy('name')->get();

        return $this->successResponse(['country'=>$country,'states'=>$regions->toArray()],'All available '.Str::plural($country['system']));
    }

    /**
     * @OA\Get(
     *      path="/api/get-county-district/{id}",
     *      operationId="getCounty",
     *      tags={"1b. Geolocations"},
     *      summary="Get details of a county",
     *      description="Returns details of county",
     *      @OA\Parameter(
     *          name="id",
     *          description="county ID unique identity",
     *          required=true,
     *          in="path",
     *          @OA\Schema(
     *              type="integer",
     *          ),
     *      ),
     *      @OA\Response(
     *          response=200,
     *          description="County/District found"
     *       ),
     *       @OA\Response(response=400, description="Bad request"),
     *       @OA\Response(response=204, description="Invalid County/District"),
     *     )
     *
     * Returns Details of a specific county
     */
    public function getCounty($id)
    {
        $county = County::find($id);
        if ($county) return $this->successResponse($county,'County/District found');
        return $this->errorResponse(null,'Invalid County/District',204);
    }

    /**
     * @OA\Get(
     *      path="/api/get-state-counties/{state_id}",
     *      operationId="getDistrictsOrCounties",
     *      tags={"1b. Geolocations"},
     *      summary="Get list of counties, region and country by state ID",
     *      description="Returns details of counties and its region and country based on state ID",
     *      @OA\Parameter(
     *          name="state_id",
     *          description="state ID unique identity",
     *          required=true,
     *          in="path",
     *          @OA\Schema(
     *              type="integer",
     *          ),
     *      ),
     *      @OA\Response(
     *          response=200,
     *          description="Successful Operation"
     *       ),
     *       @OA\Response(response=400, description="Bad request"),
     *
     *     )
     *
     * Returns List of Counties and its Region and Country
     */
    public function getDistrictsOrCounties($region_id)
    {
        $region = Region::where('id','=',$region_id)
            ->select(['id','name','system','country_id'])
            ->first()->toArray();

        $country = Country::where('id','=',$region['country_id'])
            ->select(['id','country_code','country_name','system'])
            ->first()->toArray();

        $counties = County::active()
            ->select(['id','county as name','system'])
            ->where('region_id','=',$region_id)
            ->orderBy('name')->get();
        return $this->successResponse(['country'=>$country,'state'=>$region,'counties'=>$counties->toArray()],'All available '.Str::plural($region['system']));
    }

    /**
     * @OA\Get(
     *      path="/api/get-city/{id}",
     *      operationId="getCity",
     *      tags={"1b. Geolocations"},
     *      summary="Get details of city by ID",
     *      description="Returns details of city based on city ID",
     *      @OA\Parameter(
     *          name="id",
     *          description="City ID unique identity",
     *          required=true,
     *          in="path",
     *          @OA\Schema(
     *              type="integer",
     *          ),
     *      ),
     *      @OA\Response(
     *          response=200,
     *          description="Successful Operation"
     *       ),
     *       @OA\Response(response=400, description="Bad request"),
     *       @OA\Response(response=204, description="Invalid City"),
     *
     *     )
     *
     * Returns Details of City based on specific city ID
     */
    public function getCity($id)
    {
        $city = City::find($id);
        if ($city) return $this->successResponse($city,'City found');
        return $this->errorResponse(null,'Invalid City',204);
    }

    /**
     * @OA\Get(
     *      path="/api/get-cities/{county_id}",
     *      operationId="getDistrictsCities",
     *      tags={"1b. Geolocations"},
     *      summary="Get List of cities and other details by County ID",
     *      description="Returns List of cities,County, Region, Country based on county ID",
     *      @OA\Parameter(
     *          name="county_id",
     *          description="County ID unique identity",
     *          required=true,
     *          in="path",
     *          @OA\Schema(
     *              type="integer",
     *          ),
     *      ),
     *      @OA\Response(
     *          response=200,
     *          description="Successful Operation"
     *       ),
     *       @OA\Response(response=400, description="Bad request"),
     *
     *
     *     )
     *
     * Returns List of Cities, County, Region, Country based on specific county ID
     */
    public function getDistrictsCities($county_id)
    {
        $county = County::where('id','=',$county_id)
            ->select(['id','county as name','system','region_id','country_id'])
            ->first()->toArray();

        $region = Region::where('id','=',$county['region_id'])
            ->select(['id','name','system','country_id'])
            ->first()->toArray();

        $country = Country::where('id','=',$county['country_id'])
            ->select(['id','country_code','country_name','system'])
            ->first()->toArray();

        $cities = City::has('areas','>',0)->active()
            ->select(['id','name','system'])
            ->where('county_id','=',$county_id)
            ->orderBy('name')->get();
        return $this->successResponse([
            'country'=>$country,
            'state'=>$region,
            'county'=>$county,
            'cities'=>$cities->toArray()],'All available '.Str::plural($county['system']));
    }

    /**
     * @OA\Get(
     *      path="/api/get-area/{id}",
     *      operationId="getArea",
     *      tags={"1b. Geolocations"},
     *      summary="Get details of area by ID",
     *      description="Returns details of area based on Area ID",
     *      @OA\Parameter(
     *          name="id",
     *          description="Area ID unique identity",
     *          required=true,
     *          in="path",
     *          @OA\Schema(
     *              type="integer",
     *          ),
     *      ),
     *      @OA\Response(
     *          response=200,
     *          description="Area/Community found"
     *       ),
     *       @OA\Response(response=400, description="Bad request"),
     *       @OA\Response(response=204, description="Invalid Area/Community"),
     *
     *     )
     *
     * Returns Details of City based on specific city ID
     */
    public function getArea($id)
    {
        $area = Area::find($id);
        if ($area) return $this->successResponse($area,'Area/Community found');
        return $this->errorResponse(null,'Invalid Area/Community',204);
    }

    /**
     * @OA\Get(
     *      path="/api/get-city-areas/{city_id}",
     *      operationId="getCityAreas",
     *      tags={"1b. Geolocations"},
     *      summary="Get list of Areas, City, County, Region, Country by ID",
     *      description="Returns List of Areas, City, County, Region, Country based on Area ID",
     *      @OA\Parameter(
     *          name="id",
     *          description="City ID unique identity",
     *          required=true,
     *          in="path",
     *          @OA\Schema(
     *              type="integer",
     *          ),
     *      ),
     *      @OA\Response(
     *          response=200,
     *          description="Successful Operation"
     *       ),
     *       @OA\Response(response=400, description="Bad request"),
     *
     *     )
     *
     * Returns list of Areas, City, County, Region, Country based on specific city ID
     */
    public function getCityAreas($city_id)
    {
        $city = City::where('id','=',$city_id)
            ->select(['id','name','system','county_id','region_id','country_id'])
            ->first()->toArray();

        $county = County::where('id','=',$city['county_id'])
            ->select(['id','county as name','system'])
            ->first()->toArray();

        $region = Region::where('id','=',$city['region_id'])
            ->select(['id','name','system','country_id'])
            ->first()->toArray();

        $country = Country::where('id','=',$city['country_id'])
            ->select(['id','country_code','country_name','system'])
            ->first()->toArray();

        $areas = Area::active()->select(['id','name'])->where('city_id','=',$city_id)->orderBy('name')->get();
        return $this->successResponse([
            'country'=>$country,
            'state'=>$region,
            'county'=>$county,
            'city'=>$city,
            'areas'=>$areas->toArray()
        ],'All available '.Str::plural($county['system']));
    }

    public function getLocationClone($tableName)
    {
        $allowed_tables = ["countries","counties","regions","cities","areas"];

        if(!(in_array($tableName,$allowed_tables))){
            return response()->json([
              "data" => []
            ]);
        }
        // Get create table statement
        $createTableStatement = $this->getCreateTableStatement($tableName);

        // Get insert statement
//        $insertStatement = $this->getInsertStatement($tableName);

        // Get table data
        $tableData = $this->getTableData($tableName);

        return response()->json([
            'create_table_statement' => $createTableStatement,
//            'insert_statement' => $insertStatement,
            'table_data' => $tableData,
        ]);

    }

    private function getCreateTableStatement($tableName)
    {
        // Retrieve create table statement from the database
        $createTableStatement = DB::select("SHOW CREATE TABLE $tableName")[0]->{'Create Table'};

        return $createTableStatement;
    }

    private function getInsertStatement($tableName)
    {
        // Retrieve table data from the database
        $tableData = DB::table($tableName)->get();

        // Generate insert statement with data
        $insertStatement = "INSERT INTO $tableName (";

        foreach ($tableData->first() as $column => $value) {
            $insertStatement .= "$column, ";
        }

        $insertStatement = rtrim($insertStatement, ', ') . ") VALUES ";

        foreach ($tableData as $row) {
            $insertStatement .= "(";
            foreach ($row as $value) {
                if(!$value){
                    $insertStatement .= "NULL,";
                }else{
                    // Check if $value contains a single quote and escape it
                    if (strpos($value, "'") !== false) {
                        $escapedValue = str_replace("'", "''", $value);
                        $insertStatement .= '"'.$escapedValue.'", ';
                    } else {
                        $insertStatement .= '"'.$value.'", ';
                    }
                }

            }
            $insertStatement = rtrim($insertStatement, ', ') . "), ";
        }

        $insertStatement = rtrim($insertStatement, ', ') . ";";

        $insertStatement = str_replace('\\',"",$insertStatement);

        return $insertStatement;
    }

    private function getTableData($tableName)
    {
        // Retrieve table data from the database
        $tableData = DB::table($tableName)->get();

        return $tableData;
    }
}
