<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Traits\HttpResponses;

use App\Models\Merchant;
use App\Models\MerchantBranch;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

/**
 * @OA\Info(
 *      version="1.0.0",
 *      title="Alleviate Api documentation",
 *      description="Api documentation",
 *      @OA\Contact(
 *          email="support@bewsys.com"
 *      ),
 *      @OA\License(
 *          name="Alleviate Consoles",
 *          url="http://bewsys.com"
 *      )
 * )
 *
 * @OA\Server(
 *      url=L5_SWAGGER_CONST_HOST,
 *      description="Alleviate API Server"
 * )
 *
 * @OA\Tag(
 *     name="1. Console",
 *     description="API Endpoints of Alleviate"
 * )
 */

class MerchantsApiController extends Controller
{

    use HttpResponses;

    /**
     * @OA\Get(
     *      path="/api/get-merchants/{merchant}",
     *      operationId="getMerchants",
     *      tags={"1a. Merchants"},
     *      summary="Get list of merchants",
     *      description="Returns list of merchants",
     *     @OA\Parameter(
     *          name="merchant",
     *          description="Array of specific merchant ids",
     *          required=false,
     *          in="path",
     *          example="1,2",
     *          @OA\Schema(
     *              type="array",
     *              @OA\Items(
     *              type="array",
     *              @OA\Items()
     *              ),
     *          ),
     *      ),
     *      @OA\Response(
     *          response=200,
     *          description="successful operation"
     *       ),
     *       @OA\Response(response=400, description="Bad request"),
     *
     *     )
     *
     * Returns list of projects
     */
    public function getMerchants($merchant_ids = null)
    {
        $merchants = Merchant::query()
            ->with('branches')
            ->where('status',1);

        if(!is_null($merchant_ids)) {

            if(is_array($merchant_ids)){
                $merchants = $merchants->whereIn('id', $merchant_ids);
            }
            else {
                $merchant_id_array = $merchant_ids ? explode(',', $merchant_ids) : null;
                $merchants = $merchants->whereIn('id', $merchant_id_array);
            }
        }

        $merchants = $merchants->get()->map(function($merchant){
            $country = $merchant->branches->first();
                return [
                    'id' => $merchant->id,
                    'name' => $merchant->name,
                    'shortname' => $merchant->shortname,
                    'swift_code' => $merchant->swift_code,
                    'logo' => $merchant->logo,
                    'type' =>  [
                            'id' => $merchant->type,
                            'name' => $merchant->_type(),
                        ],
                    'country' => [
                            'id' => $country ? $country->country->id : null,
                            'name' => $country ? $country->country->country_name : null,
                        ],
                    'branches' =>  collect($merchant->branches)->map(function ($branch) {
                        return [
                            'id' => $branch->id,
                            'name' => $branch->name,
                        ];
                    })
                ];
            })->toArray();

        return $this->successResponse($merchants,'All active merchants');
    }

    /**
     * @OA\Get(
     *      path="/api/get-merchant/{merchant_id}",
     *      operationId="getMerchantById",
     *      tags={"1a. Merchants"},
     *      summary="Get merchant details by Merchant ID",
     *      description="Returns merchant by specific merchant ID",
     *     @OA\Parameter(
     *          name="merchant_id",
     *          description="Merchant ID unique identity",
     *          required=true,
     *          in="path",
     *          @OA\Schema(
     *              type="integer",
     *          ),
     *      ),
     *      @OA\Response(
     *          response=200,
     *          description="successful operation"
     *       ),
     *       @OA\Response(response=400, description="Bad request"),
     *
     *     )
     *
     * Returns Details of merchant based on a specific ID
     */
    public function getMerchantById($merchant_id)
    {
        $all_merchant = Merchant::query()
            ->where('status',1)
            ->where('id',$merchant_id)
            ->select('name','type','shortname','swift_code','logo')
            ->get()->map(function($merchant){
            $country = $merchant->branches->first();
            return [
                'id' => $merchant->id,
                'name' => $merchant->name,
                'shortname' => $merchant->shortname,
                'swift_code' => $merchant->swift_code,
                'logo' => $merchant->logo,
                'type' =>  [
                    'id' => $merchant->type,
                    'name' => $merchant->_type(),
                ],
                'country' => [
                    'id' => $country ? $country->country->id : null,
                    'name' => $country ? $country->country->country_name : null,
                ],
                'branches' =>  collect($merchant->branches)->map(function ($branch) {
                    return [
                        'id' => $branch->id,
                        'name' => $branch->name,
                    ];
                })
            ];
        })->toArray();

        if(count($all_merchant) < 1){
            $message = 'No merchant details found on requested ID';
        }else{
            $message = 'Merchant details found.';
        }
        return $this->successResponse($all_merchant,$message);

    }

    /**
     * @OA\Get(
     *      path="/api/get-branch/{branch_id}",
     *      operationId="getBranchById",
     *      tags={"1a. Merchants"},
     *      summary="Get Branch details by Branch ID",
     *      description="Returns branch details by specific branch ID",
     *     @OA\Parameter(
     *          name="branch_id",
     *          description="Branch ID unique identity",
     *          required=true,
     *          in="path",
     *          @OA\Schema(
     *              type="integer",
     *          ),
     *      ),
     *      @OA\Response(
     *          response=200,
     *          description="successful operation"
     *       ),
     *       @OA\Response(response=400, description="Bad request"),
     *
     *     )
     *
     * Returns Branch Details based on a specific ID
     */
    public function getBranchById($branch_id)
    {
        $branch = MerchantBranch::query()
            ->where('status',1)
            ->where('id',$branch_id)
            ->select('name','routing_number','email','contact','address','website')
            ->first();

        if(empty($branch)){
            $message = 'No branch details found on requested ID';
        }else{
            $message = $branch->name.' branch details found.';
        }
        return $this->successResponse($branch,$message);

    }

    /**
     * @OA\Get(
     *      path="/api/get-all-merchant-branches",
     *      operationId="getAllActiveMerchantBranches",
     *      tags={"1a. Merchants"},
     *      summary="Get List of All Active Branches",
     *      description="Returns all active branches",
     *      @OA\Response(
     *          response=200,
     *          description="successful operation"
     *       ),
     *       @OA\Response(response=400, description="Bad request"),
     *
     *     )
     *
     * Returns List of All Active Branches
     */
    public function getAllActiveMerchantBranches()
    {
        $all_merchant_branches = MerchantBranch::query()
            ->where('status',1)
            ->select('id','name')
            ->groupBy('id','name')
            ->get()->toArray();

        return $this->successResponse($all_merchant_branches,'All active merchant branches');
    }

    /**
     * @OA\Get(
     *      path="/api/get-merchant-branches/{merchant_id}",
     *      operationId="getMerchantBranchesByMerchant",
     *      tags={"1a. Merchants"},
     *      summary="Get List of Merchant Branches by Merchant ID",
     *      description="Returns merchant branches based on a merchant ID",
     *      @OA\Parameter(
     *          name="merchant_id",
     *          description="Merchant ID unique identity",
     *          required=true,
     *          in="path",
     *          @OA\Schema(
     *              type="integer",
     *          ),
     *      ),
     *      @OA\Response(
     *          response=200,
     *          description="successful operation"
     *       ),
     *       @OA\Response(response=400, description="Bad request"),
     *
     *     )
     *
     * Returns List of Merchant Branches based on a specific merchant ID
     */
    public function getMerchantBranchesByMerchant($merchant_id)
    {
        $all_merchant_branches = MerchantBranch::query()
            ->where('merchant_id',$merchant_id)
            ->where('status',1)
            ->select('id','name')
            ->groupBy('id','name')
            ->get()->toArray();

        return $this->successResponse($all_merchant_branches,'All branches based on a specific merchant');
    }

    /**
     * @OA\Get(
     *      path="/api/get-merchant-countries",
     *      operationId="getMerchantCountries",
     *      tags={"1a. Merchants"},
     *      summary="Get List of Active Merchant Countries Only",
     *      description="Returns countries based on active merchants only",
     *      @OA\Response(
     *          response=200,
     *          description="successful operation"
     *       ),
     *       @OA\Response(response=400, description="Bad request"),
     *
     *     )
     *
     * Returns List of countries based on active merchants
     */
    public function getMerchantCountries()
    {
        $merchants_countries = MerchantBranch::query()
            ->join('countries','countries.id','=','merchant_branches.country_id')
            ->where('merchant_branches.status',1)
            ->select('countries.id as id','countries.country_name as country')
            ->groupBy('id','country')
            ->get()->toArray();
        return $this->successResponse($merchants_countries,'All active merchant countries');
    }

    /**
     * @OA\Get(
     *      path="/api/get-merchant-types",
     *      operationId="getMerchantTypes",
     *      tags={"1a. Merchants"},
         *      summary="Get List of Active Merchant Types Only",
     *      description="Returns types only based on active merchants",
     *      @OA\Response(
     *          response=200,
     *          description="successful operation"
     *       ),
     *       @OA\Response(response=400, description="Bad request"),
     *
     *     )
     *
     * Returns List of types only based on active merchants
     */
    public function getMerchantTypes()
    {
        $merchants_types = MerchantBranch::query()
            ->join('merchants','merchants.id','=','merchant_branches.merchant_id')
            ->where('merchant_branches.status',1)
            ->select('merchants.type as type')
            ->groupBy('type')
            ->get()->map(function ($query){
               return  [
                   "id" => $query->type,
                   "name" => getMerchantTypes()[$query->type]
               ];
            })->toArray();

        return $this->successResponse($merchants_types,'All active merchant types');
    }

    /**
     * @OA\Get(
     *      path="/api/get-merchant-by-type/{type}",
     *      operationId="getMerchantByType",
     *      tags={"1a. Merchants"},
     *      summary="Get Merchant Details based on the Type ID",
     *      description="Returns details of merchant based on type ID",
     *      @OA\Parameter(
     *          name="type",
     *          description="Type ID unique identity",
     *          required=true,
     *          in="path",
     *          @OA\Schema(
     *              type="integer",
     *          ),
     *      ),
     *      @OA\Response(
     *          response=200,
     *          description="successful operation"
     *       ),
     *       @OA\Response(response=400, description="Bad request"),
     *
     *     )
     *
     * Returns Details of merchants based on the type ID
     */
    public function getMerchantByType($type,$country_id=null)
    {
//        $all_types = getMerchantTypes();

        if(!(is_numeric($type))){
            $type = array_search($type,getMerchantTypes());
        }
        $message = 'All Merchants Under '.getMerchantTypes()[$type];
        $merchants_types = Merchant::query()
            ->where('type',$type)
            ->where('status',1)
            ->get()->map(function($merchant) {
                $country = $merchant->branches->first();
                return [
                        'id' => $merchant->id,
                        'name' => $merchant->name,
                        'shortname' => $merchant->shortname,
                        'swift_code' => $merchant->swift_code,
                        'logo' => $merchant->logo,
                        'type' => [
                            'id' => $merchant->type,
                            'name' => $merchant->_type(),
                        ],
                        'country' => [
                            'id' => $country ? $country->country->id : null,
                            'name' => $country ? $country->country->country_name : null,
                        ],
                        'branches' =>
                            collect($merchant->branches)
                                ->map(function ($branch) {
                                    return [
                                        'id' => $branch->id,
                                        'name' => $branch->name,
                                    ];
                                })
                    ];
            })
            ->filter(function ($merchants_types) use ($country_id) {
                if($country_id == null){
                    return true;
                }
                return $merchants_types['country']['id'] == $country_id;
            })
            ->toArray();


        return $this->successResponse($merchants_types,$message);
    }

    /**
     * @OA\Get(
     *      path="/api/get-types-by-country/{country_id}",
     *      operationId="getTypesUnderCountry",
     *      tags={"1a. Merchants"},
     *      summary="Get List of Types based on the Country ID",
     *      description="Returns list of types based on country ID",
     *      @OA\Parameter(
     *          name="country_id",
     *          description="Country ID unique identity",
     *          required=true,
     *          in="path",
     *          @OA\Schema(
     *              type="integer",
     *          ),
     *      ),
     *      @OA\Response(
     *          response=200,
     *          description="successful operation"
     *       ),
     *       @OA\Response(response=400, description="Bad request"),
     *
     *     )
     *
     * Returns List of Types based on Country ID
     */
    public function getTypesUnderCountry($country_id)
    {

        $types_under_country = MerchantBranch::query()
            ->join('countries','countries.id','=','merchant_branches.country_id')
            ->join('merchants','merchants.id','=','merchant_branches.merchant_id')
            ->where('merchant_branches.status',1)
            ->where('merchant_branches.country_id',$country_id)
            ->select('merchants.id as merchant_id','merchants.name as merchant','merchants.type as type','merchant_branches.id as branch_id','merchant_branches.name as branch')
            ->groupBy('merchant_id','merchant','type','branch_id','branch')
            ->get()->toArray();

        $dataset = [
          'type' =>  collect($types_under_country)->pluck('type')->unique()->toArray(),
          'merchant' =>  collect($types_under_country)->pluck('merchant','merchant_id')->unique()->toArray(),
          'branch' =>  collect($types_under_country)->pluck('branch','branch_id')->unique()->toArray()
        ];

        return $this->successResponse($dataset,"All types under country");
    }

    /**
     * @OA\Get(
     *      path="/api/get-types-only-country/{country_id}",
     *      operationId="getTypesOnlyUnderCountry",
     *      tags={"1a. Merchants"},
     *      summary="Get Types Only based on the Country ID",
     *      description="Returns list of types only based on country ID",
     *      @OA\Parameter(
     *          name="country_id",
     *          description="Country ID unique identity",
     *          required=true,
     *          in="path",
     *          @OA\Schema(
     *              type="integer",
     *          ),
     *      ),
     *      @OA\Response(
     *          response=200,
     *          description="successful operation"
     *       ),
     *       @OA\Response(response=400, description="Bad request"),
     *
     *     )
     *
     * Returns List of Types Only based on the country ID
     */
    public function getTypesOnlyUnderCountry($country_id)
    {

        $types_only_under_country = MerchantBranch::query()
            ->join('countries','countries.id','=','merchant_branches.country_id')
            ->join('merchants','merchants.id','=','merchant_branches.merchant_id')
            ->where('merchant_branches.status',1)
            ->where('merchant_branches.country_id',$country_id)
            ->select('merchants.type as type')
            ->groupBy('type')
            ->get()->map(function ($query){
                return  [$query->type => getMerchantTypes()[$query->type]];
            })->toArray();


        return $this->successResponse($types_only_under_country,"All types only under a specific country");
    }

    public function getMerchantClone($tableName)
    {

        $allowed_tables = ["merchants","merchant_branches"];

        if(!(in_array($tableName,$allowed_tables))){
            return response()->json([
                "data" => []
            ]);
        }

        // Get create table statement
        $createTableStatement = $this->getCreateTableStatement($tableName);

        // Get insert statement
//        $insertStatement = $this->getInsertStatement($tableName);

//        // Get table data
        $tableData = $this->getTableData($tableName);

        return response()->json([
            'create_table_statement' => $createTableStatement,
//            'insert_statement' => $insertStatement,
            'table_data' => $tableData,
        ]);
    }

    private function getCreateTableStatement($tableName)
    {
        // Retrieve create table statement from the database
        $createTableStatement = DB::select("SHOW CREATE TABLE $tableName")[0]->{'Create Table'};

        return $createTableStatement;
    }

    private function getInsertStatement($tableName)
    {
        // Retrieve table data from the database
        $tableData = DB::table($tableName)->get();

        // Generate insert statement with data
        $insertStatement = "INSERT INTO $tableName (";

        foreach ($tableData->first() as $column => $value) {
            $insertStatement .= "$column, ";
        }

        $insertStatement = rtrim($insertStatement, ', ') . ") VALUES ";

        foreach ($tableData as $row) {
            $insertStatement .= "(";
            foreach ($row as $value) {
                if(!$value){
                    $insertStatement .= "NULL,";
                }else{
                    // Check if $value contains a single quote and escape it
                    if (strpos($value, "'") !== false) {
                        $escapedValue = str_replace("'", "''", $value);
                        $insertStatement .= '"'.$escapedValue.'", ';
                    } else {
                        $insertStatement .= '"'.$value.'", ';
                    }
                }
            }
            $insertStatement = rtrim($insertStatement, ', ') . "), ";
        }

        $insertStatement = rtrim($insertStatement, ', ') . ";";

        $insertStatement = str_replace("\\","",$insertStatement);
        return $insertStatement;
    }

    private function getTableData($tableName)
    {
        // Retrieve table data from the database
        $tableData = DB::table($tableName)->get();

        return $tableData;
    }
}
