<?php

namespace App\Http\Controllers;

use App\Http\Traits\App\BaseSurvey;
use App\Http\Traits\MembershipTrait;
use App\Http\Traits\HttpResponses;
use App\Http\Traits\ModelTracker;
use App\Models\AccountDetail;
use App\Models\Applicant;
use App\Models\Application;
use App\Models\ApplicationsProcessing;
use App\Models\Bank;
use App\Models\BankBranch;
use App\Models\Benefit;
use App\Models\CaseManagement;
use App\Models\ContactDetail;
use App\Models\EmploymentDetail;
use App\Models\Entry;
use App\Models\Family;
use App\Models\HealthDetail;
use App\Models\IdentificationDetail;
use App\Models\LookupCode;
use App\Models\MaritalDetail;
use App\Models\Announcement;
use App\Models\AnnouncementsRead;
use App\Models\Survey;
use App\Models\SurveyForm;
use App\Models\User;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Artisan;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;

class ApiController extends Controller
{
    use HttpResponses,MembershipTrait,ModelTracker,BaseSurvey;

    public $successStatus = 200;
    public $noContentStatus = 204;
    public $badRequestStatus = 400;
    public $invalidAuthStatus = 401;
    public $hiddenCases = ['user_ids','role_ids'];
    public $hiddenFieldColumns = ['created_at','updated_at','workflow_actors'];
    public $hiddendates = ['created_at','updated_at'];
    public $hiddenSurveysColumns = ['created_at','updated_at','status','mode','self_respondents','team_respondents',
        'created_by','updated_by','settings','duration'];
    public $hiddenUserColumns = ['email_verified_at','deleted_at','password','remember_token','created_at','updated_at'];

    /**
     * Checks if the user is using a mobile or email username to login
     * @return string
     */
    public function username() :string
    {
        if(is_numeric(request()->phone)) {
            return 'phone';
        }
        else {
            return 'email';
        }
    }

    /**
     * Check if the logged in account has been verified
     * @return JsonResponse
     */
    public function verifyStatus() : JsonResponse {
        if(is_null(auth()->user()->account_verified_at)) {
            return $this->successResponse(['verified' => false],'Account is not verified.');
        }
        else {
            return $this->successResponse(['verified' => true],'Account is verified.');
        }
    }

    /**
     * login api
     * @param Request $request
     * @return JsonResponse
     */
    public function login(Request $request):JsonResponse{
        $user = User::where($this->username(),'=',$request->phone)->first();

        $tokenName = "APP_TOKEN";
        if(isset($request->device_name)) {
            $tokenName = $request->device_name;
        }

        if (isset($user)) {

            if (Hash::check($request->password, $user->password)) {
                $login_credentials = [
                    $this->username() => $request->phone,
                    'password' => $request->password,
                ];
                if (auth()->attempt($login_credentials)) {
                    $user->makeHidden($this->hiddenUserColumns);
                    $user->profile;

                    $data = $user->toArray();
                    $data = Arr::prepend($data,'Bearer','token_type');
                    $data = Arr::prepend($data,$user->createToken($tokenName)->plainTextToken,'token');

                    $verified_note = null;
                    if(is_null(auth()->user()->account_verified_at)) $verified_note = "Your account has not been activated.
                    Please enter the 4-letter code sent to phone after registration.";

                    return $this->successResponse($data,'You are logged in. '.$verified_note);
                }
                else {
                    return $this->errorResponse(null,'Sorry, Account cannot be verified. Try again','You are logged in.', $this->successStatus);
                }

            } else {
                return $this->errorResponse(null,'Incorrect credentials. Try again!', $this->successStatus);
            }
        }
        else {
            return $this->errorResponse(null,'Sorry! Invalid Account.', $this->successStatus);
        }
    }

    /**
     * Check User login api
     *
     * @return JsonResponse
     */
    public function userLoggedIn():JsonResponse{
        if (Auth::guard('api')->check()) {
            return $this->successResponse(['logged_in'=>true],'Logged in');
        } else {
            return $this->errorResponse(['logged_in'=>false],'Logged out',401);
        }
    }

    /**
     * Register api
     *
     * @param Request $request
     * @return JsonResponse
     */
    public function register(Request $request):JsonResponse
    {
        $user = User::where('phone','=',$request->phone)->first();
        $email_user = User::where('email','=',$request->email)->first();

        if ($request->name == null or $request->name == '') {
            return $this->errorResponse(null,'Last name is required!', 401);
        }
        if ($request->firstname == null or $request->firstname == '') {
            return $this->errorResponse(null,'First name is required!', 401);
        }
        if ($request->phone == null or $request->phone == '') {
            return $this->errorResponse(null,'Phone Number is required!', 401);
        }
        if ($user) {
            return $this->errorResponse(null,'Phone Number is already being used!', 401);
        }
        if ($email_user) {
            return $this->errorResponse(null,'Email already exists!', 401);
        }
        if ($request->password == null or $request->password == '' or $request->c_password == null or $request->c_password == '') {
            return $this->errorResponse(null,'Password is required!', 401);
        }
        if ($request->password != $request->c_password) {
            return $this->errorResponse(null,'Passwords do not match!', 401);
        }

        $code = strtoupper(Str::random(4));
        $input['name'] = $request->name;
        $input['firstname'] = $request->firstname;
        $input['phone'] = $request->phone;
        $input['email'] = $request->email;
        $input['password'] = Hash::make($request->password);
        $input['api_token'] = Str::random(60);
        $input['is_admin'] = false;
        $input['verification_code'] = $code;

        $user = User::create($input);
        $message = 'Something might have gone wrong.';
        if ($user) {
            $user->roles()->sync(env('DEFAULT_ROLE'));
            $message = 'Your mobile number has been recorded. A 4-letter code has been sent to the number via sms for verification.';
            $sms = setAppDefault('app_project_name').": Your account has been created. Use the verification code - $code to activate your account. Thank you.";
            sendSMS($request->phone,$sms);
        }

        $tokenName = "APP_TOKEN";
        if(isset($request->device_name)) {
            $tokenName = $request->device_name;
        }

        $user->makeHidden($this->hiddenUserColumns);
        $user->profile;

        $data = $user->toArray();
        $data = Arr::prepend($data,'Bearer','token_type');
        $data = Arr::prepend($data,$user->createToken($tokenName)->plainTextToken,'token');
        return $this->successResponse($data,$message);
    }

    /**
     * Verify Verification Code
     * @param Request $request
     * @return JsonResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function verifyCode(Request $request):JsonResponse
    {
        $this->validate($request, [
            'verification_code' => 'required|string|min:4|max:4',
        ]);

        if (Str::upper(auth()->user()->verification_code) == Str::upper($request->verification_code)) {
            auth()->user()->verification_code = null;
            auth()->user()->account_verified_at = now();
            auth()->user()->status = 1;
            auth()->user()->save();
            $message = 'Your account has been verified.';
            return $this->successResponse(null,$message);
        }
        else {
            $message = "Sorry! Invalid Code.";
            return $this->errorResponse('Error',$message,401);
        }

    }

    /**
     * Resend Verification Code
     * @return JsonResponse
     */
    public function ReSendVerifyCode():JsonResponse
    {
        $code = strtoupper(Str::random(4));
        if (auth()->user()->account_verified_at == null) {
            auth()->user()->verification_code = null;
            auth()->user()->save();
            $sms = setAppDefault('app_project_name').": Use this verification code - $code to activate your account. Thank you.";
            sendSMS(auth()->user()->phone,$sms);
            $message = 'A new verification code has been sent to your phone number.';
            return $this->successResponse(null,$message);
        }
        else {
            $message = "Your account as already been verified.";
            return $this->errorResponse('Error',$message,401);
        }

    }

    /**
     * Change Password
     * @param Request $request
     * @return JsonResponse
     */
    public function changePassword(Request $request):JsonResponse
    {
        $validator = Validator::make($request->all(), [
            'password' => 'required',
            'confirm_password' => 'required|same:password',
        ]);

        if ($validator->fails()) {
            return $this->errorResponse($validator->errors(),'Password does not match.',401);
        }

        auth()->user()->password = Hash::make($request->password);
        auth()->user()->remember_token = Str::random(60);

        if (auth()->user()->save()) {
            return $this->successResponse(null,'Password has been changed successfully.');
        }
        else {
            return $this->successResponse(null,'Password failed to change. Please try again.', $this->badRequestStatus);
        }
    }

    /**
     * Reset Password
     * @param Request $request
     * @return JsonResponse
     */
    public function resetPassword(Request $request):JsonResponse
    {
        $validator = Validator::make($request->all(), [
            'phone' => 'required',
            'password' => 'required',
            'confirm_password' => 'required|same:password',
        ]);
        if ($validator->fails()) {
            return $this->errorResponse($validator->errors(),'Password may not match',$this->invalidAuthStatus);
        }

        $user = User::where('phone','=',$request->phone)->first();

        if (!isset($user)) {
            return $this->errorResponse(null,'Invalid User account',$this->badRequestStatus);
        }

        $user->password = Hash::make($request->password);
        $user->remember_token = Str::random(60);


        if ($user->save()) {
            $message = 'Password has been reset successfully.';
            return $this->successResponse(null,$message);
        }
        else {
            $message = 'Password failed to reset. Please try again.';
            return $this->errorResponse(null,$message,$this->badRequestStatus);
        }
    }

    /**
     * @return JsonResponse
     */
    public function logout() :JsonResponse
    {
        Auth::user()->tokens->each(function($token, $key) {
            $token->delete();
        });

        return response()->json('Successfully logged out');
    }

    /**
     * Get User Account Profile
     *
     * @return JsonResponse
     */
    public function profile():JsonResponse
    {
        $user = User::with(['applicant'])->where('id',auth()->id())->first();
        if ($user) {
            $user->makeHidden($this->hiddenUserColumns);
            $user->profile;
            return $this->successResponse($user->toArray(),'My Profile');
        }
        else {
            return $this->errorResponse(null,'Invalid Account!',$this->badRequestStatus);
        }
    }

    public function getApplicantCompletionLevel()
    {
        $this->checkApplicantStatus();
        $completion_level = applicantCompletion(auth()->user()->applicant_number);

        return $this->successResponse($completion_level,'Success');
    }

    public function getBenefit($benefit_id)
    {
        $this->checkApplicantStatus();
        $benefits = Benefit::select(config('settings.benefit_json_fields'))->where('id','=',$benefit_id)->first();

        return $this->successResponse($benefits,'Success');
    }

    public function getApplicantEligibleBenefits()
    {
        $this->checkApplicantStatus();
        $eligibles = applicant_eligible_benefits(auth()->user()->applicant_number);
        $enrolled = Application::where('applicant_number', auth()->user()->applicant_number)->pluck('benefit_id')->toArray();
//        $benefits = Benefit::whereIn('id',array_diff($eligibles, $enrolled))->pluck('titles','id')->toArray();
        $benefits = Benefit::select(config('settings.benefit_json_fields'))->whereIn('id',array_diff($eligibles, $enrolled))->get();

        return $this->successResponse($benefits,'Success');
    }

    public function getApplicantEnrolledBenefits()
    {
        $this->checkApplicantStatus();
        $enrolled = Application::where('applicant_number', auth()->user()->applicant_number)->pluck('benefit_id')->toArray();
        $benefits = Benefit::select(config('settings.benefit_json_fields'))->whereIn('id',$enrolled)->get();

        return $this->successResponse($benefits,'Success');
    }

    /**
     * New Enrolment
     * @param Request $request
     * @return JsonResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function saveCustomerRegistration(Request $request):JsonResponse
    {
        $this->validate($request,[
            'surnames' => 'required|string',
            'first_names' => 'required|string',
            'other_names' => 'nullable|string',
            'dobs' => 'required|string',
            'genders' => 'required|string',
            'place_of_births' => 'required|string',
            'country_of_births' => 'required|string',
            'nationalities' => 'required|string',
        ]);

        if (!is_null(auth()->user()->applicant_number)) {
            return $this->errorResponse(null,'Sorry! This account has an existing attached '
                .settings()->get('applicant_title').'. Detach this '.settings()->get('applicant_title').' account to allow another account.',
                $this->badRequestStatus);
        }
        else {
            if (!isset($request->id)){
                if ($regID = $this->generateRefID($request)) {
                    $applicant = new Applicant();
                    $applicant->regID = $regID;
                    $applicant->added_by = Auth::id();
                }else {
                    return $this->errorResponse(null,'Something went wrong. Please try again',$this->badRequestStatus);
                }
            }
            else {
                $applicant = Applicant::find($request->id);
            }

            $passport_pic_path = null;
            if($request->file('upload_passport_pic')) {
                $file = $this->uploadProfile('upload_passport_pic');
                $passport_pic_path = 'app-uploads/'.$file;
            }

            $applicant->surnames = $request->surnames;
            $applicant->first_names = $request->first_names;
            $applicant->other_names = $request->other_names;
            $applicant->dobs = date('Y-m-d', strtotime($request->dobs));
            $applicant->genders = $request->genders;
            $applicant->country_of_births = $request->country_of_births;
            $applicant->place_of_births = $request->place_of_births;
            $applicant->nationalities = $request->nationalities;
            $applicant->upload_passport_pic = $passport_pic_path;
            $applicant->avatar = $passport_pic_path;
            $applicant->updated_by = Auth::id();

            if ($applicant->save()){
                addApplicantDocument($applicant->id,$passport_pic_path,'Profile Image');
                auth()->user()->applicant_number = $regID;
                auth()->user()->save();
                $message = settings()->get('applicant_title').' has been added successfully';
                return $this->successResponse(null,$message);
            }
            else {
                $message = 'Something went wrong. The '.strtolower(settings()->get('applicant_title')).' was not saved. Please try again';
                return $this->errorResponse(null,$message,$this->badRequestStatus);
            }
        }
    }

    /**
     * Update Applicant Record
     * @param Request $request
     * @return JsonResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function updateCustomerRecord(Request $request):JsonResponse
    {
        $this->checkApplicantStatus();
        if (isset($request->request_delete)){
            if ($request->request_delete != "true"){
                if (ContactDetail::where('mobile','=',$request->mobile)->count() > 0) {
                    $message = 'Sorry! This mobile number is already in use.';
                    return $this->errorResponse(null,$message,$this->badRequestStatus);
                }

                if (ContactDetail::where('email','=',$request->email)->count() > 0) {
                    $message = 'Sorry! This email is already in use.';
                    return $this->errorResponse(null,$message,$this->badRequestStatus);
                }
            }
        }

        $applicant_id = auth()->user()->applicant->id;

        $applicant = Applicant::find($applicant_id);
        $check_applicant_number = $applicant->regID;

        $extra = $doc = $record_id = null;
        $model = config('tracker.model','RecordTracker');

        if ($request->form_type == 'personal') {

            $this->validate($request,[
                'surnames' => 'required|string',
                'first_names' => 'required|string',
                'other_names' => 'nullable|string',
                'dobs' => 'required|string',
                'genders' => 'required|string',
                'place_of_births' => 'required|string',
                'country_of_births' => 'required|string',
                'nationalities' => 'required|string',
                'reason' => 'required|string',
            ]);

            if($request->file('upload_passport_pic')) {
                $file = $this->uploadProfile('upload_passport_pic');
                $extra['upload_passport_pic'] = 'app-uploads/'.$file;
                $extra['avatar'] = 'app-uploads/'.$file;
            }

            if ($record_references = $this->track('applicants',$applicant_id,$request->all(),$extra)){
                if (!is_null($extra)) addApplicantDocument($applicant_id,$extra['upload_passport_pic'],'Profile Image');

                $this->recordReferenceEvidence($request,$model,$record_references,$applicant);

                $message = 'The Request for Amendment has been received successfully and is pending approval.';
                return $this->successResponse(null,$message);
            }
            else {
                $message = 'Something went wrong. The applicant was not saved. Please try again';
                return $this->errorResponse(null,$message,$this->badRequestStatus);
            }
        }
        elseif ($request->form_type == 'family') {
            $this->validate($request,[
                'form_type' => 'required|string',
                'surname' => 'required|string',
                'first_name' => 'required|string',
                'other_name' => 'nullable|string',
                'date_of_birth' => 'required|string',
                'dependent_reference_id' => 'required|string',
                'id_type' => 'required|string',
                'is_dependant' => 'required|string'
            ]);

            if (isset($request->record_id)) {
                $record_id = $request->record_id;
                $record = Family::find($record_id);

                if ($check_applicant_number != $record->applicant_number) {
                    $message = 'This record does not exist. Please try again';
                    return $this->errorResponse(null,$message,$this->badRequestStatus);
                }
            }
            else {
                $record = new Family;
                $record->applicant_number = $applicant->regID;
            }
            $record_references = $this->track($request->form_type,$record_id,$request->all(),$extra);

            $record->type = $request->type;
            $record->surname = $request->surname;
            $record->first_name = $request->first_name;
            $record->other_name = $request->other_name;
            $record->date_of_birth = date('Y-m-d', strtotime($request->date_of_birth));
            $record->id_type = $request->id_type;
            $record->dependent_reference_id = $request->dependent_reference_id;
            $record->is_dependent = $request->is_dependant;
            $record->removed = 0;
            $record->status = "New";
            $record->addedBy = auth()->id();

            if ($record->save()){

                $this->recordReferenceEvidence($request,$model,$record_references,$record,$applicant);

                $message = 'The Request for Amendment has been received successfully and is pending approval.';
                return $this->successResponse($record,$message);
            }
            else {
                $message = 'Something went wrong. The record was not saved. Please try again';
                return $this->errorResponse(null,$message,$this->badRequestStatus);
            }
        }
        elseif ($request->form_type == 'contact_details') {

            if (isset($request->record_id)) {
                $record_id = $request->record_id;
                $record = ContactDetail::find($record_id);

                if ($check_applicant_number != $record->applicant_number) {
                    $message = 'This record does not exist. Please try again';
                    return $this->errorResponse(null,$message,$this->badRequestStatus);
                }
            }
            else {
                $record = new ContactDetail();
                $record->applicant_number = $applicant->regID;
            }

            $res_document_path = null;
            if($request->file('upload_residency')) {
                $directory = 'public/applicants/'.$applicant->id.'/proof-residence';
                Storage::putFileAs("$directory", $request->file('upload_residency'), $request->file('upload_residency')->getClientOriginalName());
                $res_document_path = $directory.'/'.$request->file('upload_residency')->getClientOriginalName();
                $record->upload_residency = $res_document_path;
                $extra['upload_residency'] = $res_document_path;
            }

            $record_references = $this->track($request->form_type,$record_id,$request->all(),$extra);

            $record->type = $request->type;
            $record->type_name = getLookupSelectedOptionTextByValue($request->type);
            $message = 'The Request for Amendment has been received successfully and is pending approval.';
            if (isset($request->mobile)) {

                $message = 'Your mobile number has been recorded. A 4-letter code has been sent to the number via sms for verification.';
                $code = verificationCode();
                $record->mobile = $request->mobile;
                $record->code = $code;
            }
            if (isset($request->village)) $record->village = $request->village;
            if (isset($request->telephone)) $record->telephone = $request->telephone;
            if (isset($request->email)) {
                $message = 'Your email has been recorded. A 4-letter code has been sent to the email for verification.';
                $code = verificationCode();
                $record->email = $request->email;
                $record->code = $code;
            }
            if (isset($request->address)) $record->address = $request->address;
            if (isset($request->postal_address)) $record->postal_address = $request->postal_address;
            if (isset($request->island)) $record->island = $request->island;
            $record->status = 'New';
            $record->addedBy = Auth::id();

            if ($record->save()){
                if (isset($request->mobile) && $request->request_delete != "true") {
                    $sms = setAppDefault('app_project_name').": Your verification code is ".$record->code.".";
                    sendSMS($request->mobile,$sms);
                }
                $this->recordReferenceEvidence($request,$model,$record_references,$record,$applicant);
                addApplicantDocument($applicant->id,$res_document_path,'Proof of Residence');
                return $this->successResponse(null,$message);
            }
            else {
                $message = 'Something went wrong. The record was not saved. Please try again';
                return $this->errorResponse(null,$message,$this->badRequestStatus);
            }
        }
        elseif ($request->form_type == 'identification_details') {
            $this->validate($request,[
                'form_type' => 'required|string',
                'id_types' => 'required|string',
                'id_number' => 'required|string',
                'expiry' => 'nullable|string',
                'document' => 'nullable|file'
            ]);

            if (isset($request->record_id)) {
                $record_id = $request->record_id;
                $record = IdentificationDetail::find($record_id);

                if (!$record) {
                    $message = 'Invalid record selected';
                    return $this->errorResponse(null,$message,$this->badRequestStatus);
                }

                if ($check_applicant_number != $record->applicant_number) {
                    $message = 'This record does not exist. Please try again';
                    return $this->errorResponse(null,$message,$this->badRequestStatus);
                }
            }
            else {
                $record = new IdentificationDetail;
                $record->applicant_number = $applicant->regID;
            }

            $res_document_path = null;
            if($request->file('document')) {
                $directory = 'public/applicants/'.$applicant->id.'/identifications';
                Storage::putFileAs("$directory", $request->file('document'), $request->file('document')->getClientOriginalName());
                $res_document_path = $directory.'/'.$request->file('document')->getClientOriginalName();
                $record->document = $res_document_path;
                $extra['document'] = $directory.'/'.$request->file('document')->getClientOriginalName();
            }

            $record_references = $this->track($request->form_type,$record_id,$request->all(),$extra);

            $record->id_type = $request->id_types;
            $record->id_number = $request->id_number;
            if (!is_null($request->expiry)) {
                $record->expiry = date('Y-m-d', strtotime($request->expiry));
            }
            else {
                $record->expiry = null;
            }
            $record->removed = 0;
            $record->status = "New";
            $record->addedBy = Auth::id();

            if ($record->save()){
                $this->recordReferenceEvidence($request,$model,$record_references,$record,$applicant);
                addApplicantDocument($applicant->id,$res_document_path,$record->id_type);
                $message = 'The Request for Amendment has been received successfully and is pending approval.';
                return $this->successResponse($record,$message);
            }
            else {
                $message = 'Something went wrong. The record was not saved. Please try again';
                return $this->errorResponse(null,$message,$this->badRequestStatus);
            }
        }
        elseif ($request->form_type == 'marital_details') {
            $this->validate($request,[
                'form_type' => 'required|string',
                'marriage_status' => 'required|string',
                'marriage_type' => 'required|string',
                'spouse_name' => 'required|string',
                'spouse_ref_id' => 'nullable|string',
                'marriage_start_date' => 'nullable|string',
                'marriage_end_date' => 'nullable|string'
            ]);

            if (isset($request->record_id)) {
                $record_id = $request->record_id;
                $record = MaritalDetail::find($record_id);

                if ($check_applicant_number != $record->applicant_number) {
                    $message = 'This record does not exist. Please try again';
                    return $this->errorResponse(null,$message,$this->badRequestStatus);
                }
            }
            else {
                $record = new MaritalDetail;
                $record->applicant_number = $applicant->regID;
            }
            $record_references = $this->track($request->form_type,$record_id,$request->all(),$extra);

            $record->marriage_status = $request->marriage_status;
            $record->marriage_type = $request->marriage_type;
            $record->spouse_name = $request->spouse_name;
            $record->spouse_ref_id = $request->spouse_ref_id;
            $record->marriage_start_date = (!is_null($request->marriage_start_date))?date('Y-m-d', strtotime($request->marriage_start_date)):null;
            $record->marriage_end_date = (!is_null($request->marriage_end_date))?date('Y-m-d', strtotime($request->marriage_end_date)):null;
            $record->status = 'New';
            $record->addedBy = Auth::id();

            if ($record->save()){

                $this->recordReferenceEvidence($request,$model,$record_references,$record,$applicant);

                $message = 'The Request for Amendment has been received successfully and is pending approval.';
                return $this->successResponse($record,$message);
            }
            else {
                $message = 'Something went wrong. The record was not saved. Please try again';
                return $this->errorResponse(null,$message,$this->badRequestStatus);
            }
        }
        elseif ($request->form_type == 'employment_details') {
            $this->validate($request,[
                'form_type' => 'required|string',
                'employment_type' => 'required|string',
                'employment_sector' => 'required|string',
                'employer_name' => 'required|string',
                'employer_telephone' => 'nullable|string',
                'employer_mobile' => 'nullable|string',
                'employer_email' => 'nullable|string',
                'employer_rmd' => 'nullable|string',
                'employer_bank_account_name' => 'nullable|string',
                'employer_bank_account_number' => 'nullable|string',
                'employer_postal_address' => 'nullable|string',
                'employer_island' => 'nullable|string',
                'employee_position' => 'nullable|string',
                'employee_manager' => 'nullable|string',
                'employment_start_date' => 'nullable|string',
                'employment_end_date' => 'nullable|string',
            ]);

            if (isset($request->record_id)) {
                $record_id = $request->record_id;
                $record = EmploymentDetail::find($record_id);

                if ($check_applicant_number != $record->applicant_number) {
                    $message = 'This record does not exist. Please try again';
                    return $this->errorResponse(null,$message,$this->badRequestStatus);
                }
            }
            else {
                $record = new EmploymentDetail;
                $record->applicant_number = $applicant->regID;
            }
            $record_references = $this->track($request->form_type,$record_id,$request->all(),$extra);

            $record->employment_type = $request->employment_type;
            $record->employment_sector = $request->employment_sector;
            $record->employer_name = $request->employer_name;
            $record->employer_telephone = $request->employer_telephone;
            $record->employer_mobile = $request->employer_mobile;
            $record->employer_email = $request->employer_email;
            $record->employer_rmd = $request->employer_rmd;
            $record->employer_bank_account_name = $request->employer_bank_account_name;
            $record->employer_bank_account_number = $request->employer_bank_account_number;
            $record->employer_postal_address = $request->employer_postal_address;
            $record->employer_island = $request->employer_island;
            $record->employee_position = $request->employee_position;
            $record->employee_manager = $request->employee_manager;
            $record->employment_start_date = (isset($request->employment_start_date))?date('Y-m-d', strtotime($request->employment_start_date)):null;
            $record->employment_end_date = (isset($request->employment_end_date))?date('Y-m-d', strtotime($request->employment_end_date)):null;
            $record->removed = 0;
            $record->status = "New";
            $record->addedBy = Auth::id();

            if ($record->save()){
                $this->recordReferenceEvidence($request,$model,$record_references,$record,$applicant);

                $message = 'The Request for Amendment has been received successfully and is pending approval.';
                return $this->successResponse($record,$message);
            }
            else {
                $message = 'Something went wrong. The record was not saved. Please try again';
                return $this->errorResponse(null,$message,$this->badRequestStatus);
            }
        }
        elseif ($request->form_type == 'account_details') {
            $this->validate($request,[
                'form_type' => 'required|string',
                'bank_name' => 'required|integer',
                'account_type' => 'required|string',
                'account_holder_name' => 'required|string',
                'account_number' => 'required|string',
                'branch' => 'nullable|string',
                'upload_account' => 'nullable|file'
            ]);

            if (isset($request->record_id)) {
                $record_id = $request->record_id;
                $record = AccountDetail::find($record_id);

                if ($check_applicant_number != $record->regID) {
                    $message = 'This record does not exist. Please try again';
                    return $this->errorResponse(null,$message,$this->badRequestStatus);
                }
            }
            else {
                $record = new AccountDetail();
                $record->regID = $applicant->regID;
            }

            $res_document_path = null;
            if($request->file('upload_account')) {
                $directory = 'public/applicants/'.$applicant->id.'/bank';
                Storage::putFileAs("$directory", $request->file('upload_account'), $request->file('upload_account')->getClientOriginalName());
                $res_document_path = $directory.'/'.$request->file('upload_account')->getClientOriginalName();
                $record->upload_account = $res_document_path;
                $extra['upload_account'] = $res_document_path;
            }

            $bankName = Bank::where('id','=',$request->bank_name)->pluck('name')->first();
            $branchName = BankBranch::where('id','=',$request->branch)->pluck('name')->first();
            $extra['bank_name'] = $bankName;
            $extra['branch'] = $branchName;
            $record_references = $this->track($request->form_type,$record_id,$request->all(),$extra);

            $record->bank_name = $bankName;
            $record->bank_id = $request->bank_name;
            $record->account_holder_name = $request->account_holder_name;
            $record->bank_branch = $branchName;
            $record->branch_id = $request->branch;
            $record->account_type = $request->account_type;
            $record->account_number = $request->account_number;
            $record->removed = 0;
            $record->status = 'New';
            $record->addedBy = Auth::id();

            if ($record->save()){
                $this->recordReferenceEvidence($request,$model,$record_references,$record,$applicant);
                if($request->file('upload_account')) {
                    addApplicantDocument($applicant->id, $res_document_path, 'Proof of Account');
                }
                $message = 'The Request for Amendment has been received successfully and is pending approval.';
                return $this->successResponse($record,$message);
            }
            else {
                $message = 'Something went wrong. The record was not saved. Please try again';
                return $this->errorResponse(null,$message,$this->badRequestStatus);
            }
        }
        elseif ($request->form_type == 'health_details') {
            $this->validate($request,[
                'form_type' => 'required|string',
                'health_type' => 'required|string',
                'date_started' => 'nullable|string',
                'confirmed_by' => 'nullable|string',
                'supporting_document' => 'nullable|file'
            ]);

            if (isset($request->record_id)) {
                $record_id = $request->record_id;
                $record = HealthDetail::find($record_id);

                if ($check_applicant_number != $record->regID) {
                    $message = 'This record does not exist. Please try again';
                    return $this->errorResponse(null,$message,$this->badRequestStatus);
                }
            }
            else {
                $record = new HealthDetail();
                $record->regID = $applicant->regID;
            }

            $record_references = $this->track($request->form_type,$record_id,$request->all(),$extra);

            $res_document_path = null;
            if($request->file('supporting_document')) {
                $directory = 'public/applicants/'.$applicant->id.'/health';
                Storage::putFileAs("$directory", $request->file('supporting_document'), $request->file('supporting_document')->getClientOriginalName());
                $res_document_path = $directory.'/'.$request->file('supporting_document')->getClientOriginalName();
                $record->supporting_document = $res_document_path;
                $extra['supporting_document'] = $directory.'/'.$request->file('supporting_document')->getClientOriginalName();
            }

            $record->health_type = $request->health_type;
            $record->date_started = date('Y-m-d', strtotime($request->date_started));
            $record->confirmed_by = $request->confirmed_by;
            $record->removed = 0;
            $record->status = 'New';
            $record->addedBy = Auth::id();

            if ($record->save()){

                $this->recordReferenceEvidence($request,$model,$record_references,$record,$applicant);

                addApplicantDocument($applicant->id,$res_document_path,'Proof of Health State');
                $message = 'The Request for Amendment has been received successfully and is pending approval.';
                return $this->successResponse($record,$message);
            }
            else {
                $message = 'Something went wrong. The record was not saved. Please try again';
                return $this->errorResponse(null,$message,$this->badRequestStatus);
            }
        }
    }

    public function recordReferenceEvidence(Request $request,$model,$record_references,$applicant,$record=null)
    {
        $doc = null;
        if(isset($request->evidences)) {
            foreach($request->evidences as $evidence) {
                $directory = 'public/member-records';
                Storage::putFileAs("$directory", $evidence, $evidence->getClientOriginalName());
                $doc[] = $directory.'/'.$evidence->getClientOriginalName();
            }
        }

        if ($request->form_type == 'personal') {
            $model::whereIn('id',$record_references)->update([
                'reason'    =>  $request->reason,
                'evidences'    =>  $doc
            ]);
        }
        else {
            $model::whereIn('id',$record_references)->update([
                'record_id'    =>  $record->id,
                'applicant_number'    =>  $applicant->regID,
                'reason'    =>  $request->reason,
                'evidences'    =>  $doc
            ]);
        }
    }

    /**
     * details api
     *
     * @return \Illuminate\Http\Response
     */
    public function applicantDetails()
    {
        $this->checkApplicantStatus();
        $user = User::with(['applicant'=>[
            'idDetails',
            'contactDetails',
            'maritalDetails',
            'employmentDetails',
            'accountDetails',
            'familyDetails',
            'healthDetails'
        ]])->where('id',auth()->id())->first();
        if ($user) {
            $user->makeHidden($this->hiddenUserColumns);
            $user->profile;
            return $this->successResponse($user->toArray(),'Applcant details');
        }
        else {
            return $this->errorResponse(null,'Invalid Account!',$this->badRequestStatus);
        }
    }

    /**
     * Applications api
     *
     * @return \Illuminate\Http\Response
     */
    public function applications()
    {
        $this->checkApplicantStatus();
        $user = User::with(['applicant','applications'])->where('id',auth()->id())->first();
        if ($user) {
            $user->makeHidden($this->hiddenUserColumns);
            $user->profile;
            return $this->successResponse($user->toArray(),'My applcations');
        }
        else {
            return $this->errorResponse(null,'Invalid Account!',$this->badRequestStatus);
        }
    }

    public function connectToApplication(Request $request)
    {
        $request->validate([
            'applicant_number' => 'required|string'
        ]);

        $applicant = Applicant::where('regID',$request->applicant_number)->first();

        if ($applicant) {
            if ($applicant->assigned == 1) {
                $message = 'Sorry! the beneficiary account has already been assigned. Please contact support for assistance.';
                return $this->errorResponse(null,$message,$this->successStatus);
            }

            auth()->user()->pending_application = $request->applicant_number;
            auth()->user()->save();

            $code = verificationCode();

            $applicant->assigned_code = $code;
            $applicant->save();
            $sms = setAppDefault('app_project_name').": Use the verification code - $code to reset your password. Thank you.";
            sendSMS(auth()->user()->phone,$sms);

            $message = 'Successful! Please enter the 4-digit code sent to your phone to verify.';
            return $this->successResponse(null,$message,$this->successStatus);
        }
        else {
            $message = 'Sorry! the beneficiary account does not exist.';
            return $this->errorResponse(null,$message,$this->badRequestStatus);
        }
    }

    public function verifyApplication(Request $request)
    {

        if (isset($request->submit)) {
            $request->validate([
                'code' => 'required|string|min:4|max:4'
            ]);
            $applicant = Applicant::where('regID',auth()->user()->pending_application)->where('assigned_code',$request->code)->first();

            if ($applicant) {
                $applicant->assigned_code = null;
                $applicant->assigned = 1;
                $applicant->save();

                auth()->user()->pending_application = null;
                auth()->user()->applicant_number = $applicant->regID;

                if(auth()->user()->save()) {
                    User::where('pending_application',$applicant->regID)->where('id','<>',auth()->id())->update([
                        'pending_application' => NULL
                    ]);

                $message = 'Account has been attached successfully.';
                return $this->successResponse(null,$message);
                }
            }
            else {
                $message = 'The code is invalid.';
                return $this->errorResponse(null,$message,$this->successStatus);
            }
        }

        if (isset($request->detach)) {
            auth()->user()->pending_application = null;
            auth()->user()->save();
            $message = 'The Beneficiary Account has been detached.';
            return $this->successResponse(null,$message);
        }
    }

    public function accountInfoUpdate(Request $request)
    {
        $user = User::where('phone','=',$request->phone)->where('id','!=',auth()->id())->first();
        $email_user = User::where('email','=',$request->email)->where('id','!=',auth()->id())->first();

        if ($request->name == null or $request->name == '') {
            return $this->errorResponse(null,'Last name is required!', 401);
        }
        if ($request->firstname == null or $request->firstname == '') {
            return $this->errorResponse(null,'First name is required!', 401);
        }
        if ($request->phone == null or $request->phone == '') {
            return $this->errorResponse(null,'Phone Number is required!', 401);
        }
        if ($user) {
            return $this->errorResponse(null,'Phone Number is already being used!', 401);
        }
        if ($email_user) {
            return $this->errorResponse(null,'Email already exists!', 401);
        }

        auth()->user()->name = $request->name;
        auth()->user()->firstname = $request->firstname;
        auth()->user()->othername = $request->othername;
        auth()->user()->phone = $request->phone;
        auth()->user()->email = $request->email;

        $message = 'Something might have gone wrong.';
        if (auth()->user()->save()) {
            $message = 'Your information has been updated successfully';

            auth()->user()->makeHidden($this->hiddenUserColumns);
            auth()->user()->profile;

            $data = auth()->user()->toArray();
            return $this->successResponse($data,$message);
        }
        else {
            return $this->errorResponse(null,$message, 401);
        }
    }

    /**
     * Submit benefit application form
     * @param Request $request
     * @param $benefit_id
     * @return JsonResponse
     */
    public function saveApplication(Request $request,$benefit_id)
    {
        $this->checkApplicantStatus();
        $already_exist = Application::where('applicant_number','=',$request->applicant_number)
            ->where('benefit_id','=',$benefit_id)
            ->whereNotIn('status',['Rejected','Declined'])->first();
        if ($already_exist) {
            return $this->errorResponse(null,'This account is already enrolled on this benefit.', 401);
        }

        $benefit = Benefit::find($benefit_id);
        if ($request->comfirmation != 1) {
            return $this->errorResponse(null,'Sorry! You must agree and confirm the terms of the application to proceed.', 401);
        }

        if (is_null($benefit)) {
            return $this->errorResponse(null,'Benefit does not exist.', 401);
        }

        $docs = $request->documentations_existing;

        $applicant = Applicant::where('regID','=',$request->applicant_number)->first();
        if (is_null($applicant)) {
            return $this->errorResponse(null,'Sorry! Applicant does not exist.', 401);
        }

        $account = DB::table('account_details')
            ->where('regID','=',$request->applicant_number)
            ->where('id','=',$request->bank_account_id)
            ->first();
        if (!$account) {
            return $this->errorResponse(null,'Bank Account details does not exist.', 401);
        }

        try{
            if (!is_null($request->documentations_new)) {
                foreach($request->documentations_new as $key=>$document){
                    $doc = Str::snake($document);
                    $doc_name = "upload_$doc";
                    if($request->file($doc_name)) {
                        $directory = 'public/applicants/'.$applicant->regID;
                        Storage::putFileAs("$directory", $request->file($doc_name), $request->file($doc_name)->getClientOriginalName());
                        $file_path = $directory.'/'.$request->file($doc_name)->getClientOriginalName();

                        $docs[] = $document;

                        addApplicantDocument($applicant->id,$file_path,$document);
                    }
                }
            }

            $app = Application::create([
                'form_id' => $benefit->form_id,
                'benefit_id' => $benefit->id,
                'applicant_number' => $request->applicant_number,
                'applicationable_id' => Auth::id(),
                'applicationable_type' => 'App/Models/User',
                'modified_by' => Auth::id(),
                'type' => $benefit->titles,
                'bank_account_id' => $request->bank_account_id,
                'reporter_surname' => $request->reporter_surname,
                'reporter_first_name' => $request->reporter_first_name,
                'reporter_contact' => $request->reporter_contact,
                'reporter_id_type' => $request->reporter_id_type,
                'reporter_relationship' => $request->reporter_relationship,
                'documentations' => $docs,
                'status' => 'Submitted',
            ]);

            if ($app) {
                ApplicationsProcessing::create([
                    'application_id'    =>  $app->id,
                    'applicable_id'           =>  Auth::id(),
                    'applicable_type'           =>  'App\Models\User',
                    'action'            =>  $app->status,
                    'access_type'            =>  'Mobile',
                    'stage'             =>  'New Order',
                    'remarks'             =>  ''
                ]);
            }

            return $this->successResponse([
                'status'=>true,
                'application_benefit_type'=>$app->type,
                'application_status'=>$app->status,
                'application_created_date'=>date('Y-m-d', strtotime($app->created_at)),
            ],'Your application has been submitted.');
        }
        catch (\Exception $exception){
            return $this->errorResponse(null,'Sorry! Submission failed.', 401);
        }

    }

    /**
     * Validates the Applicant and checks for eligibility and returns the various benefit document requirements
     * @param Request $request
     * @return JsonResponse
     */
    public function validateApplicant(Request $request)
    {
        $this->checkApplicantStatus();
        if (auth()->user()->applicant_number != $request->applicant_number) {
            return $this->errorResponse(null,'Invalid Account!',$this->badRequestStatus);
        }
        $eligible_data = isApplicantEligible(auth()->user()->applicant_number,$request->benefit_id);
        if(!$eligible_data['status']) {
            return $this->errorResponse(null,'Applicant is not eligible for this benefit claim',$this->badRequestStatus);
        }

        if (!is_null(auth()->user()->applicant_number)) {
            $benefit = Benefit::find($request->benefit_id);
            $documents = $benefit->supporting_documents;
            $applicant_documents = auth()->user()->applicant->documents->pluck('type','id')->toArray();
            $benefit_supporting_documents = [];
            if ($documents){
                foreach ($documents as $document) {
                    $benefit_supporting_documents[Str::snake($document)] = $document;
                }
            }

            return $this->successResponse([
                'applicant_documents'=>$applicant_documents,
                'benefit_supporting_documents'=>$benefit_supporting_documents
            ],'Applicant is valid.');
        }
        else {
            return $this->errorResponse(null,'No Record Found',$this->badRequestStatus);
        }
    }

    /**
     * Get Active Parent Options
     * @return mixed
     */
    public function getParentOption() {
        $options = LookupCode::where('activeFlag','=',1)->orderBy('lookupName','asc')->pluck('lookupName','id');
        return $this->successResponse($options,'successful');
    }

    /**
     * Get All Active Options
     * @param $lookupCode_id
     * @return mixed
     */
    public function getActiveOptionsFor(Request $request) {
        $options = LookupCode::activeOptions($request->lookupCode_id);
        return $this->successResponse($options,'successful');
    }

    /**
     * Get specific Option
     * @param $lookupCode_id
     * @return mixed
     */
    public function getOptionValue(Request $request) {
        $option = getLookupSelectedOptionTextByValue($request->option_id);
        return $this->successResponse($option,'successful');
    }

    /**
     * Get Banks
     * @return JsonResponse
     */
    public function getBanks() {
        $banks = Bank::where('status',1)->get();
        return $this->successResponse($banks,'successful');
    }

    /**
     * Get Banks Branches
     * @return JsonResponse
     */
    public function getBankBranches(Request $request) {
        $branches = BankBranch::where('status',1)->where('bank_id',$request->bank_id)->get();
        return $this->successResponse($branches,'Successful');
    }

    public function readNotificationPosts()
    {
        $read_notifications = auth()->user()->readNotifications()->pluck('notification_post_id')->toArray();
        $notifications = Announcement::with(['addedBy:id,name,firstname,othername'])->whereIn('id',$read_notifications)->get();

        return $this->successResponse($notifications->makeHidden(['recipients']),'successful');
    }

    public function unReadNotificationPosts()
    {
        $read_notifications = auth()->user()->readNotifications()->pluck('notification_post_id')->toArray();
        $unread_notifications = Announcement::with(['addedBy:id,name,firstname,othername'])->whereNotIn('id',$read_notifications)->get();

        return $this->successResponse($unread_notifications->makeHidden(['recipients']),'successful');
    }

    public function openNotificationPost($id)
    {
        $notification_post = Announcement::with(['addedBy:id,name,firstname,othername'])->where('id','=',$id)->first();
        if ($notification_post) {

            if(!hasReadNotification($id)){
                AnnouncementsRead::create([
                    'user_id'   =>  auth()->id(),
                    'notification_post_id'   =>  $id,
                    'read_at'   =>  now(),
                ]);
            }
            return $this->successResponse($notification_post->makeHidden(['recipients']),'successful');
        }
        else {
            return $this->errorResponse(null,'Notification Post does not exist',$this->badRequestStatus);
        }
    }

    //CASE MANAGEMENT
    public function saveCase(Request $request)
    {
//        $request->validate([
//            'name'  =>  'required|min:3',
//            'contact'  =>  'required|min:3',
//            'reference'  =>  'required|min:3',
//            'title'  =>  'required|min:3',
//            'category'  =>  'required|min:3',
//            'type'  =>  'required',
//            'severity'  =>  'required|min:3',
//            'description'  =>  'required|min:3',
//            'beneficiary'  =>  'nullable',
//            'file_upload'  =>  'nullable|min:3',
//        ]);

        $beneficiary_id = null;
        if (!is_null($request->beneficiary)) {
            $beneficiary_id = Applicant::where('regID', '=', $request->beneficiary)->pluck('id')->first();
            if(!$beneficiary_id) {
               return $this->errorResponse(null,'Applicant does not exist',$this->badRequestStatus);
            }
        }
        if ($request->case_id == null) {

            $case = new CaseManagement();

            $alert_message = "A New Case has been created.";
            $case->stage = 'Opened';
        }
        else {
            $case = CaseManagement::find($request->case_id);
            $alert_message = "Your Case has been updated.";
        }

        $case->beneficiary_id = $beneficiary_id;
        $case->user_id = Auth::id();
        $case->title = $request->title;
        $case->description = $request->description;
        $case->severity = $request->severity;
        $case->category = $request->category;
        $case->type = $request->type;
        $case->name = auth()->user()->fullName;
        $case->contact = auth()->user()->phone;
        $case->reference = $request->reference;


        if(isset($request->remove_file)) {
            $case->file_upload = null;
        }
        if($request->file('file_upload')) {
            $file = $this->uploadDocument();
            $case->file_upload = 'app-uploads/'.$file;
        }

        if ($case->save()) {
            return $this->successResponse($case->makeHidden($this->hiddenCases),$alert_message);
        }
        else {
            return $this->errorResponse(null,'Failed to save! Something went wrong.',$this->badRequestStatus);
        }
    }

    public function uploadDocument($key = 'file_upload', $folder = 'cases', $validation = 'nullable|file|mimes:jpeg,png,jpg,docx,pdf,txt,xlsx|max:10048|sometimes')
    {
        request()->validate([$key => $validation]);

        $file = null;
        if (request()->hasFile($key)) {
            $file = Storage::disk('public_uploads')->putFile($folder, request()->file($key), 'public');
        }

        return $file;
    }

    public function getCase($case_id)
    {
        $case = CaseManagement::with(['_category:id,fullName','_type:id,fullName',
            'comments'=>['respond:id,name,firstname,othername']])->where('id','=',$case_id)->where('user_id','=', auth()->id())->first();

        if(!$case) {
            return $this->errorResponse(null,'Complaint does not exist',$this->badRequestStatus);
        }

        return $this->successResponse($case->makeHidden($this->hiddenCases),'Successful');
    }
    public function cases()
    {
        $cases = CaseManagement::with(['_category:id,fullName','_type:id,fullName',
            'comments'=>['respond:id,name,firstname,othername']])->where('user_id','=',auth()->id())->get();

        return $this->successResponse($cases->makeHidden($this->hiddenCases),'Successful');
    }

    /**
     *  Delete a Case
     * @param $case_id
     * @return JsonResponse
     */
    public function deleteCase(Request $request)
    {
        if (CaseManagement::where('id','=',$request->case_id)->where('user_id','=', auth()->id())->delete()) {
            return $this->successResponse(null,'Complaint has been deleted');
        }
        return $this->errorResponse(null,'Complaint does not exist',$this->badRequestStatus);
    }

    /**
     * Get All User Survey Entries
     */
    public function mySurveys()
    {
        $surveys = $this->mySurveysList();
        return $this->successResponse($surveys->makeHidden($this->hiddenSurveysColumns),'My Surveys');
    }

    /**
     * Get Survey Form
     */
    public function getSurveyForm($survey_id)
    {
        $survey = Survey::find($survey_id);

        if (!$survey) {
            return $this->errorResponse(null,'Invalid Survey',$this->badRequestStatus);
        }
        $group = ['all-users','applicants'];
        if (auth()->user()->is_admin) $group = ['all-users','administrators'];

        if ($survey->mode == 'self' && in_array($survey->self_respondents,$group)) {

            $form = Survey::with(['form'=>function($query) {
                $query->with(['sections','questions'])->select(['id','name','category']);
            }])->where('id','=',$survey_id)->first();

            return $this->successResponse($form->makeHidden($this->hiddenSurveysColumns),'');
        }
        else {
            return $this->errorResponse(null,'Survey does not exist',$this->badRequestStatus);
        }
    }

    /**
     * Save an Entry
     * @param $survey_id
     * @param Request $request
     * @return JsonResponse|\Illuminate\Http\RedirectResponse|void
     * @throws \Illuminate\Validation\ValidationException
     */
    public function saveEntry($survey_id,Request $request)
    {
        $survey = Survey::findOrFail($survey_id);

        if (!$survey) {
            return $this->errorResponse(null,'Invalid Survey',$this->badRequestStatus);
        }

        if (!$survey->isEligible(auth()->user())) {
            return $this->errorResponse(null,'Sorry,you are not eligible to complete this survey.',$this->badRequestStatus);
        }

        if (strtotime($survey->end_date) < strtotime(date('Y-m-d'))) {
            return $this->errorResponse(null,'Sorry! The survey is overdue',$this->badRequestStatus);
        }

        if (strtotime($survey->start_date) > strtotime(date('Y-m-d'))) {
            return $this->errorResponse(null,'Invalid Survey',$this->badRequestStatus);
        }

        $answers = $this->validate($request, $survey->form->rules);

        $entry = (new Entry)->for($survey->form)->onSurvey($survey)->by(auth()->user())->fromArray($answers)->push();

        if($entry){
            return $this->successResponse($entry,'Entry has been saved');
        }
        else {
            return $this->errorResponse(null,"Something went wrong. Try again.",$this->badRequestStatus);
        }
    }

    /**
     * Get Survey Entry
     */
    public function getSurveyFormEntry($survey_id)
    {
        $survey = $this->getSurveyFormEntryById($survey_id);

        return $this->successResponse($survey->makeHidden($this->hiddenSurveysColumns),'');
    }

    public function checkApplicantStatus()
    {
        if (is_null(auth()->user()->applicant_number)){
            return $this->errorResponse(null,'This account has no applicant attached',$this->badRequestStatus);
        }
        return $this->profile();
    }

    /*******************************************************************
     * ENDPOINTS FOR CONSOLE
     *******************************************************************/

    /**
     * @param $console_code
     * @return JsonResponse
     */
    public function getUsers($console_code)
    {
        if ($console_code != env('CONSOLE_DEPLOYMENT_CODE')) {
            $users = 0;
        }
        else {
            $users = User::count();
        }

        return $this->successResponse(['total'=>$users],'Users found');
    }

    /**
     * @param $console_code
     * @return JsonResponse
     */
    public function getApplicants($console_code)
    {
        if ($console_code != env('CONSOLE_DEPLOYMENT_CODE')) {
            $applicants = 0;
        }
        else {
            $applicants = Applicant::active()->count();
        }

        return $this->successResponse(['total'=>$applicants],'Applicants found');
    }

    /**
     * @param $console_code
     * @return JsonResponse
     */
    public function getApplications($console_code)
    {
        if ($console_code != env('CONSOLE_DEPLOYMENT_CODE')) {
            $applications = 0;
        }
        else {
            $applications = Application::count();
        }

        return $this->successResponse(['total'=>$applications],'Applications found');
    }

    public function updateProject($console_code)
    {
        if ($console_code == env('CONSOLE_DEPLOYMENT_CODE')) {

            if (Artisan::call('console:update')) {
                return $this->successResponse(['status'=>true],'Applications found');
            }
            else {
                return $this->errorResponse(null,'Update did not complete. Try again.',$this->badRequestStatus);
            }
        }

        return $this->errorResponse(null,'Invalid request',$this->badRequestStatus);

    }
}
