<?php

namespace App\Http\Controllers;

use App\Http\Middleware\LoadUserPermissions;
use App\Models\Bank;
use App\Models\BankBranch;
use App\Models\Permission;
use App\Models\Role;
use App\Models\User;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\QueryException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Laratrust\Models\LaratrustRole;

class BanksController extends Controller
{
    public function __construct()
    {
        $this->middleware(['auth:web', LoadUserPermissions::class]);
    }

    public function index(Request $request){

        $banks = Bank::withCount('branches')->orderBy('name');
        if (isset($request->bank)) {
            $banks = $banks->where('name','like','%'.$request->bank.'%')->orWhere('shortname','like','%'.$request->bank.'%');
        }
        if (isset($request->type)) {
            $banks = $banks->where('type','=',$request->type);
        }
        $banks = $banks->get();
        return view('banks.index', compact('banks'));
    }


    /**
     * Function for upload image
     *
     * @param  string  $folder
     * @param  string  $key
     * @param  string  $validation
     *
     * @return false|string|null
     */
    public function uploadProfile($key = 'logo', $validation = 'image|mimes:jpeg,png,jpg,gif,svg|max:2048|sometimes')
    {
        $folder = 'banks';
        request()->validate([$key => $validation]);

        $file = null;
        if (request()->hasFile($key)) {
            $file = Storage::disk('public_uploads')->putFile($folder, request()->file($key), 'public');
        }

        return $file;
    }

    public function emptyForm(){
        $types = getBankTypes();
        $locations = getLocations();
        $leads = Role::all();
        $users = User::all();
        $theView = view('banks.empty_form', compact('types','locations','leads','users'));
        return $theView;
    }

    public function create(Request $request){
       $this->validate($request,[
            'name' => 'required|string|min:2|unique:banks,name',
            'shortname' => 'required|string|min:2|unique:banks,shortname',
           'type' => 'required|integer',
           'swift_code' => 'nullable|integer',
           'contact' => 'required|string',
           'address' => 'required|string',
           'email' => 'required|email',
           'website' => 'required|string',
           'logo' => 'nullable|file',
        ]);

        try{
            $bank = new Bank();
            $bank->name   = $request->name;
            $bank->shortname  = $request->shortname;
            $bank->type = $request->type;
            $bank->swift_code = $request->swift_code;
            $bank->email    = $request->email;
            $bank->website = $request->website;
            $bank->contact    = $request->contact;
            $bank->address = $request->address;
            $bank->created_by  = Auth::id();

            if($request->file('logo')) {
//                $file = $request->file('logo');
//                $name = $file->hashName();
//                $extension = $file->getClientOriginalExtension();
//                $full_name = "$name.$extension";
//                $url = 'banks/'.$bank->id.'/logo';
//                Storage::disk('public_uploads')->putFileAs($url, $request->file('logo'),$full_name);

                $file = $this->uploadProfile();
                $bank->logo = "app-uploads/$file";
            }

            $bank->save();

            $message = 'A New Bank has been added!';
            session()->flash('message', $message);
            session()->flash('status', 'success');

        }catch(\Illuminate\Database\QueryException $ex){
            $message = 'Bank may already exist or check enter appropriate data!';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        return back();
    }

    public function edit(Request $request){
        $bank = Bank::find($request->bank_id);
        $types = getBankTypes();

        $theView = view('banks.edit', compact('bank','types'))->render();
        return response()->json(['theView'=>$theView]);
    }

    public function update(Request $request)
    {
        $this->validate($request,[
            'name' => 'required|string|min:2|unique:banks,name,'.$request->id,
            'shortname' => 'required|string|min:2|unique:banks,shortname,'.$request->id,
            'type' => 'required|integer',
            'contact' => 'required|string',
            'swift_code' => 'nullable|string',
            'address' => 'required|string',
            'email' => 'required|email',
            'website' => 'required|string',
            'logo' => 'nullable|file',
        ]);

        try {
            $bank = Bank::find($request->id);
            $bank->name   = $request->name;
            $bank->shortname  = $request->shortname;
            $bank->type = $request->type;
            $bank->swift_code = $request->swift_code;
            $bank->email    = $request->email;
            $bank->website = $request->website;
            $bank->contact    = $request->contact;
            $bank->address = $request->address;
            $bank->updated_by  = Auth::id();

            if($request->file('logo')) {
//                $file = $request->file('logo');
//                $name = $file->hashName();
//                $extension = $file->getClientOriginalExtension();
//                $full_name = "$name.$extension";
//                $url = 'banks/'.$bank->id.'/logo';
//                Storage::disk('public_uploads')->putFileAs($url, $request->file('logo'),$full_name);

                $file = $this->uploadProfile();
                $bank->logo = "app-uploads/$file";
            }

            $bank->save();

            $message = 'Bank information has been updated!';
            session()->flash('message', $message);
            session()->flash('status', 'success');

            return back();
        }catch (QueryException $exception){
            $message = 'The Bank name may already exist or check enter appropriate data!'.$exception->getMessage();
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }
    }

    public function remove($id)
    {
        try {
            $bank = Crypt::decrypt($id);
            Bank::destroy($bank);

            $message = 'Bank has been removed!';
            session()->flash('message', $message);
            session()->flash('status', 'success');
            return back();
        }catch (QueryException $exception){
            $message = 'Something went wrong. Try again or contact Administrator.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }
    }

    public function activation($id)
    {
        try {
            $bank = Bank::find(Crypt::decrypt($id));
            if (!is_null($bank)) {
                if ($bank->status == 1){
                    $bank->status = 0;
                    $message = 'The Bank has been deactivated!';
                }
                else {
                    $bank->status = 1;
                    $message = 'The Bank has been activated!';
                }
                $bank->save();

                session()->flash('message', $message);
                session()->flash('status', 'success');
            }
            else {
                $message = 'Bank does not exist.';
                session()->flash('message', $message);
                session()->flash('status', 'error');
            }

            return back();
        }catch (QueryException $exception){
            $message = 'Something went wrong. Try again or contact Administrator.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }
    }

    //BRANCHES
    public function branchIndex(Request $request){

        $banks = Bank::where('status','=',1)->orderBy('name')->get();
        $branches = BankBranch::orderBy('name');

        if (isset($request->name)) {
            $branches = $branches->where('name','like','%'.$request->name.'%')->orWhere('routing_number','like','%'.$request->name.'%');
        }
        if (isset($request->bank)) {
            $branches = $branches->where('bank_id','=',$request->bank);
        }
        if (isset($request->location)) {
            $branches = $branches->where('location_id','=',$request->location);
        }
        $branches = $branches->get();
        return view('banks.branch-index', compact('banks','branches'));
    }

    public function branchEmptyForm(){

        $banks = Bank::where('status','=',1)->orderBy('name')->get();
        $theView = view('banks.empty-branch-form', compact('banks'));
        return $theView;
    }

    public function branchCreate(Request $request){
        $this->validate($request,[
            'name' => 'required|string|min:2',
            'routing_number' => 'nullable|string|min:2',
            'bank' => 'required|integer',
            'contact' => 'required|string',
            'address' => 'required|string',
            'services' => 'required|string',
            'email' => 'required|email',
            'location' => 'required|integer',
        ]);

        try{
            $bank = new BankBranch();
            $bank->name   = $request->name;
            $bank->routing_number  = $request->routing_number;
            $bank->bank_id = $request->bank;
            $bank->services = $request->services;
            $bank->email    = $request->email;
            $bank->location_id = $request->location;
            $bank->contact    = $request->contact;
            $bank->address = $request->address;
            $bank->created_by  = Auth::id();
            $bank->save();

            $message = 'A New Branch has been added!';
            session()->flash('message', $message);
            session()->flash('status', 'success');

        }catch(\Illuminate\Database\QueryException $ex){
            $message = 'Sorry! Something went wrong. Please try again. ';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        return back();
    }

    public function branchEdit(Request $request){
        $branch = BankBranch::find($request->id);
        $banks = Bank::where('status','=',1)->orderBy('name')->get();

        $theView = view('banks.edit-branch', compact('banks','branch'))->render();
        return response()->json(['theView'=>$theView]);
    }

    public function branchUpdate(Request $request)
    {
        $this->validate($request,[
            'name' => 'required|string|min:2',
            'routing_number' => 'nullable|string|min:2',
            'bank' => 'required|integer',
            'contact' => 'required|string',
            'services' => 'required|string',
            'address' => 'required|string',
            'email' => 'required|email',
            'location' => 'required|integer',
        ]);

        try {
            $bank = BankBranch::find($request->id);
            $bank->name   = $request->name;
            $bank->routing_number  = $request->routing_number;
            $bank->bank_id = $request->bank;
            $bank->services = $request->services;
            $bank->email    = $request->email;
            $bank->location_id = $request->location;
            $bank->contact    = $request->contact;
            $bank->address = $request->address;
            $bank->updated_by  = Auth::id();

            $bank->save();

            $message = 'Branch information has been updated!';
            session()->flash('message', $message);
            session()->flash('status', 'success');

            return back();
        }catch (QueryException $exception){
            $message = 'Sorry! Something went wrong. Please try again. ';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }
    }

    public function branchRemove($id)
    {
        try {
            $bank = Crypt::decrypt($id);
            Bank::destroy($bank);

            $message = 'Bank has been removed!';
            session()->flash('message', $message);
            session()->flash('status', 'success');
            return back();
        }catch (QueryException $exception){
            $message = 'Something went wrong. Try again or contact Administrator.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }
    }

    public function branchActivation($id)
    {
        try {
            $bank = Bank::find(Crypt::decrypt($id));
            if (!is_null($bank)) {
                if ($bank->status == 1){
                    $bank->status = 0;
                    $message = 'The Bank has been deactivated!';
                }
                else {
                    $bank->status = 1;
                    $message = 'The Bank has been activated!';
                }
                $bank->save();

                session()->flash('message', $message);
                session()->flash('status', 'success');
            }
            else {
                $message = 'Bank does not exist.';
                session()->flash('message', $message);
                session()->flash('status', 'error');
            }

            return back();
        }catch (QueryException $exception){
            $message = 'Something went wrong. Try again or contact Administrator.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }
    }
}
