<?php

namespace App\Http\Controllers;

use App\Http\Helpers\FormRender;
use App\Http\Traits\FormSubmission;
use App\Models\Applicant;
use App\Models\Application;
use App\Models\ApplicationApproval;
use App\Models\ApplicationForm;
use App\Models\ApplicationGroup;
use App\Models\ApplicationItem;
use App\Models\ApplicationsProcessing;
use App\Models\Benefit;
use App\Models\BenefitAmount;
use App\Models\BenefitFormOption;
use App\Models\BenefitInventoryItem;
use App\Models\BenefitPayment;
use App\Models\BenefitRegistration;
use App\Models\BenefitType;
use App\Models\Constituency;
use App\Models\Deployment;
use App\Models\District;
use App\Models\Form;
use App\Models\InventoryItem;
use App\Models\ItemIssuance;
use App\Models\LookupCode;
use App\Models\Office;
use App\Models\PaymentSchedule;
use App\Models\Pmt;
use App\Models\Role;
use App\Models\SubCounty;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Str;
use Illuminate\Validation\Rule;

class BenefitsController extends Controller
{
    use FormSubmission;

    public function __construct()
    {
        $this->middleware(['auth', 'verified']);
    }

    /**
     * Display a listing of the Benefit.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $forms = Benefit::where('instance_id','=',auth()->user()->last_instance_opened)->orderBy('titles','asc');
        if (isset($request->status)) {
            $forms = $forms->where('archived','=',$request->status);
        }
        if (isset($request->class)) {
            $forms = $forms->where('class','=',$request->class);
        }
        if (isset($request->type)) {
            $forms = $forms->where('type','=',$request->type);
        }
        $forms = $forms->with(['_class','_type'])->get();
        $types = BenefitType::all();
        $restored = true;
        $data = [];

        $benefit_amounts = BenefitAmount::select(['benefit_id','min_value','max_value'])
            ->distinct()
            ->whereIn('benefit_id',$forms->pluck('id')->toArray())
            ->get();

        if($benefit_amounts) {
            foreach($benefit_amounts as $amount){
                $data[$amount->benefit_id] = [
                    'value' => getCurrency().' '.money($amount->min_value).' - '. getCurrency().' '.money($amount->max_value),
                ];
            }
        }


        return view('benefits.index',compact('forms','restored','types','data'));
    }

    public function eligibility(Request $request)
    {
        $applicants = $result = null;
        $note = "";

        if ($request->benefit) {
            $benefit = Benefit::find((int)$request->benefit);
            $result = $benefit;

            if(!$result) {
                $message = 'No Benefit record found!';
                session()->flash('message', $message);
                session()->flash('status', 'error');
                return back();
            }

            $applicants = Applicant::active()->pluck('id');

            if(!is_null($benefit->min_age) || !is_null($benefit->max_age)) {
                $note .= " <span class='text-danger fw-bolder'>Age</span> is ";
                $fDate = Carbon::now();
                $tDate = Carbon::now();
                if ($benefit->max_age < 1) $benefit->max_age = 210;
                $from_age = $fDate->subYears((int)$benefit->max_age)->format('Y-m-d');
                $to_age = $tDate->subYears((int)$benefit->min_age)->format('Y-m-d');

                $applicants = Applicant::whereIn('id',$applicants)
                    ->whereDate('dobs','>=',$from_age)
                    ->whereDate('dobs','<=',$to_age)
                    ->pluck('id');

                if(!is_null($benefit->min_age) && !is_null($benefit->max_age)) {
                    $note .= "between ".$benefit->min_age.' & '.$benefit->max_age.' years.';
                }
                elseif (is_null($benefit->min_age) && !is_null($benefit->max_age)) {
                    $note .= "below ".$benefit->max_age.' years.';
                }
                elseif (is_null($benefit->min_age) && !is_null($benefit->max_age)) {
                    $note .= "above ".$benefit->min_age.' years.';
                }
                $note .= "<br>";
            }

            //Check Gender
            if(!is_null($benefit->gender_criteria)) {
                $applicants = Applicant::whereIn('id',$applicants)
                    ->whereIn('genders',$benefit->gender_criteria)
                    ->pluck('id');
                if (sizeof($benefit->gender_criteria) >= 1) {
                    if (sizeof($benefit->gender_criteria) > 1) $note .= ' <span class="text-danger fw-bolder">Gender</span> can be ' . implode(", ", $benefit->gender_criteria) . '. ';
                    else  $note .= ' <span class="text-danger fw-bolder">Gender</span> is ' . implode(", ", $benefit->gender_criteria) . '. ';

                    $note .= "<br>";
                }
            }

            //Check Residence
            if(!is_null($benefit->residence_criteria)) {

                /*$applicants = DB::table('applicants')->join('contact_details','applicant_number','=','regID')
                    ->where('contact_details.type_name','=','Residential Address')
                    ->whereIn('applicants.id',$applicants)
                    ->whereIn('contact_details.island',$benefit->residence_criteria)
                    ->where('contact_details.status','verified')
                    ->pluck('applicants.id');
                if (sizeof($benefit->residence_criteria) >= 1) {
                    if (sizeof($benefit->residence_criteria) > 1) $note .= ' <span class="text-danger fw-bolder">Residence</span> can be '.implode(", ",$benefit->residence_criteria).'. ';
                    else  $note .= ' <span class="text-danger fw-bolder">Residence</span> is '.implode(", ",$benefit->residence_criteria).'. ';
                    $note .= "<br>";
                }*/

                $applicants = DB::table('applicants')->join('contact_details','applicant_number','=','regID')
                    ->where('contact_details.type_name','=','Residential Address')
                    ->whereIn('applicants.id',$applicants)
                    ->whereIn('contact_details.village',$benefit->residence_criteria)
                    ->where('contact_details.status','verified')
                    ->pluck('applicants.id');
                if (sizeof($benefit->residence_criteria) >= 1) {
                    if (sizeof($benefit->residence_criteria) > 1) $note .= ' <span class="text-danger fw-bolder">Residence</span> can be '.implode(", ",$benefit->residence_criteria).'. ';
                    else  $note .= ' <span class="text-danger fw-bolder">Residence</span> is '.implode(", ",$benefit->residence_criteria).'. ';
                    $note .= "<br>";
                }
            }

            //Check nationality
            if(!is_null($benefit->nationalities)) {
                $applicants = DB::table('applicants')->whereIn('id',$applicants)
                    ->whereIn('nationalities',$benefit->nationalities)
                    ->pluck('id');
                if (sizeof($benefit->nationalities) >= 1) {
                    if (sizeof($benefit->nationalities) > 1) $note .= ' <span class="text-danger fw-bolder">Nationality</span> can be ' . implode(", ", $benefit->nationalities) . '. ';
                    else  $note .= ' <span class="text-danger fw-bolder">Nationality</span> is ' . implode(", ", $benefit->nationalities) . '. ';
                    $note .= "<br>";
                }
            }

            //Check Impairment
            if(!is_null($benefit->impairments)) {
                $applicants = DB::table('applicants')->join('health_details','health_details.regID','=','applicants.regID')
                    ->whereIn('applicants.id',$applicants)
                    ->whereIn('health_details.health_type',$benefit->impairments)
                    ->where('health_details.status','verified')
                    ->pluck('applicants.id');
                if (sizeof($benefit->impairments) >= 1) {
                    if (sizeof($benefit->impairments) > 1) $note .= ' <span class="text-danger fw-bolder">Impairments</span> can be ' . implode(", ", $benefit->impairments) . '. ';
                    else  $note .= ' <span class="text-danger fw-bolder">Impairments</span> is ' . implode(", ", $benefit->impairments) . '. ';
                    $note .= "<br>";
                }
            }

            //Check Employment Type
            if(!is_null($benefit->employment_type_criteria)) {
                $applicants = DB::table('applicants')->join('employment_details','applicant_number','=','regID')
                    ->whereIn('applicants.id',$applicants)
                    ->whereIn('employment_details.employment_type',$benefit->employment_type_criteria)
                    ->where('employment_details.status','verified')
                    ->pluck('applicants.id');
                if (sizeof($benefit->employment_type_criteria) >= 1) {
                    if (sizeof($benefit->employment_type_criteria) > 1) $note .= ' <span class="text-danger fw-bolder">Employment Type</span> can be ' . implode(", ", $benefit->employment_type_criteria) . '. ';
                    else  $note .= ' <span class="text-danger fw-bolder">Employment Type </span>  is ' . implode(", ", $benefit->employment_type_criteria) . '. ';
                    $note .= "<br>";
                }
            }

            //Check Employment Sector
            if(!is_null($benefit->employment_criteria)) {
                $applicants = DB::table('applicants')->join('employment_details','applicant_number','=','regID')
                    ->whereIn('applicants.id',$applicants)
                    ->whereIn('employment_details.employment_sector',$benefit->employment_criteria)
                    ->where('employment_details.status','verified')
                    ->pluck('applicants.id');
                if (sizeof($benefit->employment_criteria) >= 1) {
                    if (sizeof($benefit->employment_criteria) > 1) $note .= ' <span class="text-danger fw-bolder">Employment</span> sector can be ' . implode(", ", $benefit->employment_criteria) . '. ';
                    else  $note .= ' <span class="text-danger fw-bolder">Employment sector</span> is ' . implode(", ", $benefit->employment_criteria) . '. ';
                    $note .= "<br>";
                }
            }

            $applicants = Applicant::with([
                'applications'=>['benefit']
            ])->whereIn('id',$applicants)
                ->orderBy('regID','asc')
                ->cursorPaginate(150)->withQueryString();
        }

        $benefits = Benefit::with('form')->where('instance_id','=',auth()->user()->last_instance_opened)->active()->get();

        return view('benefits.eligibility',compact('applicants','benefits','result','note'));
    }

    /**
     * Display a listing of the Benefit Amount.
     *
     */
    public function benefitAmount(Request $request)
    {
        $amounts = null;$count = 0; $data = [];
        if (isset($request->benefit)) {
            $amounts = BenefitAmount::where('benefit_id','=',$request->benefit)
            ->where('year','=',$request->date)
            ->orderBy('month','asc')
            ->get();

            foreach($amounts as $amount){
                $data[$amount->month] = [
                    'id' => $amount->id,
                    'created_at' => date('d M Y', strtotime($amount->created_at)),
                    'value' => getCurrency().' '.money($amount->min_value).' - '. getCurrency().' '.money($amount->max_value),
                ];
            }

            $count = $amounts->count();
        }

        $benefits = Benefit::where('instance_id','=',auth()->user()->last_instance_opened)->pluck('titles','id');

        return view('benefits.amounts',compact('benefits','amounts','count','data'));
    }

    public function saveAmount(Request $request)
    {
        $this->validate($request,[
            'year'   =>  'required|integer|min:1',
            'benefit'   =>  'required|integer|min:1',
            'min_value'   =>  'required|integer|min:1',
            'max_value'   =>  'required|integer|min:1',
            'start_date'   =>  'required|string|min:1',
            'end_date'   =>  'required|string|min:1',
        ]);

        if((integer)$request->end_date < (integer)$request->start_date) {
            $message = 'The End Month must be greater than or equal to the Start Month.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back()->withInput();
        }

        if((integer)$request->max_value < (integer)$request->min_value) {
            $message = 'The Max Value must be greater than or equal to the Min Value.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back()->withInput();
        }

        $counter = (int)$request->end_date - (int)$request->start_date + 1;
        $looped = 0;

        for ($month=(int)$request->start_date; $month <= (int)$request->end_date; $month++) {
            BenefitAmount::updateOrCreate([
                'year' => (int)$request->year,
                'benefit_id' => (int)$request->benefit,
                'month' => $month,
            ],[
                'min_value' => (int)$request->min_value,
                'max_value' => (int)$request->max_value,
                'created_by' => Auth::id(),
            ]);
            $looped += 1;
        }

        if($counter == $looped) {
            $message = 'Amount has been submitted!';
            session()->flash('message', $message);
            session()->flash('status', 'success');
        }
        elseif($counter > $looped) {
            $message = "Only $looped out of $counter months were added. Please review to add the remaining months.";
            session()->flash('message', $message);
            session()->flash('status', 'warning');
        }
        else {
            $message = 'Sorry! Something went wrong. Please try again!';
            session()->flash('message', $message);
            session()->flash('status', 'error');
        }

        return back();
    }

    public function updateAmount(Request $request)
    {
        $this->validate($request,[
            'min_value'   =>  'required|integer|min:1',
            'max_value'   =>  'required|integer|min:1',
        ]);

        if((integer)$request->max_value < (integer)$request->min_value) {
            $message = 'The Max Value must be greater than or equal to the Min Value.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back()->withInput();
        }

        $amount = BenefitAmount::find(Crypt::decrypt($request->id));
        $amount->min_value = $request->min_value;
        $amount->max_value = $request->max_value;


        if($amount->save()) {
            $message = "Value have been updated successfully.";
            session()->flash('message', $message);
            session()->flash('status', 'success');
        }
        else {
            $message = 'Sorry! Something went wrong. Please try again!';
            session()->flash('message', $message);
            session()->flash('status', 'error');
        }
        return back();
    }

    public function amountEdit(Request $request)
    {
        $amount = BenefitAmount::find($request->id);
        $start_date = $end_date = null;
        if(!is_null($amount->start_date)) {
            $start_date = date('Y-m', strtotime($amount->start_date));
        }
        if(!is_null($amount->end_date)) {
            $end_date = date('Y-m', strtotime($amount->start_date));
        }
        return response()->json([
            'min_value'=>$amount->min_value,
            'max_value'=>$amount->max_value,
            'benefit_id'=>$amount->benefit_id,
            'start_date'=>  $start_date,
            'end_date'=> $end_date,
            'id'=>cryptThis($request->id)
        ]);
    }

    public function amountDelete(Request $request)
    {
        if (BenefitAmount::destroy(Crypt::decrypt($request->id))) {
            $message = 'Amount has been deleted.';
            session()->flash('message', $message);
            session()->flash('status', 'success');
        }
        else {
            $message = 'Sorry! Try again';
            session()->flash('message', $message);
            session()->flash('status', 'error');
        }
        return back();
    }

    /**
     * Display a listing of the Benefit.
     *
     * @return \Illuminate\Http\Response
     */
    public function benefitTypes(Request $request)
    {
        $types = BenefitType::orderBy('name','asc');
        if (isset($request->status)) {
            $types = $types->where('status','=',$request->status);
        }
        if (isset($request->class)) {
            $types = $types->where('class','=',$request->class);
        }
        $types = $types->get();
        $classes = getBenefitClasses();

        return view('benefits.types',compact('types','classes'));
    }

    /**
     * Adding a new Option
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function typeSave(Request $request){

        $this->validate($request,[
            'name' => ['required','string','min:2',Rule::unique('benefit_types','name')->where(function ($query) use($request) {
                return $query->where('class', $request->class);
            })],
            'class' => 'required|integer',
            'description' => 'nullable|string',
        ]);
        try{
            $createType = new BenefitType();
            $createType->name  = Str::title($request->name);
            $createType->class  = $request->class; // optional
            $createType->description  = $request->description; // optional
            $createType->created_by  = Auth::id(); // optional
            $createType->save();

            $message = 'Type has been created!';
            session()->flash('message', $message);
            session()->flash('status', 'success');

        }catch(\Exception $exception){
            $message = 'Sorry! Option name already exist or check and enter appropriate data. Please try again. '.$exception->getMessage();
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        return back();
    }

    /**
     * Collect an Option data and post to frontend
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function typeEdit(Request $request)
    {
        $option = BenefitType::find($request->id);
        return response()->json($option);
    }

    /**
     * Collect an Option data and post to frontend
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function typeView(Request $request)
    {
        $type = BenefitType::find($request->id);
        $view = view('benefits.view-type', compact('type'))->render();
        return response()->json(['view'=>$view]);
    }

    public function activateType($type_id)
    {
        $type = BenefitType::find(Crypt::decrypt($type_id));
        //Restore an Archived Benefit
        if ($type) {
            if ($type->status) {
                $type->status = 0;
                $type->save();
                $message = 'Type has been disabled!';
                session()->flash('message', $message);
                session()->flash('status', 'success');
            }
            else {
                $type->status = 1;
                $type->save();
                $message = 'Type has been enabled!';
                session()->flash('message', $message);
                session()->flash('status', 'success');
            }
        }
        else {
            $message = 'Sorry! Something went wrong. Try again';
            session()->flash('message', $message);
            session()->flash('status', 'error');
        }

        return back();
    }

    /**
     * Update an Option
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function typeUpdate(Request $request)
    {
        $this->validate($request,[
            'id' => 'required|numeric|min:1',
            'name' => ['required','string','min:1',Rule::unique('benefit_types','name')->ignore($request->id)],
            'class' => 'required|string',
            'description' => 'nullable|string',
        ]);

        try{
            $updateType = BenefitType::find($request->id);

            $updateType->name  = Str::title($request->name);
            $updateType->class  = $request->class; // optional
            $updateType->description  = $request->description; // optional
            $updateType->updated_by  = Auth::id(); // optional
            $updateType->save();

            $message = 'Benefit Type has been updated!';
            session()->flash('message', $message);
            session()->flash('status', 'success');

        }catch(\Illuminate\Database\QueryException $ex){
            $message = 'Sorry! Option name already exist or check and enter appropriate data. Please try again. '.$ex->getMessage();
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }
        return back();
    }

    /**
     * List of Archived Benefits
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     */
    public function archives()
    {
        $forms = Benefit::where('archived','=',true)->orderBy('updated_at','desc')->get();
        $restored = false;

        return view('benefits.index',compact('forms','restored'));
    }

    /**
     * Archiving and Restoring a Benefit method.
     * @param $benefit_id
     * @return mixed
     */
    public function archive($benefit_id)
    {
        $benefit = Benefit::find(Crypt::decrypt($benefit_id));
        return archiveBenefit($benefit);
    }

    /**
     * Show the form for creating a new Benefit.
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     * @throws \Throwable
     */
    public function create()
    {
        $page_title = 'Create';
        $header = 'Add New Program';
        $forms = ApplicationForm::where('status','=',1)->orderBy('name','asc')->get();
        $roles = Role::where('status','=',1)
            ->where('is_general','=',1)
            ->orWhere('instance_id','=',auth()->user()->last_instance_opened)
            ->orderBy('name','asc')->get();
        $offices = Office::where('status','=',1)->orderBy('name','asc')->get();

        $gender_list = LookupCode::activeOptions(config('settings.gender_code'));
        $employment_sector_list = LookupCode::activeOptions(config('settings.employment_sector_code'));
        $employment_type_list = LookupCode::activeOptions(config('settings.employment_type_code'));
        $document_type_list = LookupCode::activeOptions(config('settings.document_type_code'));
        $benefit_list = Benefit::where('archived','=',0)->pluck('titles','id');
        $impairment_type_code = LookupCode::activeOptions(config('settings.impairment_type_code'));
        $inventory_items = InventoryItem::orderBy('item')->get();

        $sub_counties = SubCounty::where([['country_id', 115], ['county_id', env('COUNTY_ID', 1104)]])->orderBy('name')->get();

        if(canDo('administrator','settings')){
            $inventory_items = InventoryItem::orderBy('item')->get();
        }else{
            $userDeploymentIds = auth()->user()->deployments->pluck('id');
            $inventory_items = InventoryItem::whereIn('deployment_id', $userDeploymentIds)->orderBy('id', 'desc')->get();
        }

        $selectedItems = [];

        

        $view = false;


        return view('benefits.create-benefit', compact('page_title','view','header',
            'gender_list','inventory_items','document_type_list','employment_sector_list','benefit_list','employment_type_list','impairment_type_code','forms','roles','offices', 'selectedItems', 'sub_counties'));
    }

    /**
     * Store a newly created Benefit in storage.
     * @param Request $request
     *
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function save(Request $request)
    {
        $this->validate($request,[
            'titles'            => 'required|string|min:2',
            'description'       => 'required|string|min:2',
            'class'              => 'required|integer|min:1',
            'type'              => 'required|integer|min:1',
            'financial'         => 'required|string|min:8',
            'cycle'    => 'required|string|min:1',
            'begin_period'    => 'required|string|min:1',
            'form'    => 'required|integer|min:1',
            'respondent'    => 'required|integer|min:1',
            // 'process_duration'    => 'required|integer|min:1',
            'visibility'    => 'required|integer|min:0|max:1',
        ]);

        //dd($request);
        if(isset($request->id)) {
            $check_benefit = Benefit::where('instance_id','=',getActiveInstance())->where('titles','=',$request->titles)->where('id','<>',Crypt::decrypt($request->id))->count();
        }
        else {
            $check_benefit = Benefit::where('instance_id','=',getActiveInstance())->where('titles','=',$request->titles)->count();
        }
        if ($check_benefit > 0) {
            alertNotify('Sorry, this benefit name already exists. Please use another name.','', 'danger');
            return back()->withInput();
        }

        $from = date('Y-m-d',strtotime($request->begin_period));
        $to = date('Y-m-d',strtotime($request->end_period));
        $check_begin_period = 0; $check_end_period = 0;

        if (isset($request->schedule)){
            $schedule = $request->schedule;
            $schedule_benefit = $request->schedule_benefit;
        }

        try{
            if ($check_begin_period > 0) {
                alertNotify(__('The start period and end period should not intersect with any other period!'), null, 'danger');
                return back();
            }
            if(isset($request->id)) {
                $benefit = Benefit::find(Crypt::decrypt($request->id));
                $benefit->updated_by = Auth::id();
            }
            else {
                $benefit = new Benefit();
                $benefit->created_by = Auth::id();
                $benefit->instance_id = getActiveInstance();
            }

            $country_name = getCountry($request->country,'country_name');
            $residences = null;
            if($request->residence){
                foreach ($request->residence as $key=>$item) {
                    //$residences[] = getState($item,'name');
                    $residences[] = getVillage($item,'name');
                }
            }

            $benefit->titles = $request->titles;
            $benefit->class = $request->class;
            $benefit->visibility = $request->visibility;
            $benefit->type = $request->type;
            $benefit->financial = $request->financial;
            $benefit->issuance_mode_id = get_issuance_mode_id($request->financial) ?? null;
            $benefit->description = $request->description;
            $benefit->cycle = $request->cycle;
            $benefit->allow_multiple = $request->allow_multiple ?? 0;
            $benefit->begin_period = $request->begin_period;
            $benefit->end_period = $request->end_period;
            $benefit->age_format = $request->age_format;
            $benefit->min_age = $request->min_age;
            $benefit->max_age = $request->max_age;
            $benefit->gender_criteria = $request->gender;
            $benefit->nationalities = $request->nationalities;
            $benefit->country_id = $request->country ?? 115;
            $benefit->country = $country_name ?? 'Kenya';
            $benefit->residences_id = $request->residence;
            $benefit->residence_criteria = $residences;
            $benefit->concurrent_benefits = $request->concurrent_benefits;
            $benefit->employment_criteria = $request->employment;
            $benefit->employment_type_criteria = $request->employment_type;
            $benefit->impairments = $request->impairments;
            $benefit->supporting_documents = $request->supporting_documents;
            $benefit->form_id = $request->form;
            $benefit->department_unit = $request->department_unit;
            $benefit->respondent = $request->respondent;
            $benefit->approver = $request->approver;
            // $benefit->process_duration = $request->process_duration;
            $benefit->save();

            PaymentSchedule::updateOrCreate([
                'benefit_id'   =>  $benefit->id,
            ],[
                'begin_period'              =>  $from,
                'end_period'              =>  $to,
                'cycle'              =>  $request->frequency,
                'user_id'           =>  Auth::id(),
            ]);

            if(!isset($request->id)){
                if(count($request->input('additional_items', [])) > 0){
                    foreach($request->input('additional_items', []) as $item){

                        BenefitInventoryItem::create([
                            'benefit_id' => $benefit->id,
                            'inventory_item_id'=> $item
                        ]);

                    }
                }
                
            }else{
                if (count($request->input('additional_items', [])) > 0) {
                    // Sync the items to the benefit
                    $benefit->inventoryItems()->sync($request->input('additional_items', []));
                }
            }


        }
        catch(\Exception $ex){
            $message = 'Sorry! Something went wrong. Please try again. '.$ex->getMessage();
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }

        $message = 'Benefit has been added successfully';
        session()->flash('status', 'success');
        session()->flash('message', $message);
        if(isset($request->id)) {
            $message = "Benefit has been updated.";
            session()->flash('status', 'success');
            session()->flash('message', $message);
        }
        return redirect()->route('benefit.list');
    }

    public function classTypes(Request $request)
    {
        $types = BenefitType::where('class','=',$request->class_id)->get();
        $view = '<option>Select</option>';
        foreach ($types as $type) {
            $view .= '<option value="'.$type->id.'">'.$type->name.'</option>';
        }
        return response()->json(['options'=>$view]);
    }

    /**
     * Show the form for editing a Benefit.
     * @param $request_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     * @throws \Throwable
     */
    public function edit($request_id)
    {
        $benefit = Benefit::find(Crypt::decrypt($request_id));
        $schedule = PaymentSchedule::where('benefit_id','=',Crypt::decrypt($request_id))->latest()->first();
        $page_title = 'Update';
        $header =$benefit->titles.' - Edit';
        // $forms = Form::where('form_type','=','registration')->orderBy('title','asc')->get();
        $forms = ApplicationForm::where('status','=',1)->orderBy('name','asc')->get();
        $roles = Role::where('status','=',1)->orderBy('name','asc')->get();
        $offices = Office::where('status','=',1)->orderBy('name','asc')->get();

        $island_list = LookupCode::activeOptions(config('settings.island_code'));
        $gender_list = LookupCode::activeOptions(config('settings.gender_code'));
        $employment_sector_list = LookupCode::activeOptions(config('settings.employment_sector_code'));
        $employment_type_list = LookupCode::activeOptions(config('settings.employment_type_code'));
        $document_type_list = LookupCode::activeOptions(config('settings.document_type_code'));
        $impairment_type_code = LookupCode::activeOptions(config('settings.impairment_type_code'));
        $benefit_list = Benefit::where('archived','=',0)->pluck('titles','id');
        $view = false;
        if(canDo('administrator','settings')){
            $inventory_items = InventoryItem::orderBy('item')->get();
        }else{
            $userDeploymentIds = auth()->user()->deployments->pluck('id');
            $inventory_items = InventoryItem::whereIn('deployment_id', $userDeploymentIds)->orderBy('id', 'desc')->get();
        }

        $selectedItems = $benefit->inventoryItems()->pluck('id')->toArray();
        $sub_counties = SubCounty::where('country_id', 115)->orderBy('name')->get();

        $verification_processes_lookup = LookupCode::where('activeFlag','=',1)->where('is_process','=',1)->orderBy('lookupName','asc')->get();
        return view('benefits.create-benefit', compact('page_title','benefit','forms',
            'verification_processes_lookup','island_list','gender_list','document_type_list','employment_sector_list',
            'benefit_list','employment_type_list','schedule','impairment_type_code','view','header','roles','offices', 'inventory_items', 'selectedItems', 'sub_counties'));
    }

    public function filterApplication(Request $request)
    {
        if ($request->filter == 'app_period' && $request->date_range == null) {
            alertNotify('Please specify the period','Data Missing','warning');
            return back()->withInput();
        }
        $filter_request['filter'] = $request->filter;
        $filter_request['date_range'] = $request->date_range;

        return view('benefit-registration.list', compact('filter_request'));
    }

    public function delete($id)
    {
        if (Benefit::destroy(Crypt::decrypt($id))) {
            alertNotify('Benefit has been deleted.',null,'success');
        }
        else {
            alertNotify('Sorry! Something went wrong. Try again.',null,'danger');
        }
        return back();
    }

    public function saveBenefitPayment(Request $request)
    {
        
        $request->validate([
            'name'=>'required',
            'beneficiary_name'=>'required',
            'payment_date'=>'required',
            'payment_mode'=>'required',
            'application_id'=>'required'
        ]);

        $application = Application::findOrFail($request->application_id);
        //dd($request);

        foreach ($request->input('issuable_item', []) as $item) {
            $quantity_approved = ApplicationItem::where([['application_id', $application->id] , ['item_id', $item['item']]])->first()->quantity_approved;
            if($item['quantity'] > $quantity_approved){
                return back()->with('error', 'You cannot issue more than approved quantity');
                exit();

            }

        }

        
        $application->status = 'Issued';
        $application->last_issuance_date = now();
        $application->next_issuance_date = $this->calculateNextIssuanceDate(now(), $application->benefit->cycle);
        $application->save();

        /*$items = ItemIssuance::where('application_id', $application->id)->get();

        foreach($items as $item){
            $item->issuance_date = now();
            $item->quantity_issued = $item->item_quantity_approved;
            $item->save();
        }*/


        foreach ($request->input('issuable_item', []) as $item) {
            $quantity_approved = ApplicationItem::where([['application_id', $application->id] , ['item_id', $item['item']]])->first()->quantity_approved ?? 0;
            $application_approval_id = ApplicationApproval::where('application_id', $application->id)->first()->id;

            ItemIssuance::create([
                'application_id'=>$application->id,
                'item_id'=>$item['item'],
                'issuance_date'=>now(),
                'quantity_issued'=>$item['quantity'],
                'item_quantity_approved' => $quantity_approved,
                'user_id'=>auth()->user()->id,
                'applcation_approval_id' => $application_approval_id ?? null ,
            ]);

        }


        BenefitPayment::create([
            'application_id'=>$request->application_id,
            'name'=>$request->name,
            'beneficiary_name'=>$request->beneficiary_name,
            'payment_date'=>$request->payment_date,
            'payment_type'=>$request->payment_mode,
            'remarks'=>$request->remarks,
            'created_by'=>auth()->user()->id

        ]);

        

        ApplicationsProcessing::create([
            'instance_id'    =>  $application->instance_id,
            'application_id'    =>  $application->id,
            'applicable_id'           =>  Auth::id(),
            'applicable_type'           =>  'App\Models\User',
            'action'            =>  'Paid',
            'access_type'            =>  'Portal',
            'stage'             =>  'Payment',
            'remarks'             =>  $request->remarks
        ]);

        

        return back()->with('success', 'Benefit Issuance Record Saved!');
    }

    public function getApplication($application_id){
        $application = Application::find($application_id);
        
        //$issuable = ItemIssuance::where('application_id', $application->id)->orderBy('id')->get();
        $issuable = ApplicationItem::where('application_id', $application->id)->get();
        return response()->json([
            'application'=>$application, 
            'applicant'=>$application->applicant->first_names.' '.$application->applicant->surnames,
            'items'=> $issuable,
            'benefit'=>$application->benefit,
        ]);
    }
    public function getBenefitPayment($application_id){
        $application = Application::findorFail($application_id);
        $payment = BenefitPayment::where('application_id', $application_id)->first();
        $issuable = ItemIssuance::where('application_id', $application->id)->orderBy('id')->get();
        return response()->json([
            'application'=>$application, 
            'applicant'=>$application->applicant->first_names.' '.$application->applicant->surnames,
            'payment'=>$payment,
            'payment_type'=>$payment->paymentType->fullName,
            'items'=> $issuable
        ]);
    }

    protected function calculateNextIssuanceDate($lastIssuanceDate, $cycle)
    {
        switch ($cycle) {
            case 'Daily':
                return $lastIssuanceDate->addDay();
            case 'Weekly':
                return $lastIssuanceDate->addWeek();
            case 'Bi-Weekly':
                return $lastIssuanceDate->addWeeks(2);
            case 'Monthly':
                return $lastIssuanceDate->addMonth();
            case 'Bi-Monthly':
                return $lastIssuanceDate->addMonths(2);
            case 'Quarterly':
                return $lastIssuanceDate->addMonths(3);
            case 'Yearly':
                return $lastIssuanceDate->addYear();
            case 'One-Off':
                // No further issuance for one-time benefits
                return null;
            default:
                throw new \Exception("Invalid cycle type: {$cycle}");
        }
    }

    public function create_application_group(Request $request)
    {
        //dd($request);
        $benefit = Benefit::find($request->benefit);
        foreach($request->input('applicants', []) as $applicant){
            $applicant_record = Applicant::find($applicant);
            $application = Application::create([
                'instance_id'=>$benefit->instance_id,
                'benefit_id'=>$benefit->id,
                'form_id'=>1,
                'type'=>$benefit->titles,
                'applicant_number' => $applicant_record->regID,
                'reporter_surname' => auth()->user()->name,
                'reporter_first_name'=>auth()->user()->firstname,
                'reporter_contact'=> auth()->user()->call_code.' '.auth()->user()->phone,
                'reporter_relationship'=>'Staff',
                'applicationable_type'=> 'App/Models/User',
                'applicationable_id'=> auth()->user()->id,
                'status'=>'Submitted',
                'modified_by'=>auth()->user()->id,
            ]);

            ApplicationsProcessing::create([
                'instance_id'    =>  $benefit->instance_id,
                'application_id'    =>  $application->id,
                'applicable_id'           =>  Auth::id(),
                'applicable_type'           =>  'App\Models\User',
                'action'            =>  'Submitted',
                'access_type'            =>  'Portal',
                'stage'             =>  'New Application',
                'remarks'             =>  'Bulk Application By'.auth()->user()->firstname.' '.auth()->user()->name
            ]);

            ApplicationGroup::create([
                'applicant_id'  => $applicant_record->id,
                'applicant_number' => $applicant_record->regID,
                'program_id'=> $request->benefit,
                'status'=> 'New',
                'created_by'=>auth()->user()->id,
                'program_name'=> $benefit->titles,
                'application_id'=>$application->id
            ]);
        }

        return redirect()->back()->with('success', 'Done');
        
    }

    public function custom_beneficiaries(Request $request)
    {
        $deployment = Deployment::find(getActiveInstance());
        $applicants = $deployment->applicants()->orderBy('surnames', 'desc');

        if(isset($request->form_type) && $request->form_type == 'basic' ){

            if($request->query_option == 'name'){

                if(isset($request->name)){
                    $applicants = $applicants->where('first_names', 'like', '%'.$request->name.'%')->orWhere('surnames', 'like', '%'.$request->name.'%')->orWhere('other_names', 'like', '%'.$request->name.'%');
                }

            }else if($request->query_option == 'id'){

                if(isset($request->basic_id_type) && isset($request->basic_id_number)){

                    $applicants = $applicants->whereHas('idDetails', function($query) use ($request) {
                        $query->where([['id_type', $request->basic_id_type], ['id_number', $request->basic_id_number]]);
                    });
                    
                }
            }

        }

        if(isset($request->form_type) && $request->form_type == 'advanced' ){

            if (isset($request->village)) {
                $applicants = $applicants->where('village_id', $request->village);
            } else {
                if(isset($request->district)){

                    $applicants = $applicants->whereHas('village', function($query) use ($request) {
                        $query->where('district_id', $request->district);
                    });

                }else{

                    if (isset($request->constituency)) {
                        $applicants = $applicants->whereHas('village', function($query) use ($request) {
                            $query->where('constituency_id', $request->constituency);
                        });
                    }

                }

                
            }

            if(isset($request->max_age) && isset($request->min_age)){

                $applicants = $applicants->whereBetween('dobs', [
                    Carbon::now()->subYears($request->max_age), 
                    Carbon::now()->subYears($request->min_age)
                ]);

            }elseif(isset($request->max_age) && !isset($request->min_age)){
    
                $applicants = $applicants->whereDate('dobs', '>', Carbon::now()->subYears($request->max_age));
            }elseif(!isset($request->max_age) && isset($request->min_age)){
                $applicants = $applicants->whereDate('dobs', '<', Carbon::now()->subYears($request->min_age));
            }

            if(isset($request->orphan_status)){
                switch ($request->orphan_status){

                    case 'maternal' :
                        $applicants = $applicants->where([['mother_alive', 0],['father_alive', 1]]);
                    break;
                    case 'paternal' :
                        $applicants = $applicants->where([['mother_alive', 1],['father_alive', 0]]);
                    break;
                    case 'double' :
                        $applicants = $applicants->where([['mother_alive', 0],['father_alive', 0]]);
                    break;
                    
                }
            }

            if(isset($request->pmt)){
    
                switch ($request->pmt) {
                    case 'Extremely Poor':
                        $applicants = $applicants->whereBetween(DB::raw('ROUND(pmtScore, 2)'), [0.00, 20.00]);
                      break;
                    case 'Poor':
                        $applicants = $applicants->whereBetween(DB::raw('ROUND(pmtScore, 2)'), [21.00, 40.00]);
                    break;
                    case 'Near Poor':
                        $applicants = $applicants->whereBetween(DB::raw('ROUND(pmtScore, 2)'), [41.00, 60.00]);
                    break;
                    case 'Vulnerable Non-Poor':
                        $applicants = $applicants->whereBetween(DB::raw('ROUND(pmtScore, 2)'), [61.00, 80.00]);
                    break;
                    case 'Economically Stable':
                        $applicants = $applicants->whereBetween(DB::raw('ROUND(pmtScore, 2)'), [81.00, 100.00]);
                    break;
    
                    default:
                    
                }
                
            }

            $genders = [];

            // Check if male checkbox is selected
            if (isset($request->gender_male)) {
                $genders[] = 'Male';
            }

            // Check if female checkbox is selected
            if (isset($request->gender_female)) {
                $genders[] = 'Female';
            }

            // Check if other checkbox is selected
            if (isset($request->gender_other)) {
                $genders[] = 'Other';
            }

            // If any genders are selected, filter applicants
            if (!empty($genders)) {
                $applicants = $applicants->whereIn('genders', $genders);
            }

            $maritals = [];
            if (isset($request->marital_single)) {
                $maritals[] = $request->marital_single;
            }
            if (isset($request->marital_married)) {
                $maritals[] = $request->marital_married;
            }
            if (isset($request->marital_mono)) {
                $maritals[] = $request->marital_mono;
            }
            if (isset($request->marital_poly)) {
                $maritals[] = $request->marital_poly;
            }
            if (isset($request->marital_divorced)) {
                $maritals[] = $request->marital_divorced;
            }
            if (isset($request->marital_separated)) {
                $maritals[] = $request->marital_separated;
            }
            if (isset($request->marital_annulled)) {
                $maritals[] = $request->marital_annulled;
            }
            if (isset($request->marital_death)) {
                $maritals[] = $request->marital_death;
            }

            if(!empty($maritals)){

                $applicants = $applicants->whereHas('maritalDetails', function($query) use ($maritals) {
                    $query->latest()->whereIn('marriage_status', $maritals);
                });

            }

            $health = [];
            if (isset($request->health_visual)) {
                $health[] = $request->health_visual;
            }
            if (isset($request->health_hearing)) {
                $health[] = $request->health_hearing;
            }
            if (isset($request->health_mobility)) {
                $health[] = $request->health_mobility;
            }
            if (isset($request->health_speech)) {
                $health[] = $request->health_speech;
            }
            if (isset($request->health_chronic)) {
                $health[] = $request->health_chronic;
            }
            if(!empty($health)){

                //there is a health_details table. one applicant can have multiple details
                
                $applicants = $applicants->whereHas('healthDetails', function($query) use ($health) {
                    $query->whereIn('health_type', $health);
                });
                
                
            }

            $education = [];
            if (isset($request->education_none)) {
                $education[] = $request->education_none;
            }
            if (isset($request->kindergaten)) {
                $education[] = $request->kindergaten;
            }
            if (isset($request->education_standard_1)) {
                $education[] = $request->education_standard_1;
            }
            if (isset($request->education_standard_2)) {
                $education[] = $request->education_standard_2;
            }
            if (isset($request->education_standard_3)) {
                $education[] = $request->education_standard_3;
            }
            if (isset($request->education_standard_4)) {
                $education[] = $request->education_standard_4;
            }
            if (isset($request->education_standard_5)) {
                $education[] = $request->education_standard_5;
            }
            if (isset($request->education_standard_6)) {
                $education[] = $request->education_standard_6;
            }
            if (isset($request->education_standard_7)) {
                $education[] = $request->education_standard_7;
            }
            if (isset($request->education_standard_8)) {
                $education[] = $request->education_standard_8;
            }
            if (isset($request->education_form_1)) {
                $education[] = $request->education_form_1;
            }
            if (isset($request->education_form_2)) {
                $education[] = $request->education_form_2;
            }
            if (isset($request->education_form_3)) {
                $education[] = $request->education_form_3;
            }
            if (isset($request->complete_post_secondary)) {
                $education[] = $request->education_complete_post_secondary;
            }

            if(!empty($education)){
                $applicants = $applicants->whereIn('highest_education_level', $education);
            }



        }

        $applicants = $applicants->paginate(20);

        $active = $deployment->applicants()->active()->count();
        $inactive = $deployment->applicants()->count() - $active;
        $pmts = Pmt::all();
        $constituencies = Constituency::where('country_id', 115)->orderBy('name')->get();

        return view('beneficiaries.index', compact('deployment', 'applicants', 'active', 'inactive', 'pmts', 'constituencies'));
    }

    public function get_constituency($constituency_id)
    {
        $constituency = Constituency::findOrFail($constituency_id);
        return response()->json(['constituency'=>$constituency, 'villages'=>$constituency->villages, 'districts'=>$constituency->districts]);
    }

    public function get_district($district_id)
    {
        $district = District::findOrFail($district_id);
        return response()->json(['district'=>$district, 'villages'=>$district->villages]);
    }
}
