<?php

namespace App\Http\Controllers;

use App\Http\Helpers\FormRender;
use App\Http\Traits\EmailSender;
use App\Http\Traits\FormSubmission;
use App\Http\Traits\VerificationProcesses;
use App\Models\Complaint;
use App\Models\ComplaintsFeedbackStage;
use App\Models\LookupCode;
use App\Models\Role;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;
use function PHPUnit\Framework\isNull;
use Yajra\DataTables\DataTables;

class ComplaintController extends Controller
{
    use FormSubmission;
    use EmailSender;
    use VerificationProcesses;

    public function __construct()
    {
        $this->middleware(['auth', 'verified']);
    }

    /**
     * Display a listing of Complaints.
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     */
    public function index()
    {
        return view('complaints.list');
    }

    /**
     * Get the list of Complaints DataTable
     * @return mixed
     * @throws \Exception
     */
    public function getComplaintsList()
    {
        $complaints = Complaint::orderBy('updated_at','asc')->get();

        return DataTables::of($complaints)
            ->addColumn('action', function ($complaint) {
                if(Auth::id() == $complaint->user_id) {
                    $edit = '<a href="'.route('complaint.editComplaint',[cryptThis($complaint->id)]).
                        '" class="btn btn-sm btn-icon btn-light-success mb-1" title="'. __("Edit Complaint") .'"><i class="la la-edit"></i></i></a>';
                    $view = '<a href="'.route('complaint.viewComplaint',[cryptThis($complaint->id)]).'" 
                        class="btn btn-sm btn-primary btn-icon mb-1" title="'. __("View Complaint") .'"><i class="la la-eye"></i></a>';
                    $delete = '<a href="'.route('complaint.deleteComplaint',[cryptThis($complaint->id)]).
                        '" onclick="return confirm(\'The complaint will be taken off the list completely . Do you want to proceed ? \')" 
                        class="btn btn-sm btn-icon btn-danger mb-1" title="'. __("Delete Complaint").'"><i class="la la-trash"></i></a>';

                    if($complaint->stage == 'Close') {
                        $edit = null; $delete = null;
                    }
                    return $edit." ".$view." ".$delete;
                }
                else {
                    return '<a href="'.route('complaint.viewComplaint',[cryptThis($complaint->id)]).'" 
                        class="btn btn-sm btn-primary mb-1 btn-icon" title="'. __("View Complaint") .'"><i class="la la-eye"></i></a>';
                }

            })
            ->editColumn('subjects', function ($complaint) {
                return $complaint->subjects;
            })
            ->editColumn('user_id', function ($complaint) {
                return $complaint->owner->name;
            })
            ->editColumn('stage_lookup_id', function ($complaint) {
                return getComplaintLabel($complaint->stage);
//                if ($complaint->status) {
//                    return getVerificationLabel($complaint->stage_lookup_id);
//                }
//                else {
//                    return '<span class="label label-light-info label-inline">'.__("Not Assigned").'</span>';
//                }
            })
            ->editColumn('created_at', function ($complaint) {
                return date('d M, Y',strtotime($complaint->created_at));
            })
            ->escapeColumns([])
            ->make();
    }


    /**
     * Create Complaint information
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function createComplaint()
    {
        $get_render_form = new FormRender();
        $get_render_form->submitUrl(route('complaint.saveComplaint'));
        $render_form = $get_render_form->renderFormView('complaints');
        $page_title = __('Create Complaint');
        return view('complaints.edit', compact('render_form','page_title'));
    }

    /**
     * Edit Complaint
     * @param $complaint_id
     *
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function editComplaint($complaint_id)
    {
        $get_render_form = new FormRender();
        $get_render_form->request_id($complaint_id);
        $get_render_form->submitUrl(route('complaint.saveComplaint'));
        $render_form = $get_render_form->renderFormView('complaints');
        $page_title = __('Edit Complaint');
        return view('complaints.edit', compact('render_form','page_title'));
    }

    /**
     * Save Complaint information
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     */
    public function saveComplaint(Request $request)
    {
        $complaint_id = $this->submit($request);

        $fmessage = "The Complaint has been updated.";
        $type = "success";
        if ($complaint_id) {
            if ($request->request_id == null) {
                $complaint = Complaint::find(Crypt::decrypt($complaint_id));
                $complaint->user_id = Auth::id();
                $complaint->status = 1;
                $complaint->stage = 'Opened';
//                $complaint->stage_lookup_id = getVerificationStartIDByLookupCode(config('settings.complaint_process_lookupCode'));

                if ($complaint->save()) {
                    $message = "Dear Sir/Madam, <br><br>";
                    $emails = getUsersEmailAndNameByRoles([config('settings.admin_role_id')]);
                    $message .= __("A new complaint has been reported. Please assign the complaint appropriate body.");
                    $message .= "<br><br>";
                    $message .= "Subject: <br><strong>".$complaint->subjects."</strong><br><br>";
                    $message .= "Description: <br>";
                    $message .= $complaint->reasons;
                    $message .= "<br><br> Thank you.<br><br>Best Regards,";
                    $subject = "A New Complaint Reported";

                    $this->blast_mails($message,$subject,$emails);

                    $fmessage = "The Complaint has been created.";
                    $type = "success";
                }
            }
            alertNotify($fmessage,null,$type);
        }
        return redirect()->route('complaint.list');
    }

    /**
     * @param Request $request
     * @param $complaint_id
     *
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function assignStage(Request $request,$complaint_id)
    {
        $this->validate($request, [
            'assign_to' => 'required|string|min:4',
        ]);

        $complaint = Complaint::find(Crypt::decrypt($complaint_id));
        $complaint->stage = "Pending Review";

        $message = "Dear All <br><br>";
        if ($request->assign_to == 'roles') {
            $complaint->role_ids = $request->role_ids;
            $complaint->user_ids = null;
            $emails = getUsersEmailAndNameByRoles($request->role_ids);
        }
        else {
            $complaint->role_ids = null;
            $complaint->user_ids = $request->user_ids;
            $emails = getUsersEmailAndNameByUsers($request->user_ids);
            if (sizeof($request->user_ids) == 1) {
                $message = "Dear Sir/Madam <br><br>";
            }
        }

        if ($complaint->save()) {
            $message .= __("A complaint has been opened for your support. The details are below.");
            $message .= "<br><br>";
            $message .= "Subject: <br><strong>".$complaint->subjects."</strong><br><br>";
            $message .= "Description: <br>";
            $message .= $complaint->reasons;
            $message .= "<br><br> Thank you.<br><br>Best Regards,";
            $subject = "A New Complaint has been assigned";

            $this->blast_mails($message,$subject,$emails);

            $message = __("The Complaint has been assigned.");
            $type = "success";
            alertNotify($message,null,$type);
            return redirect()->route('complaint.list');
        }
        else {
            $message = __("Sorry! Something went wrong. Please try again.");
            $type = "danger";
            alertNotify($message,null,$type);
            return back();
        }
    }

    public function viewComplaint($complaint_id)
    {
        $display_responses = new FormRender();
        $display_responses->request_id($complaint_id);
        $display_responses->displayAsDetails(true);
        $render_form = $display_responses->renderFormView('complaints');
        $page_title = __('View Complaint');

        $roles = Role::all();
        $users = User::where('email_verified_at','!=', null)->where('deleted_at','=', null)->get();

        $complaint = Complaint::find(Crypt::decrypt($complaint_id));
        $feedbacks = $complaint->feedbacks->sortByDesc('created_at');

        $isEditor = false;
        $canRespond = false;
        if (!is_null($complaint->role_ids)) {
            $user = isUserEligibleByRole($complaint->role_ids,Auth::id());
            if ($user->count() > 0) {
                $isEditor = true;
                $canRespond = true;
            }
        }
        elseif (!is_null($complaint->user_ids)) {
            if (in_array(Auth::id(),$complaint->user_ids)) {
                $isEditor = true;
                $canRespond = true;
            }
        }
        if ($complaint->user_id == Auth::id() || Auth::user()->hasRole('administrator')) {
            $canRespond = true;
        }

        $stages = null;
        $current_stage = $complaint->stage;
        if (Auth::user()->hasRole('administrator')) {
            $stages = config('settings.feedback_stage');
        }
        elseif ($isEditor) {
            $stages = config('settings.feedback_stage_for_assigned');
        }
        elseif (Auth::id() == $complaint->user_id) {
            if($current_stage == "Resolved") {
                $current_stage = 'Close';
            }
            $stages = config('settings.feedback_stage_for_owner');
        }

        return view('complaints.view',compact('render_form','page_title','complaint','feedbacks',
            'stages','roles','users','isEditor','current_stage','canRespond'));
    }

    public function saveResponse(Request $request, $complaint_id)
    {
        $complaint = Complaint::find(Crypt::decrypt($complaint_id));
        $complaint->stage = $request->stage;
        $complaint->save();

        $feedback = new ComplaintsFeedbackStage();
        $feedback->complaint_id = Crypt::decrypt($complaint_id);
        $feedback->responder = Auth::id();
        $feedback->stage = $request->stage;
        $feedback->feedback = $request->response;

        if ($feedback->save()) {
            if($request->stage == 'Resolved') {
                $message = "Dear Sir/Madam <br><br>";
                $message .= __("The Complaint with Subject,");
                $message .= " <strong>".$complaint->subjects."</strong>, has been marked as RESOLVED. Please review and CLOSE the complaint if issue has been addressed.";
                $message .= "<br><br> Thank you.<br><br>Best Regards,";
                $subject = "A Complaint has been Resolved";

                $cc = getUsersEmailAndNameByRoles([config('settings.admin_role_id')]);
                $emails = getUsersEmailAndNameByUsers([Auth::id()]);
                $this->blast_mails($message,$subject,$emails,$cc);
            }
            if($request->stage == 'Pending Review' && Auth::id() != $complaint->user_id) {
                $message = "Dear Sir/Madam <br><br>";
                $message .= __("A new feedback has been added to your Complaint with Subject,");
                $message .= " <strong>".$complaint->subjects."</strong>.";
                $message .= " <br><br> Feedback: <br>".$request->response;
                $message .= "<br><br> Thank you.<br><br>Best Regards,";
                $subject = "New Complaint Feedback";

                $emails = getUsersEmailAndNameByUsers([Auth::id()]);
                $this->blast_mails($message,$subject,$emails);
            }
            $message = "The Response has been saved.";
            $type = "success";
        }
        else {
            $message = "Sorry! Response wasn't saved. Please try again.";
            $type = "danger";
        }
        alertNotify($message,null,$type);
        return back();
    }

    /**
     * Delete a Complaint
     * @param $complaint_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function deleteComplaint($complaint_id)
    {
        if (Complaint::destroy(Crypt::decrypt($complaint_id))) {
            alertNotify('Complaint has been deleted.',null,'success');
        }
        else {
            alertNotify('Sorry! Something went wrong. Try again.',null,'danger');
        }
        return back();
    }

}
