<?php

namespace App\Http\Controllers;

use App\Http\Traits\MembershipTrait;
use App\Models\Applicant;
use App\Models\ContactDetail;
use App\Models\HouseHold;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Str;
use Yajra\DataTables\Facades\DataTables;

class HouseHoldManagementController extends Controller
{
    use MembershipTrait;
    /**
     * Get Household Data
     * @param Request $request
     *
     * @return string
     */
    public function index(Request $request)
    {
        $page_title = "Household Register";
        $household = $type = $location = $tenure = $area = $query = 0;

        if (isset($request->type)) { $type = $request->type; $query = 1;}
        if (isset($request->area)) { $area = $request->area; $query = 1;}
        if (isset($request->tenure)) { $tenure = $request->tenure; $query = 1;}
        if (isset($request->location)) { $location = $request->location; $query = 1;}
        return view('household.index', compact('page_title','query','area','type','location','tenure'));
    }

    /**
     * Create Household
     *
     * @return string
     */
    public function create(Request $request)
    {
        $request->validate([
            'head'  =>  'nullable|string',
            'type'  =>  'required',
            'area'  =>  'required',
            'household_count'  =>  'required|numeric|min:1',
            'tenure'  =>  'required',
            'location'  =>  'required',
            'status'  =>  'required',
        ]);

        if(isset($request->head)) {
            //Check if Applicant Number is valid. If not return back with a response.
            $isApplicantValid = Applicant::active()->where('regID','=',$request->head)->first();
            if(!$isApplicantValid) {
                $message = "The Applicant Number is invalid";
                session()->flash('status', 'error');
                session()->flash('message', $message);
                return back()->withInput();
            }

            //Check if Applicant Number already belongs to a household.
            if(!is_null($request->head)) {
                $belongsToHousehold = HouseHold::where('head','=',$request->head)->count();
                if($belongsToHousehold > 0) {
                    $message = "Sorry! The Head already belongs to a household.";
                    session()->flash('status', 'error');
                    session()->flash('message', $message);
                    return back()->withInput();
                }
            }
        }

        $household_last_id = HouseHold::orderBy('created_at', 'DESC')->pluck('id')->first();
        if (is_null($household_last_id)) $household_last_id = 1;

        $reference_id = generateHouseholdCode($household_last_id,'AA');
        if(HouseHold::where('reference_id','=',$reference_id)->count() > 0) {
            $reference_id = generateHouseholdCode($household_last_id+1,'AA');
        }

        $household = HouseHold::create([
            'reference_id'  =>  $reference_id,
            'instance_id'  =>  getActiveInstance(),
            'head'  =>  $request->head,
            'type'  =>  $request->type,
            'area'  =>  $request->area,
            'tenure'  =>  $request->tenure,
            'location'  =>  $request->location,
            'household_count'  =>  $request->household_count,
            'status'  =>  $request->status,
            'created_by'  =>  auth()->id(),
        ]);

        if($household) {
            $message = "A new Household has been added.";
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return back();
        }
        else {
            $message = "Sorry! Something went wrong. Please try again.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }
    }

    public function edit(Request $request) {
        $reference_id = $request->reference_id;
        $household = HouseHold::where('reference_id','=',$reference_id)->first();
        // $household = HouseHold::find($reference_id);
        $view = view('household.edit-view', compact(['household']))->render();
        return response()->json(['theView'=>$view]);
    }

    public function validateApplicant(Request $request) {
        $applicant_number = $request->applicant_number;
        $applicant = Applicant::where('regID','=',$applicant_number)->first();
        $passed = 0;
        if(!$applicant) {
            $message = "<p class='text-danger'>The Applicant Number is invalid</p>";
        }
        else {
            $message = "<p class='text-success'><span class='fw-bold'>Applicant Name:</span> ".$applicant->fullName."</p>";
            $message .= "<p class='text-success'><span class='fw-bold'>Gender:</span> ".$applicant->genders."</p>";
            $message .= "<p class='text-success'><span class='fw-bold'>Date of Birth:</span>  ".$applicant->dobs."</p>";
            $passed = 1;
        }

        return response()->json(['message'=>$message,'passed'=>$passed]);
    }

    /**
     * Update Household
     *
     * @return string
     */
    public function update(Request $request)
    {
        $request->validate([
            'head'  =>  'nullable|string',
            'type'  =>  'required',
            'area'  =>  'required',
            'household_count'  =>  'required|numeric|min:1',
            'tenure'  =>  'required',
            'location'  =>  'required',
            'status'  =>  'required',
        ]);

        if(isset($request->head)) {
            //Check if Applicant Number is valid. If not return back with a response.
            $isApplicantValid = Applicant::active()->where('regID','=',$request->head)->first();
            if(!$isApplicantValid) {
                $message = "The Applicant Number is invalid";
                session()->flash('status', 'error');
                session()->flash('message', $message);
                return back()->withInput();
            }

            //Check if Applicant Number already belongs to a household.
            if(!is_null($request->head)) {
                $belongsToHousehold = HouseHold::where('head','=',$request->head)->where('reference_id','!=',$request->reference_id)->count();
                if($belongsToHousehold > 0) {
                    $message = "Sorry! The Head already belongs to a household.";
                    session()->flash('status', 'error');
                    session()->flash('message', $message);
                    return back()->withInput();
                }
            }
        }

        $household = HouseHold::where('reference_id','=', $request->reference_id)->update([
            'head'  =>  $request->applicant_number,
            'type'  =>  $request->type,
            'area'  =>  $request->area,
            'tenure'  =>  $request->tenure,
            'location'  =>  $request->location,
            'household_count'  =>  $request->household_count,
            'status'  =>  $request->status,
            'updated_by'  =>  auth()->id(),
        ]);

        if($household) {
            $message = "Household has been updated.";
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return back();
        }
        else {
            $message = "Sorry! Something went wrong. Please try again.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }
    }

    public function view($id) {

        $household = HouseHold::with(['_head','_type','_tenure','_area','_location',
        'members'=>['applicant']
        ])
        ->where('id','=', Crypt::decrypt($id))->first();
        if (!$household) {
            $message = "The Household is invalid";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back();
        }

        return view('household.view', compact('household'));
    }

    public function delete($reference_id) {

    }

    // public function setNewMember($applicant_number,$household_reference_id) {
    //     $record = new ContactDetail();
    //     $record->applicant_number = $applicant_number;
    //     $record->type = config('settings.contact_type.household');
    //     $record->type_name = getLookupSelectedOptionTextByValue(config('settings.contact_type.household'));
    //     $record->household_number = $household_reference_id;
    //     $record->is_household_head = 0;
    //     $record->status = 'Verified';
    //     $record->addedBy = auth()->id();

    //     if($record->save()) {
    //         return true;
    //     }
    //     else {
    //         return false;
    //     }
    // }

    public function addMember($id) {
        $household = HouseHold::findOrFail(Crypt::decrypt($id));
        if (!$household) {
            $message = "The Household is invalid";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back();
        }
        $applicant = null;
        $applicant_docs = null;
        $pending_approval = false;
        $is_new = true;

        // $islands = Lookup::where('lookup_code_id','=', config('settings.island_code'))->pluck('fullName','short_code')->toArray();
        $countries = DB::table('countries')->orderBy('country_name')->get();
        // $place_of_birth = $islands;
        $nationalities = null;

        return view('household.add-member', compact('household','is_new','nationalities','applicant','applicant_docs','countries'));
    }

    public function saveMember(Request $request) {
        $household_id = Crypt::decrypt($request->household_id);

        $household = HouseHold::findOrFail($household_id);
        if (!$household) {
            $message = "Invalid Household";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back();
        }

        $applicant = Applicant::where('regID','=',$request->applicant_number)->first();
        if (!$applicant) {
            $message = "Invalid Applicant Number";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back();
        }

        if(setNewMember($request->applicant_number,$household->reference_id)) {
            $message = "Member has been added successfully.";
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return back();
        }
        else {
            $message = "Sorry! Information was not saved. Please try again.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back();
        }

    }

    public function getHouseholdList($type=null,$area=null,$tenure=null,$location=null)
    {
        $households = HouseHold::with(['_head','_type','_tenure','_area','_location','members'])
            ->where('instance_id','=',getActiveInstance())
            ->orderBy('created_at','desc');
        $query = false;
        if ($type > 0) {
            $households = $households->where('type','=',$type);
        }

        if ($area > 0) {
            $households = $households->where('area','=',$area);
        }

        if ($tenure > 0) {
            $households = $households->where('tenure','=',$tenure);
        }

        if ($location > 0) {
            $households = $households->where('location','=',$location);
        }

        $households = $households->get();

        return DataTables::of($households)
            ->addColumn('action', function ($household) {
                $view = '<a href="'.route('household.view',[cryptThis($household->id)]).'"
                class="btn btn-sm btn-link text-info mr-2" href="javascript:;" data-bs-toggle="tooltip" data-bs-placement="top" title="'. __("View Household") .'"><i class="fa fa-folder-open"></i></a>';

                $edit = '<button type="button" class="btn btn-sm btn-link text-primary mr-2" onclick="editHousehold(\''.makeString($household->reference_id).'\')" data-bs-toggle="modal" data-bs-target="#editHousehold" data-bs-placement="top" title="'. __("Edit Household") .'"><i class="fa fa-edit"></i></button>';

                // if($household->status == 0) {
                //     $archive = '<a href="'.route('household.delete',[cryptThis($household->id)]).
                //         '" onclick="return confirm(\'Are you sure you want to activate this Household?\')"
                //         class="btn btn-sm btn-link text-success mr-2" href="javascript:;" data-bs-toggle="tooltip" data-bs-placement="top" title="'. __("Restore Household").'"><i class="fa fa-play"></i></a>';

                // }
                // else {

                //     $archive = '<a href="'.route('household.delete',[cryptThis($household->id)]).
                //         '" onclick="return confirm(\'Are you sure you want to disble this Household?\')"
                //         class="btn btn-sm btn-link text-danger mr-2" href="javascript:;" data-bs-toggle="tooltip" data-bs-placement="top" title="'. __("Delete Household").'"><i class=" fa fa-pause"></i></a>';
                // }

                return $view. $edit;

            })
            ->addColumn('members', function ($household) {
                return date('d M Y', strtotime($household->created_at)).'<br><small>'.$household->members->count().'/' . $household->household_count.' Members</small>';
            })
            ->editColumn('reference_id', function ($household) {
                if(isset($household->_type)) {
                    $type = $household->_type->fullName.', ';
                }
                else {
                    $type = "<span class='text-danger italic'>(Not Set)</span>";
                }
                if(isset($household->_head)) {
                    $household_headname = ' Headed by '.$household->_head->fullName;
                }
                else {
                    $household_headname =  "<span class='text-danger italic'>No Household Head Set</span>";
                }

                return $household->reference_id.'<br><small class="text-muted">'.$type.$household_headname.'</small>';;
            })
            ->editColumn('id', function ($household) {

                return $household->id;
            })
            ->editColumn('head', function ($household) {
                if(isset($household->_head)) {
                    return $household->_head->fullName;
                }
                else {
                    return " - ";
                }

            })
            ->editColumn('type', function ($household) {
                if(isset($household->_type)) {
                    return $household->_type->fullName;
                }
                else {
                    return " - ";
                }
            })
            ->editColumn('area', function ($household) {
                if(isset($household->_area)) {
                    return $household->_area->fullName;
                }
                else {
                    return "<span class='text-danger italic'>(Not Set)</span>";
                }
            })
            ->editColumn('tenure', function ($household) {
                if(isset($household->_area)) {
                    $area = $household->_area->fullName.', ';
                }
                else {
                    $area = "<span class='text-danger italic'>(Not Set)</span>";
                }

                if(isset($household->_location)) {
                    $location = $household->_location->fullName;
                }
                else {
                    $location = "<span class='text-danger italic'>(Not Set)</span>";
                }

                if(isset($household->_tenure)) {
                    $tenure = $household->_tenure->fullName;
                }
                else {
                    $tenure = "<span class='text-danger italic'>(Not Set)</span>";
                }

                return $tenure.'<br><small class="text-muted">'.$area.'in '.$location.'</small>';

            })
            ->editColumn('location', function ($household) {
                if(isset($household->_location)) {
                    return $household->_location->fullName;
                }
                else {
                    return " - ";
                }
            })
            ->escapeColumns([])
            ->make();
    }
}
