<?php

namespace App\Http\Controllers;
use App\Http\Middleware\LoadUserPermissions;
use App\Models\AccountDetail;
use App\Models\ApplicationApproval;
use App\Models\Bank;
use App\Models\BankBranch;
use App\Models\Deployment;
use App\Models\Lookup;
use App\Models\Merchant;
use App\Models\MerchantBranch;
use App\Models\Role;
use App\Models\User;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\QueryException;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Laratrust\Models\LaratrustRole;
use Illuminate\Http\Request;
use PhpOffice\PhpSpreadsheet\Cell\Coordinate;
use PhpOffice\PhpSpreadsheet\IOFactory;

class MerchantController extends Controller
{
    public function __construct()
    {
        $this->middleware(['auth:web']);
    }

    public function index(Request $request){
        $deployment = getActiveDeployment();
        $memberships = $deployment->applications()->where('status','=','Approved')->pluck('bank_account_id')->toArray();
        $accounts = AccountDetail::whereIn('id',$memberships);
        if ($request->type) {
            $accounts = $accounts->where('account_type','=',$request->type);
        }
        if ($request->country) {
            $accounts = $accounts->where('country_id','=',$request->country);
        }
        if ($request->bank) {
            $accounts = $accounts->where('bank_name','like',$request->bank.'%');
        }

        $accounts = $accounts->distinct()->pluck('bank_id')->toArray();
        $bank_members = AccountDetail::whereIn('id',$memberships)->selectRaw('count(regID) as members, bank_id')
            ->groupBy('bank_id')->pluck('members','bank_id')->toArray();

        if (is_null($accounts) || sizeof($accounts) < 1) $merchants = [];
        else $merchants = getMerchants($accounts);

        return view('merchant.index', compact('merchants','bank_members'));
    }

    public function merchantDetails(Request $request,$merchant_id)
    {
        $merchant = getMerchant(Crypt::decrypt($merchant_id));
        $applicants = DB::table('applicants')
            ->join('applications','applications.applicant_number','=','applicants.regID')
            ->join('benefits','applications.benefit_id','=','benefits.id')
            ->join('account_details','account_details.id','=','applications.bank_account_id')
            ->join('application_approvals','applications.id','=','application_approvals.application_id')
            ->selectRaw('applicants.id,applicants.regID as applicant_number,applicants.surnames,applicants.first_names,
            applicants.other_names,benefits.titles as benefit_title,benefits.id as benefit_id,account_details.bank_branch,
            account_details.branch_id,application_approvals.amount')
            ->where('account_details.bank_id','=',Crypt::decrypt($merchant_id));

        $raw_applicants = $applicants->get();

        if ($request->beneficiary) {
            $applicants = $applicants->where('surnames','like',$request->beneficiary.'%')
                ->orWhere('first_names','like',$request->beneficiary.'%')
                ->orWhere('other_names','like',$request->beneficiary.'%');
        }
        if ($request->beneficiary) {
            $applicants = $applicants->where('benefit_id','=',$request->benefit);
        }
        if ($request->branch) {
            $applicants = $applicants->where('branch_id','=',(int)$request->branch);
        }
//        if ($request->status) {
//
//        }
        $applicants = $applicants->get();
        $working_branches = $raw_applicants->pluck('branch_id')->toArray();
        $working_benefits = $raw_applicants->pluck('benefit_id')->toArray();
        return view('merchant.view-page', compact('merchant','applicants','working_branches','working_benefits'));
    }

    public function merchantBranchDetails($merchant_branch_id)
    {
        $merchant = BankBranch::findOrFail(Crypt::decrypt($merchant_branch_id));
        return view('merchant.view-page', compact('merchant'));
    }


    /**
     * Function for upload image
     *
     * @param  string  $folder
     * @param  string  $key
     * @param  string  $validation
     *
     * @return false|string|null
     */
    public function uploadProfile($key = 'logo', $validation = 'image|mimes:jpeg,png,jpg,gif,svg|max:2048|sometimes')
    {
        $folder = 'banks';
        request()->validate([$key => $validation]);

        $file = null;
        if (request()->hasFile($key)) {
            $file = Storage::disk('public_uploads')->putFile($folder, request()->file($key), 'public');
        }

        return $file;
    }

    public function emptyForm(){
        $types = getBankTypes();
        $locations = getLocations();
        $leads = Role::all();
        $users = User::all();
        $theView = view('merchant.empty_form', compact('types','locations','leads','users'));
        return $theView;
    }

    public function create(Request $request){
       $this->validate($request,[
            'name' => 'required|string|min:2|unique:banks,name',
            'shortname' => 'required|string|min:2|unique:banks,shortname',
           'type' => 'required|integer',
           'country' => 'required|integer',
           'swift_code' => 'nullable|integer',
           'contact' => 'nullable|string',
           'address' => 'nullable|string',
           'email' => 'nullable|email',
           'website' => 'nullable|string',
           'logo' => 'nullable|file',
        ]);

        try{
            $bank = new Bank();
            $bank->name   = $request->name;
            $bank->shortname  = $request->shortname;
            $bank->type = $request->type;
            $bank->country_id = $request->country;
            $bank->swift_code = $request->swift_code;
            $bank->email    = $request->email;
            $bank->website = $request->website;
            $bank->contact    = $request->contact;
            $bank->address = $request->address;
            $bank->created_by  = Auth::id();

            if($request->file('logo')) {
                $file = $this->uploadProfile();
                $bank->logo = "app-uploads/$file";
            }

            $bank->save();

            $message = 'A New Bank has been added!';
            session()->flash('message', $message);
            session()->flash('status', 'success');

        }catch(\Illuminate\Database\QueryException $ex){
            $message = 'Bank may already exist or check enter appropriate data!';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        return back();
    }

    public function branchesUpload(Request $request)
    {
        $this->validate($request,[
            'merchant' => 'required|numeric',
            'uploadFile' => 'required|file|mimes:xlsx',
        ]);

        if ($request->file('uploadFile')->isValid()) {

            //Extracting data from sheet
            $path = $request->file('uploadFile')->getRealPath();

            $reader = IOFactory::createReader('Xlsx');
            $reader->setReadDataOnly(true);
            $spreadsheet = $reader->load($path);
            $worksheet = $spreadsheet->getActiveSheet();

            // Get the highest row and column numbers referenced in the worksheet
            $highestRow = $worksheet->getHighestRow(); // e.g. 10
            $highestRow = (integer)$highestRow; // e.g. 10

            // $highestColumn = 'E'; // e.g 'F'
            $highestColumn = $worksheet->getHighestColumn(); // e.g 'F'

            $highestColumnIndex = Coordinate::columnIndexFromString($highestColumn); // e.g. 5
            $highestColumnIndex = (integer)$highestColumnIndex; // e.g. 5

            $insertData =[];
            $count = 0;
            $column_headers = config('settings.branches_upload_headers');

            for ($row=2; $row <= $highestRow; $row++) {
                $insertData[$count]['bank_id'] = $request->merchant;
                $insertData[$count]['status'] = 1;
                $insertData[$count]['created_by'] = auth()->id();
                //Get the various DLRs column coordinates
                for ($col = 1; $col <= $highestColumnIndex; $col++) {
                    if(isset($column_headers[$col-1])) {

                        if($column_headers[$col-1] == 'location_id') {

                            $location = null;
                            if ($spreadsheet->getActiveSheet()->getCell([$col, $row])->getValue() != '') {
                                $region = str_replace("Region","",$spreadsheet->getActiveSheet()->getCell([$col, $row])->getValue());
                                $location = Lookup::where('lookup_code_id','=',9)->where('fullName','like',"%$region%")->pluck('id')->first();
                            }

                            $insertData[$count][$column_headers[$col-1]] = $location;
                        }
                        else {
                            $insertData[$count][$column_headers[$col-1]] = $spreadsheet->getActiveSheet()->getCell([$col, $row])->getValue();
                        }

                    }
                }
                $count += 1;
            }
            //End
        }

//        foreach($insertData as $data) {
//            BankBranch::updateOrCreate([
//                'bank_id'   => $data['bank_id'],
//            ]);
//        }
        if (BankBranch::upsert(
            $insertData,
            ['bank_id','routing_number'],
            ['name','email','contact','status','location_id','address','created_by']
        ))
        {
            $message = 'Branches have been uploaded successfully';
            session()->flash('message', $message);
            session()->flash('status', 'success');
        }

        return back();
    }

    public function edit(Request $request){
        $bank = Bank::find($request->bank_id);
        $types = getBankTypes();

        $theView = view('merchant.edit', compact('bank','types'))->render();
        return response()->json(['theView'=>$theView]);
    }

    public function update(Request $request)
    {
        $this->validate($request,[
            'name' => 'required|string|min:2|unique:banks,name,'.$request->id,
            'shortname' => 'required|string|min:2|unique:banks,shortname,'.$request->id,
            'type' => 'required|integer',
            'country' => 'required|string',
            'contact' => 'nullable|string',
            'swift_code' => 'nullable|string',
            'address' => 'nullable|string',
            'email' => 'nullable|email',
            'website' => 'nullable|string',
            'logo' => 'nullable|file',
        ]);

        try {
            $bank = Bank::find($request->id);
            $bank->name   = $request->name;
            $bank->shortname  = $request->shortname;
            $bank->type = $request->type;
            $bank->country_id = $request->country;
            $bank->swift_code = $request->swift_code;
            $bank->email    = $request->email;
            $bank->website = $request->website;
            $bank->contact    = $request->contact;
            $bank->address = $request->address;
            $bank->updated_by  = Auth::id();

            if($request->file('logo')) {

                $file = $this->uploadProfile();
                $bank->logo = "app-uploads/$file";
            }

            $bank->save();

            $message = 'Bank information has been updated!';
            session()->flash('message', $message);
            session()->flash('status', 'success');

            return back();
        }catch (QueryException $exception){
            $message = 'The Bank name may already exist or check enter appropriate data!'.$exception->getMessage();
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }
    }

    public function remove($id)
    {
        try {
            $bank = Crypt::decrypt($id);
            Bank::destroy($bank);

            $message = 'Bank has been removed!';
            session()->flash('message', $message);
            session()->flash('status', 'success');
            return back();
        }catch (QueryException $exception){
            $message = 'Something went wrong. Try again or contact Administrator.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }
    }

    public function activation($id)
    {
        try {
            $bank = Bank::find(Crypt::decrypt($id));
            if (!is_null($bank)) {
                if ($bank->status == 1){
                    $bank->status = 0;
                    $message = 'The Bank has been deactivated!';
                }
                else {
                    $bank->status = 1;
                    $message = 'The Bank has been activated!';
                }
                $bank->save();

                session()->flash('message', $message);
                session()->flash('status', 'success');
            }
            else {
                $message = 'Bank does not exist.';
                session()->flash('message', $message);
                session()->flash('status', 'error');
            }

            return back();
        }catch (QueryException $exception){
            $message = 'Something went wrong. Try again or contact Administrator.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }
    }

    //BRANCHES
    public function branchIndex(Request $request){

        $banks = Bank::where('status','=',1)->orderBy('name')->get();
        $branches = BankBranch::orderBy('name');

        if (isset($request->name)) {
            $branches = $branches->where('name','like','%'.$request->name.'%')->orWhere('routing_number','like','%'.$request->name.'%');
        }
        if (isset($request->bank)) {
            $branches = $branches->where('bank_id','=',$request->bank);
        }
        if (isset($request->location)) {
            $branches = $branches->where('location_id','=',$request->location);
        }
        $branches = $branches->get();
        return view('merchant.branch-index', compact('banks','branches'));
    }

    public function branchEmptyForm(){

        $banks = Bank::where('status','=',1)->orderBy('name')->get();
        $theView = view('merchant.empty-branch-form', compact('banks'));
        return $theView;
    }

    public function branchCreate(Request $request){
        $this->validate($request,[
            'name' => 'required|string|min:2',
            'routing_number' => 'required|string|min:2',
            'bank' => 'required|integer',
            'services' => 'required|string',
            'contact' => 'required|string',
            'address' => 'required|string',
            'email' => 'required|email',
            'location' => 'required|integer',
        ]);

        try{
            $bank = new BankBranch();
            $bank->name   = $request->name;
            $bank->routing_number  = $request->routing_number;
            $bank->bank_id = $request->bank;
            $bank->email    = $request->email;
            $bank->services = $request->services;
            $bank->location_id = $request->location;
            $bank->contact    = $request->contact;
            $bank->address = $request->address;
            $bank->created_by  = Auth::id();
            $bank->save();

            $message = 'A New Branch has been added!';
            session()->flash('message', $message);
            session()->flash('status', 'success');

        }catch(\Illuminate\Database\QueryException $ex){
            $message = 'Sorry! Something went wrong. Please try again. ';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        return back();
    }

    public function branchEdit(Request $request){
        $branch = BankBranch::find($request->id);
        $banks = Bank::where('status','=',1)->orderBy('name')->get();

        $theView = view('merchant.edit-branch', compact('banks','branch'))->render();
        return response()->json(['theView'=>$theView]);
    }

    public function branchUpdate(Request $request)
    {
        $this->validate($request,[
            'name' => 'required|string|min:2',
            'routing_number' => 'required|string|min:2',
            'bank' => 'required|integer',
            'contact' => 'required|string',
            'services' => 'required|string',
            'address' => 'required|string',
            'email' => 'required|email',
            'location' => 'required|integer',
        ]);

        try {
            $bank = BankBranch::find($request->id);
            $bank->name   = $request->name;
            $bank->routing_number  = $request->routing_number;
            $bank->bank_id = $request->bank;
            $bank->email    = $request->email;
            $bank->services    = $request->services;
            $bank->location_id = $request->location;
            $bank->contact    = $request->contact;
            $bank->address = $request->address;
            $bank->updated_by  = Auth::id();

            $bank->save();

            $message = 'Branch information has been updated!';
            session()->flash('message', $message);
            session()->flash('status', 'success');

            return back();
        }catch (QueryException $exception){
            $message = 'Sorry! Something went wrong. Please try again. ';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }
    }

    public function branchRemove($id)
    {
        try {
            $branch = Crypt::decrypt($id);
            BankBranch::destroy($branch);

            $message = 'Branch has been removed!';
            session()->flash('message', $message);
            session()->flash('status', 'success');
            return back();
        }catch (QueryException $exception){
            $message = 'Something went wrong. Try again or contact Administrator.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }
    }

    public function branchActivation($id)
    {
        try {
            $branch = BankBranch::find(Crypt::decrypt($id));
            if (!is_null($branch)) {
                if ($branch->status == 1){
                    $branch->status = 0;
                    $message = 'The Branch has been deactivated!';
                }
                else {
                    $branch->status = 1;
                    $message = 'The Branch has been activated!';
                }
                $branch->save();

                session()->flash('message', $message);
                session()->flash('status', 'success');
            }
            else {
                $message = 'Bank does not exist.';
                session()->flash('message', $message);
                session()->flash('status', 'error');
            }

            return back();
        }catch (QueryException $exception){
            $message = 'Something went wrong. Try again or contact Administrator.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }
    }

    public function getBanksByType($merchant_type)
    {
        $banks = Merchant::where('type', $merchant_type)->orderBy('name', 'asc')->get();
        return response()->json($banks);
    }

    public function getBankBranchesByMerchantId($merchant_id){
        $merchant = Merchant::findOrFail($merchant_id);
        $branches = MerchantBranch::where('merchant_id', $merchant->id)->orderBy('name', 'asc')->get();
        return response()->json($branches);
    }
}

