<?php

namespace App\Http\Controllers;

use App\Models\User;
use App\Models\UserProfile;
use Illuminate\Auth\Events\PasswordReset;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Password;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Illuminate\Validation\Rules;

class ProfileAccountController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    public function index()
    {
        $company = null;
        $website = null;
        $avatar = null;
        $contact = null;
        if (Auth::user()->profile) {
            $company = Auth::user()->profile->company_name;
            $website = Auth::user()->profile->website;
            $avatar = Auth::user()->profile->avatar_full_path;
            $contact = Auth::user()->profile->contact;
        }
        return view('profile.index', compact('company','avatar','contact','website'));
    }

    public function accountInfoUpdate(Request $request)
    {
        $request->validate([
            'name' => 'required|string|min:2',
            'email' => 'required|email:rfc,dns',
            'company' => 'nullable|string|min:2',
            'contact' => 'nullable|digits_between:10,15',
            'website' => 'nullable|string',
//            'agree' => 'required|string',
            'profile_avatar' => 'nullable|file|image|mimes:jpg,jpeg,png',
            'profile_avatar_remove' => 'nullable|boolean',
        ]);

        try {
            DB::beginTransaction();
            $user = User::find(Auth::id());

            if ($user->profile) {
                $profile = $user->profile;
            }
            else {
                $profile = new UserProfile();
            }

            $user->name = $request->name;
            $user->email = $request->email;

            $profile->user_id = Auth::id();
            $profile->company_name = $request->company;
            $profile->contact = $request->contact;
            $profile->website = $request->website;
            $avatar = null;

            if ($request->file('profile_avatar')) {
                $image_path = Auth::id()."/avatar/";
                $image_name = time();
                $extension = $request->file('profile_avatar')->getClientOriginalExtension();

                Storage::putFileAs("public/".$image_path, $request->file('profile_avatar'), $image_name.".".$extension);
                if ($profile || $request->profile_avatar_remove) {
                    Storage::delete($profile->avatar);
                }
                $avatar = asset("storage/$image_path$image_name.$extension");
                $profile->avatar = "$image_name.$extension";
                $profile->avatar_full_path = $avatar;
            }
            if ($request->profile_avatar_remove) {
                $avatar = null;
                $profile->avatar_full_path = null;
            }

            if ($profile->saveOrFail() && $user->saveOrFail()) {
                DB::commit();
//                session(['avatar' => $avatar]);
            }

        } catch (\Throwable $e) {
            Log::error($e->getMessage() . ' @ ' . $e->getLine());
            DB::rollBack();

            alertNotify("Oops! Something went wrong. Please try again",null,'danger');

            return back()->withInput();
        }

        alertNotify("Your account profile has been updated.",null,'success');

        return back();
    }

    public function changePassword()
    {
        return view('profile.change-password');
    }

    public function changePasswordUpdate(Request $request)
    {
        $user = User::find(Auth::id());
        if (!Hash::check($request->old_password, $user->password)) {
            alertNotify( __("Your current password does not match in our records."),'','danger' );
            return back();
        }
        $request->validate([
            'old_password' => 'required',
            'password' => ['required', 'confirmed', Rules\Password::defaults()],
        ]);

        $user->password = Hash::make($request->password);
        $user->remember_token = Str::random(60);
        $user->save();
        if ($user->save()) {
            alertNotify( __("Your current password has been changed. Please login."),'','success' );
            Auth::logout();
            return redirect()->route('login');
        }
        else {
            alertNotify( __("Sorry! Something went wrong. Try again."),'','warning' );
            return back();
        }


    }
}
