<?php

namespace App\Http\Controllers;

use App\Models\Benefit;
use App\Models\Currency;
use App\Models\Partner;
use App\Models\Program;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Yajra\DataTables\Facades\DataTables;

class ProgramManagementController extends Controller
{
    /**
     * Get Household Data
     * @param Request $request
     *
     * @return string
     */
    public function index(Request $request)
    {
        $page_title = "Programs Management";
        $name_query = $location_query = $partner_query = $manager_query = $query = 0;
        $program = null;

        $managers = DB::table('users')->join('role_user','users.id', '=', 'role_user.user_id')
        ->join('roles','roles.id', '=', 'role_user.role_id')
        ->join('deployment_user','deployment_user.user_id', '=', 'users.id')
        ->select('users.name as surname','users.firstname','users.othername','users.id')
        ->where('users.account_verified_at','!=', null)
        ->where('deployment_user.deployment_id','=', getActiveInstance())
        ->orderBy('users.name')
        ->get();

        $partners = Partner::where('instance_id', '=', getActiveInstance())->where('status', 1)->pluck('name','id')->toArray();

        if (isset($request->name_query)) { $name_query = $request->type; $query = 1;}
        if (isset($request->manager_query)) { $manager_query = $request->manager_query; $query = 1;}
        if (isset($request->partner_query)) { $partner_query = $request->partner_query; $query = 1;}
        if (isset($request->location_query)) { $location_query = $request->location_query; $query = 1;}
        return view('programs.index', compact('page_title','query','name_query','location_query',
        'manager_query','managers','partner_query','partners','program'));
    }

    /**
     * Save Program
     *
     * @return string
     */
    public function create() {
        $action = 'new';
        $page_title = 'New Program';
        $program = null;
        $managers = DB::table('users')->join('role_user','users.id', '=', 'role_user.user_id')
            ->join('roles','roles.id', '=', 'role_user.role_id')
            ->join('deployment_user','deployment_user.user_id', '=', 'users.id')
            ->select('users.name as surname','users.firstname','users.othername','users.id')
            ->where('deployment_user.deployment_id','=', getActiveInstance())
            ->where('users.email_verified_at','!=', null)
            ->orderBy('users.name')
            ->get();

        $benefits = Benefit::where('archived', 0)->where('instance_id','=', getActiveInstance())
            ->pluck('titles','id')->toArray();
        $currencies = Currency::distinct()->select('country','name','id')->get();
        $partners = Partner::where('status', 1)
            ->where('instance_id','=', getActiveInstance())->pluck('name','id')->toArray();
        return view('programs.form', compact('page_title','managers','partners','benefits','currencies','program','action'));
    }

    public function uploadDocument($key = 'document', $validation = 'file|mimes:jpeg,png,jpg,pdf,docx,xlsx|max:10048|sometimes')
    {
        $folder = 'programs';
        request()->validate([$key => $validation]);

        $file = null;
        if (request()->hasFile($key)) {
            $file = Storage::disk('public_uploads')->putFile($folder, request()->file($key), 'public');
        }

        return $file;
    }

    /**
     * Save Program
     *
     * @return string
     */
    public function save(Request $request)
    {
        $request->validate([
            'name'  =>  'required|string',
            'currencies'  =>  'required|array',
            'partners'  =>  'required|array',
            'managers'  =>  'required',
            'locations'  =>  'required|array',
            'benefits'  =>  'nullable|array',
            'description'  =>  'required|string',
            'document'  =>  'nullable|file',
        ]);

        $document_path = null;
        if($request->file('document')) {
            $file = $this->uploadDocument();
            $document_path = 'app-uploads/'.$file;
        }

        $program = Program::create([
            'instance_id'  =>  getActiveInstance(),
            'name'  =>  $request->name,
            'description'  =>  $request->description,
            'document'  =>  $document_path,
            'status'  =>  1,
            'created_by'  =>  auth()->id(),
        ]);

        if($program) {
            $program->currencies()->sync($request->currencies);
            $program->locations()->sync($request->locations);
            $program->partners()->sync($request->partners);
            $program->managers()->sync($request->managers);
            $program->benefits()->sync($request->benefits);
            $message = "A new Program has been added.";
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return back();
        }
        else {
            $message = "Sorry! Something went wrong. Please try again.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }
    }

    public function edit($program_id) {
        $action = 'edit';
        $program = Program::find(Crypt::decrypt($program_id));
        $page_title = 'Edit Program';

        $managers = DB::table('users')->join('role_user','users.id', '=', 'role_user.user_id')
            ->join('roles','roles.id', '=', 'role_user.role_id')
            ->join('deployment_user','deployment_user.user_id', '=', 'users.id')
            ->select('users.name as surname','users.firstname','users.othername','users.id')
            ->where('deployment_user.deployment_id','=', getActiveInstance())
            ->where('users.email_verified_at','!=', null)
            ->orderBy('users.name')
            ->get();

        $benefits = Benefit::where('archived', 0)->where('instance_id','=', getActiveInstance())
            ->pluck('titles','id')->toArray();
        $currencies = Currency::distinct()->select('country','name','id')->get();
        $partners = Partner::where('status', 1)
            ->where('instance_id','=', getActiveInstance())->pluck('name','id')->toArray();
        return view('programs.form', compact('page_title','managers','partners','benefits','currencies','program','action'));
    }

    /**
     * Update Household
     *
     * @return string
     */
    public function update(Request $request)
    {
        $request->validate([
            'name'  =>  'required|string',
            'currencies'  =>  'required|array',
            'partners'  =>  'required|array',
            'managers'  =>  'required|array',
            'locations'  =>  'required|array',
            'benefits'  =>  'nullable|array',
            'description'  =>  'required|string',
            'document'  =>  'nullable|file',
        ]);


        $program = Program::find(Crypt::decrypt($request->program_id));

        if(!$program) {
            $message = "Sorry! This program does not exist.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }

        $program->name = $request->name;
        $program->description = $request->description;
        $program->updated_by = auth()->id();

        if($request->file('document')) {
            $file = $this->uploadDocument();
            $document_path = 'app-uploads/'.$file;
            $program->document = $document_path;
        }

        if($program->save()) {
            $program->currencies()->sync($request->currencies);
            $program->locations()->sync($request->locations);
            $program->partners()->sync($request->partners);
            $program->managers()->sync($request->managers);
            $program->benefits()->sync($request->benefits);
            $message = "Program has been updated.";
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return back();
        }
        else {
            $message = "Sorry! Something went wrong. Please try again.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }
    }

    public function view($program_id) {
        $program = Program::with(['managers','benefits'=>['_type','_class'],
        'partners','currencies','locations','surveys'=>['_type','_status']])
            ->where('instance_id','=', getActiveInstance())
            ->where('id','=',Crypt::decrypt($program_id))->first();
        $page_title = 'Program Details';

        return view('programs.view', compact('page_title','program'));
    }

    public function delete($program_id) {

    }

    public function getProgramList($name=null,$manager=null,$partner=null,$location=null)
    {
        $programs = Program::with(['managers','benefits','partners','currencies','locations'])
            ->where('instance_id','=', getActiveInstance())
            ->orderBy('created_at','desc');
        $query = false;
        if ($name > 0) {
            $programs = $programs->where('name','=',$name);
        }

        if ($manager > 0) {
            $programs = $programs->managers->where('id','=',$manager);
        }

        if ($partner > 0) {
            $programs = $programs->partners->where('id','=',$partner);
        }

        if ($location > 0) {
            $programs = $programs->locations->where('id','=',$location);
        }

        $programs = $programs->get();

        return DataTables::of($programs)
            ->addColumn('action', function ($program) {
                $view = '<a href="'.route('program.view',[cryptThis($program->id)]).'"
                class="btn btn-sm btn-link text-info mr-2" href="javascript:;" data-bs-toggle="tooltip" data-bs-placement="top" title="'. __("View Program") .'">
                <i class="fa fa-folder-open"></i></a>';

                $edit = '<a href="'.route('program.edit',[cryptThis($program->id)]).'" class="btn btn-sm btn-link text-primary mr-2" data-bs-toggle="tooltip"
                data-bs-placement="top" title="'. __("Edit Program") .'"><i class="fa fa-edit"></i></a>';

                if (!canDo('administrator','view-program-info')) {
                    $view = '';
                }
                if (!canDo('administrator','edit-program')) {
                    $edit = '';
                }
                return $view. $edit;

            })
            ->editColumn('name', function ($program) {
                return '<span class="fw-bold">'.$program->name.'</span><br><small>'.$this->extraData($program->partners,'name').'</small>';
            })
            ->editColumn('id', function ($program) {
                return $program->id;
            })
            ->addColumn('managers', function ($program) {
                return $this->extraData($program->managers,'fullName').'<br><small>'.$this->extraData($program->locations,'fullName').'</small>';
            })
            ->addColumn('benefits', function ($program) {
                return $this->extraData($program->benefits,'titles').'<br><small>'.$this->extraData($program->currencies,'name').'</small>';
            })
            // ->addColumn('currencies', function ($program) {
            //     return $this->extraData($program->currencies,'name');
            // })
            // ->addColumn('partners', function ($program) {
            //     return ;
            // })
            // ->addColumn('locations', function ($program) {
            //     return $this->extraData($program->locations,'fullName');
            // })
            ->escapeColumns([])
            ->make();
    }

    public function extraData($object,$field,$limit=1)
    {
        $all = $object->count();
        $count = 0;
        $content = '';
        foreach($object as $obj) {
            if($count > 0) $content .= ', ';
            $count+=1;
            $remains = $all-$count;
            $content .= $obj->$field;

            if($count == $limit) {
                if($remains == 0) {
                    break;
                }
                else {
                    $content .= ' ,...<span class="badge rounded-pill bg-info">+ '.$remains.' more</span>';
                    break;
                }
            }
        }
        return $content;
    }
}
