<?php

namespace App\Http\Controllers;

use App\Models\Entry;
use App\Models\Program;
use App\Models\Question;
use App\Models\Section;
use App\Models\Survey;
use App\Models\SurveyForm;
use App\Models\SurveyTeam;
use App\Models\Team;
use Illuminate\Contracts\Database\Eloquent\Builder;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Str;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx\Rels;
use Yajra\DataTables\Facades\DataTables;

class SurveyController extends Controller
{
    public function index(Request $request)
    {
        $query = 0;
        $title_query = $type_query = $program_query = $status_query = 0;
        if (isset($request->title_query)) { $title_query = $request->title_query; $query = 1;}
        if (isset($request->type_query)) { $type_query = $request->type_query; $query = 1;}
        if (isset($request->program_query)) { $program_query = $request->program_query; $query = 1;}
        if (isset($request->status_query)) { $status_query = $request->status_query; $query = 1;}
        $programs = Program::active()->where('instance_id','=', getActiveInstance())->orderBy('name', 'ASC')->get();
        return view('survey.index',compact('query','programs','title_query','type_query','program_query','status_query'));
    }

    public function create($program=null)
    {
        $action = 'new';
        $page_title = 'Add Survey';
        $survey = null;
        $programs = Program::active()->where('instance_id','=',getActiveInstance())->orderBy('name', 'ASC')->get();
        $survey_forms = SurveyForm::where('instance_id','=',getActiveInstance())->orderBy('name', 'ASC')->get();
        return view('survey.form', compact('action','programs','survey','page_title','survey_forms'));
    }

    public function edit($survey_id,$program=null)
    {
        $action = 'edit';
        $page_title = 'Edit Survey';
        $survey = Survey::findOrFail(Crypt::decrypt($survey_id));
        $survey_forms = SurveyForm::where('instance_id','=',getActiveInstance())->orderBy('name', 'ASC')->get();

        if(!$survey) {
            $message = "Survey does not exist";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back();
        }

        $programs = Program::active()->where('instance_id','=',getActiveInstance())->orderBy('name', 'ASC')->get();
        return view('survey.form', compact('action','programs','survey','page_title','survey_forms'));
    }

    public function save(Request $request)
    {
        $request->validate([
            'title' => 'required|string|min:3',
            'type' => 'required',
            'program_id' => 'required',
            'duration' => 'required|string',
//            'regions' => 'required|array',
            'purpose' => 'required|string|min:3',
            'survey_form' => 'required|numeric|min:1',
            'frequency'  =>  'required|string|min:3',
            'mode'  =>  'required|string|min:3',
            'guest_entries'  =>  'nullable|string'
        ]);

        $dates = getDateRange($request->duration);

        $survey = Survey::create([
            'title' => $request->title,
            'instance_id' => getActiveInstance(),
            'type' => $request->type,
            'program_id' => $request->program_id,
            'survey_form_id' => $request->survey_form,
            'duration' => $request->duration,
            'frequency' => $request->frequency,
            'purpose' => $request->purpose,
            'mode' => $request->mode,
            'start_date' => $dates['from'],
            'end_date' => $dates['to'],
            'settings' => [
                'accept-guest-entries' => isset($request->guest_entries)?true:false,
                'limit-per-participant' => 1
            ],
            'created_by' => auth()->id(),
            'status' => config('settings.survey_default_status'),
        ]);

        if($survey) {
//            $survey->regions()->sync($request->regions);
            $message = "The Survey has been created successfully.";
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return redirect()->route('survey.view',[cryptThis($survey->id)]);
        }
        else {
            $message = "Sorry! Something went wrong. Please try again.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }
    }

    public function update(Request $request)
    {
        $request->validate([
            'title' => 'required|string|min:3',
            'type' => 'required',
            'program_id' => 'required',
            'duration' => 'required|string',
//            'regions' => 'required|array',
            'purpose' => 'required|string|min:3',
            'survey_form' => 'required|numeric|min:0',
            'frequency'  =>  'nullable|string|min:3',
            'mode'  =>  'nullable|string|min:3',
            'guest_entries'  =>  'nullable|string'
        ]);

        if($request->frequency <= 0) {
            $frequency = -1;
        }
        else {
            $frequency = $request->frequency;
        }

        $dates = getDateRange($request->duration);

        $survey = Survey::findOrFail(Crypt::decrypt($request->survey_id));

        if(!$survey) {
            $message = "Survey does not exist";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back();
        }
        $survey->title = $request->title;
        $survey->type = $request->type;
        $survey->program_id = $request->program_id;
        $survey->survey_form_id = $request->survey_form;
        $survey->duration = $request->duration;
        $survey->purpose = $request->purpose;
        $survey->frequency = $request->frequency;
        $survey->mode = $request->mode;
        $survey->settings = [
                'accept-guest-entries' => isset($request->guest_entries)?true:false,
                'limit-per-participant' => 1
            ];
        $survey->start_date = $dates['from'];
        $survey->end_date = $dates['to'];
        $survey->updated_by = auth()->id();
//        $survey->regions()->sync($request->regions);

        if($survey->save()) {
            $message = "The Survey has been updated successfully.";
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return redirect()->route('survey.list');
        }
        else {
            $message = "Sorry! Something went wrong. Please try again.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }
    }

    public function view($survey_id)
    {
        $survey = Survey::with(['regions','_status','_type','program','teams'=>[
            'members'
        ],'form'])->withCount(['surveyEntries'])->where('instance_id','=',getActiveInstance())->
        where('id',Crypt::decrypt($survey_id))->first();

//        dd($survey->regions);

        if(!$survey) {
            $message = "Survey does not exist";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back();
        }
        $page_title = 'View Survey';
        return view('survey.view', compact('survey','page_title'));
    }

    public function setRespondents(Survey $survey, Request $request)
    {
        if ($survey->mode == 'self') {
            $survey->team_respondents = null;
            $survey->self_respondents = $request->respondents;
            if($survey->save()) {
                $message = __("Survey respondents saved");
                session()->flash('status', 'success');
                session()->flash('message', $message);
            }
            else{
                $message = __("Respondents did not save. Try again");
                session()->flash('status', 'error');
                session()->flash('message', $message);
            }
        }
        elseif ($survey->mode == 'assisted') {
            $survey->self_respondents = null;
            $survey->team_respondents = $request->respondents;
            if($survey->save()) {
                $message = __("Survey respondents saved");
                session()->flash('status', 'success');
                session()->flash('message', $message);
            }
            else{
                $message = __("Respondents did not save. Try again");
                session()->flash('status', 'error');
                session()->flash('message', $message);
            }
        }
        return back();
    }

    public function viewTeams(Request $request)
    {
        $survey = Survey::with(['teams'=>['members']])->where('instance_id','=',getActiveInstance())
            ->where('id','=',$request->id)->first();

        if(!$survey) {
            $message = '<p class="text-danger">Invalid Survey</p>';
            return response()->json(['message' => $message,'status'=>0]);
        }

        $teams = Team::has('members')->where('program_id','=',$survey->program_id)->where('instance_id','=',getActiveInstance())->get();
        $page_title = view('survey.view-teams', compact('survey','teams'))->render();
        return response()->json(['message' => $page_title,'status'=>1]);
    }

    public function saveTeams(Request $request)
    {
        $assigned = SurveyTeam::updateOrCreate(
            [
                'survey_id' => $request->survey_id,
                'team_id' => $request->team_id,
            ],[
                'created_by' => auth()->id()
            ]
        );

        if ($assigned) {
            $message = "Team has been assigned";
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return back();
        }
        else {
            $message = "Sorry! Something went wrong. Try again.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back();
        }
    }

    public function changeTeamStatus(Request $request)
    {
        $team = SurveyTeam::findOrFail($request->id);

        if($team->status == 1) {
            $team->status = 0;
        }
        else {
            $team->status = 1;
        }

        $team->updated_by = auth()->id();


        if($team->save()) {
            $message = "Saved!";
            return response()->json(['message' => $message,'status'=>1]);
        }
        else {
            $message = "Did not save!";
            return response()->json(['message' => $message,'status'=>0]);
        }
    }

    public function deleteTeam(Request $request)
    {
        $team = SurveyTeam::find(Crypt::decrypt($request->id));

        if(!$team) {
            $message = "Invalid Team";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }

        $team->updated_by = auth()->id();
        $team->save();

        if($team->delete($team->id)) {
            $message = "Team has been detached";
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return back();
        }
        else {
            $message = "Something went wrong. The team cannot be detached. Try again.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back();
        }
    }

    public function getSurveyList($title_query=null,$type_query=null,$program_query=null,$status_query=null)
    {
        $surveys = Survey::with(['program','_type','_status'])->where('instance_id','=',getActiveInstance())
            ->orderBy('title','asc');
        $query = false;
        if ($title_query > 0) {
            $surveys = $surveys->where('title','like',"%$title_query%");
        }

        if ($type_query > 0) {
            $surveys = $surveys->where('type','=',$type_query);
        }

        if ($program_query > 0) {
            $surveys = $surveys->program->where('id','=',$program_query);
        }

        if ($status_query > 0) {
            $surveys = $surveys->where('status','=',$status_query);
        }

        $surveys = $surveys->get();

        return DataTables::of($surveys)
            ->addColumn('action', function ($survey) {
                $view = '<a href="'.route('survey.view',[cryptThis($survey->id)]).'"
                class="btn btn-sm btn-link text-info mr-2" href="javascript:;" data-bs-toggle="tooltip" data-bs-placement="top" title="'. __("View Survey") .'">
                <i class="fa fa-folder-open"></i></a>';

                $edit = '<a href="'.route('survey.edit',[cryptThis($survey->id)]).'" class="btn btn-sm btn-link text-primary mr-2" data-bs-toggle="tooltip"
                data-bs-placement="top" title="'. __("Edit Survey") .'"><i class="fa fa-edit"></i></a>';

                $team = '<a href="#" class="btn btn-sm btn-link text-primary mr-2" data-bs-toggle="modal" data-bs-target="#viewTeamModal"  onclick="viewTeam('.$survey->id.')"
                data-bs-placement="top" title="'. __("Teams") .'"><i class="bi-people"></i></a>';

                $status = '<a href="'.route('survey.edit',[cryptThis($survey->id)]).'" class="btn btn-sm btn-link text-primary mr-2" data-bs-toggle="tooltip"
                data-bs-placement="top" title="'. __("View Survey") .'"><i class="fa fa-pause text-danger"></i></a>';

                if($survey->status != config('settings.survey_opened_status')) {
                    $status = '<a href="'.route('survey.edit',[cryptThis($survey->id)]).'" class="btn btn-sm btn-link text-primary mr-2" data-bs-toggle="tooltip"
                    data-bs-placement="top" title="'. __("Open for entries") .'"><i class="fa fa-play text-success"></i></a>';
                }


                return $view.$edit.$team;
//                return $view.$edit.$team.$status;

            })
            ->editColumn('id', function ($survey) {
                return $survey->id;
            })
            ->editColumn('title', function ($survey) {
                return '<strong>'.$survey->title.'</strong><br><small class"text-muted">'.$survey->program->name.'</small>';
            })
            ->editColumn('type', function ($survey) {
                $respondent = (!is_null($survey->self_respondents))?'<small class="text-muted me-2">-</small><small class="text-muted">For '.Str::headline($survey->self_respondents).'</small>':null;
                return $survey->_type->fullName.'<br><small class="text-muted me-2">'.Str::headline($survey->mode).' Entry</small>'.$respondent;
            })
            ->editColumn('created_at', function ($survey) {
                return date('d M Y', strtotime($survey->created_at));
            })
            ->editColumn('status', function ($survey) {
                return $survey->_status->fullName;
            })
            ->addColumn('timeline', function ($survey) {
                return date('d M Y', strtotime($survey->start_date))." to ".date('d M Y', strtotime($survey->end_date));
            })
            ->escapeColumns([])
            ->make();
    }

    public function entries(Survey $survey)
    {
        $entries = Entry::where('survey_form_id','=',$survey->survey_form_id)
            ->where('survey_id','=',$survey->id)->paginate(1,'*','individual-entries');

        $entries_summary = Entry::with(['participant'])->where('survey_form_id','=',$survey->survey_form_id)
            ->where('survey_id','=',$survey->id)->paginate(100,'*','summary-entries');

        return view('survey.entries-list', compact('survey','entries','entries_summary'));
    }

    public function newEntry(Survey $survey)
    {
        $submit = true;

        if (strtotime($survey->end_date) < strtotime(date('Y-m-d'))) {
            session()->flash('status', 'error');
            session()->flash('message', 'Sorry! The survey is overdue.');
            return back();
        }

        if (strtotime($survey->start_date) > strtotime(date('Y-m-d'))) {
            session()->flash('status', 'error');
            session()->flash('message', 'Sorry! Invalid Survey.');
            return back();
        }

        return view('survey.entry', compact('submit','survey'));
    }

    public function saveEntry(Survey $survey,Request $request)
    {
        $answers = $this->validate($request, $survey->form->rules);

        $entry = (new Entry)->for($survey->form)->onSurvey($survey)->by(auth()->user())->fromArray($answers)->push();

        if($entry){
            $message = "Entry has been saved";
            $status = 'success';
        }
        else {
            $message = "Something went wrong. Try again.";
            $status = 'error';
        }
        session()->flash('status', $status);
        session()->flash('message', $message);
        return redirect()->route('survey.mySurveys');
    }

    public function deleteEntry(Request $request)
    {
        $entry = Entry::find(Crypt::decrypt($request->id));

        if(!$entry) {
            $message = "Invalid Entry";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }

        if($entry->delete()) {
            $message = "Entry has been deleted";
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return back();
        }
        else {
            $message = "Something went wrong. Try again.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back();
        }
    }

    public function mySurveys()
    {
        if (auth()->user()->is_admin){
            $surveys = Survey::with(['surveyEntries' => function ($query) {
                    $query->where('participant_id','=', auth()->id())->orderBy('created_at','desc');
                }])
                ->whereDate('start_date','<=',now())
                ->where('instance_id','=',getActiveInstance())
                ->where('mode','=','self')
                ->where('self_respondents','=','all-users')
                ->orWhere('self_respondents','=','administrators')->paginate(100);
        }
        else {
            $surveys = Survey::with(['surveyEntries' => function ($query) {
                    $query->where('participant_id','=', auth()->id())->orderBy('created_at','desc');
                }])
                ->whereDate('start_date','<=',now())
                ->where('instance_id','=',getActiveInstance())
                ->where('mode','=','self')
                ->where('self_respondents','=','all-users')
                ->orWhere('self_respondents','=','applicants')->paginate(100);
        }

        return view('survey.my-surveys', compact('surveys'));
    }

    public function mySurveyView(Entry $entry)
    {
        return view('survey.view-survey-entry', compact('entry'));
    }
}
