<?php

namespace App\Http\Controllers;

use App\Models\Entry;
use App\Models\Question;
use App\Models\Section;
use App\Models\Survey;
use App\Models\SurveyForm;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Crypt;

class SurveyModuleController extends Controller
{

    public function index(Request $request)
    {
        $survey_forms = SurveyForm::withCount(['surveys'])->get();

        if (isset($request->title_query)) $survey_forms = SurveyForm::where('name','like','%'.$request->title_query.'%')->get();

        return view('survey-module.index', compact('survey_forms'));
    }

    public function saveSurveyForm(Request $request)
    {
        $request->validate([
            'name'  =>  'required|string|min:3|unique:survey_forms,name',
            'category'  =>  'required|string|min:3',
        ]);

        $survey = SurveyForm::create([
            'name' => $request->name,
            'category' => $request->category,
            'instance_id' => getActiveInstance(),
        ]);

        if ($survey) {
            session()->flash('status', 'success');
            session()->flash('message', 'Survey form has been created. Add questions to the survey');

            return redirect()->route('survey_module.surveyQuestions',[$survey]);
        }
        else {
            session()->flash('status', 'error');
            session()->flash('message', 'Information did not save. Please try again');
        }

        return back();
    }

    public function updateSurveyForm(Request $request,SurveyForm $survey)
    {
        $request->validate([
            'name'  =>  'required|string|min:3',
            'category'  =>  'required|string|min:3',
        ]);

        $surveyForm = SurveyForm::where('id','=',$survey->id)->update([
            'name' => $request->name,
            'category' => $request->category,
        ]);

        if ($surveyForm) {
            session()->flash('status', 'success');
            session()->flash('message', 'Survey form has been updated');
        }
        else {
            session()->flash('status', 'error');
            session()->flash('message', 'Information did not save. Please try again');
        }

        return back();
    }

    public function surveyQuestions(SurveyForm $survey)
    {
        return view('survey-module.questions', compact('survey'));
    }

    public function addQuestion(SurveyForm $survey)
    {
        $fields = getFieldTypes();
        $action = 'add';

        return view('survey-module.question-form', compact('survey','fields','action'));
    }

    public function saveQuestion(Request $request,SurveyForm $survey)
    {
        $rules = $options = null;
        $rules[] = strtolower($request->required);
        if ($request->input_type == "number") {
            $rules[] = 'numeric';
        }
        elseif ($request->input_type == "file") {
            $rules[] = 'file';
            if (!is_null($request->file_type)) $rules[] = 'mimes:'.$request->file_type;
            if (!is_null($request->file_size)) $rules[] = 'mimes:'.$request->file_size*1000;
        }
        elseif ($request->input_type == "email") {
            $rules[] = 'email';
        }
        elseif ($request->input_type == "multiselect") {
            $rules[] = 'array';
        }
        else {
            $rules[] = 'string';
        }

        if (!is_null($request->min)) $rules[] = 'min:'.$request->min;
        if (!is_null($request->max)) $rules[] = 'max:'.$request->max;

        if ($request->options) {
            $options = explode(';',$request->options);
        }

        if (isset($request->question_id)) {
            $question = Question::find($request->question_id);
            if (!$question){
                $message = "Question does not exist";
                session()->flash('status', 'error');
                session()->flash('message', $message);
            }
            $message = "Question has been updated";
        }
        else {
            $question = new Question();
            $message = "New Question added";
        }

        $question->instance_id = getActiveInstance();
        $question->survey_form_id = $survey->id;
        $question->section_id = $request->section;
        $question->content = $request->question;
        $question->type = $request->input_type;
        $question->options = $options;
        $question->rules = $rules;
        $question->raw = $request->except(['_token','return']);


        if($question->saveQuietly()) {
            session()->flash('status', 'success');
            session()->flash('message', $message);
        }
        else {
            $message = "Question did not save. Please try again";
            session()->flash('status', 'error');
            session()->flash('message', $message);
        }

        if ($request->return == 'return') return redirect()->route('survey_module.surveyQuestions',[$survey->id]);
        return back();
    }

    public function editQuestion(SurveyForm $survey,Question $question)
    {
        $fields = getFieldTypes();
        $action = 'edit';

        return view('survey-module.question-form', compact('survey','fields','action','question'));
    }

    public function deleteQuestion(Question $question)
    {
        if (Question::destroy($question->id)){
            session()->flash('status', 'success');
            session()->flash('message', 'Question has been deleted');
        }
        else {
            $message = "Question did not delete. Please try again";
            session()->flash('status', 'error');
            session()->flash('message', $message);
        }
        return back();

    }

    public function saveSection(Request $request,SurveyForm $survey)
    {
        if (isset($request->section_id)) {
            $section = Section::find($request->section_id);

            if (!$section){
                $message = "Section does not exist";
                session()->flash('status', 'error');
                session()->flash('message', $message);
            }
            $message = "Section has been updated";
        }
        else{
            $section = new Section();
            $message = "New Section added";
        }
        $section->survey_form_id = $survey->id;
        $section->name = $request->name;

        if($section->save()) {
            session()->flash('status', 'success');
            session()->flash('message', $message);
        }
        else {
            $message = "Section did not save. Please try again";
            session()->flash('status', 'error');
            session()->flash('message', $message);
        }
        return back();
    }

    public function entryForm(SurveyForm $survey_form)
    {
        $submit = false;
        $survey = null;

        return view('survey-module.entry-form', compact('survey_form','submit','survey'));
    }

    public function saveEntry(SurveyForm $survey_form,Request $request)
    {
        $answers = $this->validate($request, $survey_form->rules);
        $survey = Survey::find(Crypt::decrypt($request->survey_id));
        if (!$survey) {
            session()->flash('status', 'error');
            session()->flash('message', 'Invalid Survey selected.');
            return back();
        }

        $entry = (new Entry)->for($survey_form)->onSurvey($survey)->by(auth()->user())->fromArray($answers)->push();

        dd($entry);
    }
}
