<?php

namespace App\Http\Controllers;

use App\Models\Program;
use App\Models\Survey;
use App\Models\Team;
use App\Models\TeamManagementMember;
use App\Models\TeamMember;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;

class TeamsController extends Controller
{
    public function index(Request $request)
    {
        $teams = Team::with(['createdBy','updatedBy','program','members','memberInvitations'])->where('instance_id','=', getActiveInstance())->orderBy('teams.name', 'ASC');

        if (isset($request->code))
        {
            $teams = $teams->where('code','like','%'.$request->code.'%')->orWhere('teams.name','like','%'.$request->code.'%');
        }

        if (isset($request->program))
        {
            $teams = $teams->where('program_id','=',$request->program);
        }

        if (isset($request->status))
        {
            $teams = $teams->where('teams.status','=',$request->status);
        }

        $teams = $teams->get();
        $programs = Program::active()->where('instance_id','=', getActiveInstance())->orderBy('name','asc')->pluck('name','id');
        $members = TeamManagementMember::active()->where('instance_id','=', getActiveInstance())->orderBy('name','asc')->get();

        return view('team.index',compact('teams','programs','members'));
    }

    public function view(Request $request)
    {
        $team = Team::with(['createdBy','updatedBy','program',
        'members'=>['teamMember','_role'],'memberInvitations'])
        ->where('id',$request->id)->where('instance_id','=', getActiveInstance())->first();

        if(!$team) {
            $message = '<div class="text-center text-danger">Team does not exist</div>';
            return response()->json(['message' => 'Invalid Team']);
        }
        else {
            $message = view('team.view', compact('team'))->render();
            return response()->json(['message' => $message]);
        }
    }

    public function save(Request $request)
    {
        $request->validate([
            'name' =>   'required',
            'program' =>   'required',
            'team_type' =>   'required',
            'status' =>   'required',
        ]);

        $new_team = Team::create([
            'instance_id' => getActiveInstance(),
            'name' => $request->name,
            'code' => getRandomString(5),
            'team_type' => $request->team_type,
            'program_id' => $request->program,
            'status' => $request->status,
            'created_by' => auth()->id(),
        ]);

        if($new_team) {
            $message = "A new Team has been created.";
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return back();
        }
        else {
            $message = "Sorry! Something went wrong. Please try again.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }
    }

    public function edit(Request $request)
    {
        $team = Team::find($request->id);

        // $surveys = Survey::where('program_id','=',$team->program_id)->get();
        // $surveyOptions = '';
        // foreach($surveys as $survey) {
        //     $selected = null;
        //     if($survey->id == $team->survey_id) $selected = 'selected';
        //     $surveyOptions .= '<option value="'.$survey->id.'" '.$selected.'>'.$survey->title.'</option>';
        // }

        if(!$team) {
            return response()->json(['status' => 0,'message' => 'Invalid Team']);
        }
        else {
            return response()->json(['status' => 1,'team' => $team]);
        }
    }

    public function getProgramSurveys(Request $request)
    {
        $surveys = Survey::where('program_id','=',$request->program_id)->where('instance_id','=', getActiveInstance())->get();
        $surveyOptions = '<option value="">-- Choose Survey --</option>';

        foreach($surveys as $survey) {
            $surveyOptions .= '<option value="'.$survey->id.'">'.$survey->title.'</option>';
        }
        return response()->json(['surveyOptions'=>$surveyOptions]);
    }

    public function update(Request $request)
    {
        $request->validate([
            'name' =>   'required',
            'program' =>   'required',
            'team_type' =>   'required',
            'status' =>   'required',
        ]);

        $team = Team::findOrFail($request->id);
        if(!$team) {
            $message = "Invalid team.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }

        $team->name = $request->name;
        $team->program_id = $request->program;
        $team->team_type = $request->team_type;
        $team->status = $request->status;
        $team->updated_by = auth()->id();

        if($team->save()) {
            $message = "Team has been updated.";
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return back();
        }
        else {
            $message = "Sorry! Something went wrong. Please try again.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }
    }

    public function delete(Request $request)
    {
        $team = Team::find(Crypt::decrypt($request->id));

        if(!$team) {
            $message = "Invalid Team";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }

        $team->updated_by = auth()->id();
        $team->save();

        if($team->delete($team->id)) {
            $message = "Team has been removed successfully";
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return back();
        }
        else {
            $message = "Something went wrong. The team cannot be removed. Try again later.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back();
        }
    }

    public function deleteMember(Request $request)
    {
        $member = TeamMember::find(Crypt::decrypt($request->id));

        if(!$member) {
            $message = "Invalid Member";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }

        $member->updated_by = auth()->id();
        $member->save();

        if($member->delete($member->id)) {
            $message = "Member has been removed successfully";
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return back();
        }
        else {
            $message = "Something went wrong. The member cannot be removed. Try again later.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back();
        }
    }

    public function saveMember(Request $request)
    {
        $request->validate([
            'team' =>   'required',
            'member' =>   'required',
            'team_role' =>   'required',
            'status' =>   'required',
        ]);

        $existing_team_member = TeamMember::where('team_id',$request->team)->where('user_id',$request->member)->count();

        if($existing_team_member > 0) {
            $message = "This Member already belongs in the Team";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }

        $new_team = TeamMember::create([
            'instance_id' => getActiveInstance(),
            'team_id' => $request->team,
            'user_id' => $request->member,
            'role' => $request->team_role,
            'status' => $request->status,
            'created_by' => auth()->id(),
        ]);

        if($new_team) {
            $message = "A new Team Member has been added.";
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return back();
        }
        else {
            $message = "Sorry! Something went wrong. Please try again.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }
    }

    public function editMember(Request $request)
    {
        $member = TeamMember::find($request->id);

        if(!$member) {
            return response()->json(['status' => 0,'message' => 'Invalid Team Member']);
        }
        else {
            return response()->json(['status' => 1,'member' => $member]);
        }
    }

    public function updateMember(Request $request)
    {
        $request->validate([
            'team_role' =>   'required',
            'status' =>   'required',
        ]);

        $existing_team_member = TeamMember::findOrFail($request->id);

        if(!$existing_team_member) {
            $message = "This Member does not exist";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }

        $existing_team_member->role = $request->team_role;
        $existing_team_member->status = $request->status;
        $existing_team_member->updated_by = auth()->id();

        if($existing_team_member->save()) {
            $message = "Team Member information has been updated.";
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return back();
        }
        else {
            $message = "Sorry! Something went wrong. Please try again.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }
    }

    public function changeTeamStatus(Request $request)
    {
        $team = Team::findOrFail($request->id);

        if($team->status == 1) {
            $team->status = 0;
        }
        else {
            $team->status = 1;
        }

        $team->updated_by = auth()->id();


        if($team->save()) {
            $message = "Saved!";
            return response()->json(['message' => $message,'status'=>1]);
        }
        else {
            $message = "Did not save!";
            return response()->json(['message' => $message,'status'=>0]);
        }
    }

    public function changeTeamMemberStatus(Request $request)
    {
        $member = TeamMember::findOrFail($request->id);

        if($member->status == 1) {
            $member->status = 0;
        }
        else {
            $member->status = 1;
        }

        $member->updated_by = auth()->id();


        if($member->save()) {
            $message = "Saved!";
            return response()->json(['message' => $message,'status'=>1]);
        }
        else {
            $message = "Did not save!";
            return response()->json(['message' => $message,'status'=>0]);
        }
    }


    //TEAM MANAGEMENT

    public function teamManagement(Request $request)
    {
        $team_members = TeamManagementMember::with(['createdBy','updatedBy','_gender','_education_level'])->where('instance_id','=', getActiveInstance())->orderBy('team_management_members.name', 'ASC');

        if (isset($request->name))
        {
            $team_members = $team_members->where('team_management_members.name','like','%'.$request->name.'%')
            ->orWhere('team_management_members.email','like','%'.$request->name.'%');
        }

        if (isset($request->gender))
        {
            $team_members = $team_members->where('gender','=',$request->gender);
        }

        if (isset($request->status))
        {
            $team_members = $team_members->where('team_management_members.status','=',$request->status);
        }

        $team_members = $team_members->get();
        $languages = DB::table('languages')->orderBy('name','asc')->pluck('name','id');

        return view('team-management.index',compact('team_members','languages'));
    }

    public function addTeamManagementMember()
    {
        $action = 'new';
        $member = null;
        $page_title = 'Add New Member';
        $languages = DB::table('languages')->orderBy('name','asc')->pluck('name','id');
        return view('team-management.form',compact('member','action','languages','page_title'));
    }

    public function uploadAvatar($key = 'avatar', $validation = 'image|mimes:jpeg,png,jpg,gif,svg|max:2048|sometimes')
    {
        request()->validate([$key => $validation]);

        $file = null;
        if (request()->hasFile($key)) {
            $location = getActiveInstance().'/team-members/avatar';
            $file = request()->file($key);
            $name = $file->hashName();
            $file = Storage::disk('public')->putFileAs($location, request()->file($key),$name);
            $file = 'storage/'.$file;
        }

        return $file;
    }

    public function uploadDocument($key = 'criminal_background_file', $folder = 'team-members/documents', $validation = 'nullable|file|mimes:jpeg,png,jpg,docx,pdf,txt,xlsx|max:10048|sometimes')
    {

        request()->validate([$key => $validation]);

        $file = null;
        if (request()->hasFile($key)) {
            $location = getActiveInstance().'/'.$folder;
            $file = request()->file($key);
            $name = $file->hashName();
            $file = Storage::disk('local')->putFileAs($location, request()->file($key),$name);
        }

        return $file;
    }

    public function saveTeamManagementMember(Request $request)
    {
        $request->validate([
            'name' =>   'required|string|min:2',
            'email' =>   'nullable|email',
            'date_of_birth' =>   'required',
            'phone' =>   'required',
            'gender' =>   'required',
            'member_languages' =>   'required',
            'address' =>   'nullable',
            'education_level' =>   'required',
            'education_institution' =>   'nullable',
            'work_experience' =>   'nullable',
            'training_skills' =>   'nullable',
            'criminal_background' =>   'required',
            'avatar' =>   'nullable|file',
            'criminal_background_file' =>   'nullable|file',
            'status' =>   'required',
        ]);

        $avatar_path = null;
        if($request->file('avatar')) {
            $avatar_path = $this->uploadAvatar();
        }

        $criminal_background_file = null;
        if($request->file('criminal_background_file')) {
            $criminal_background_file = $this->uploadDocument();
        }

        $new_team = TeamManagementMember::create([
            'instance_id' => getActiveInstance(),
            'name' => $request->name,
            'email' => $request->email,
            'date_of_birth' => date('Y-m-d', strtotime($request->date_of_birth)),
            'phone' => $request->phone,
            'gender' => $request->gender,
            'languages' => $request->member_languages,
            'address' => $request->address,
            'education_level' => $request->education_level,
            'education_institution' => $request->education_institution,
            'work_experience' => $request->work_experience,
            'training_skills' => $request->training_skills,
            'criminal_background' => $request->criminal_background,
            'criminal_background_file' => $criminal_background_file,
            'avatar_url' => $avatar_path,
            'status' => $request->status,
            'created_by' => auth()->id(),
        ]);

        if($new_team) {
            $message = "A New Team Member has been added.";
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return back();
        }
        else {
            $message = "Sorry! Something went wrong. Please try again.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }
    }

    public function viewTeamManagementMember(Request $request)
    {
        $member = TeamManagementMember::with(['createdBy','updatedBy','_gender','_education_level',
        'teamMemberships'=>['team','_role']])
        ->where('instance_id','=', getActiveInstance())->where('id',$request->id)->first();

        if(!$member) {
            $message = '<div class="text-center text-danger">Member does not exist</div>';
            return response()->json(['message' => $message]);
        }
        else {
            $message = view('team-management.view-member', compact('member'))->render();
            return response()->json(['message' => $message]);
        }
    }

    public function editTeamManagementMember($team_management_member_id)
    {
        $member = TeamManagementMember::find(Crypt::decrypt($team_management_member_id));

        if(!$member) {
            $message = 'Invalid Team Member';
            return back();
        }

        $action = 'edit';
        $page_title = 'Edit Member';
        $languages = DB::table('languages')->orderBy('name','asc')->pluck('name','id');
        return view('team-management.form',compact('member','action','languages','page_title'));
    }
    public function updateTeamManagementMember(Request $request)
    {
        $member = TeamManagementMember::find(Crypt::decrypt($request->team_member));

        if(!$member) {
            $message = 'Invalid Team Member';
            return back();
        }

        $request->validate([
            'name' =>   'required|string|min:2',
            'email' =>   'required|email',
            'date_of_birth' =>   'required',
            'phone' =>   'required',
            'gender' =>   'required',
            'member_languages' =>   'required',
            'address' =>   'nullable',
            'education_level' =>   'required',
            'education_institution' =>   'nullable',
            'work_experience' =>   'nullable',
            'training_skills' =>   'nullable',
            'criminal_background' =>   'required',
            'avatar' =>   'nullable|file',
            'criminal_background_file' =>   'nullable|file',
            'status' =>   'required',
        ]);

        $new_team = TeamManagementMember::updateOrCreate([
            'id'    => Crypt::decrypt($request->team_member)
        ],[
            'name' => $request->name,
            'email' => $request->email,
            'date_of_birth' => date('Y-m-d', strtotime($request->date_of_birth)),
            'phone' => $request->phone,
            'gender' => $request->gender,
            'languages' => $request->member_languages,
            'address' => $request->address,
            'education_level' => $request->education_level,
            'education_institution' => $request->education_institution,
            'work_experience' => $request->work_experience,
            'training_skills' => $request->training_skills,
            'criminal_background' => $request->criminal_background,
            'status' => $request->status,
            'updated_by' => auth()->id(),
        ]);

        if($new_team) {

            if($request->file('avatar')) {
                $member->avatar_url = $this->uploadAvatar();
                $member->save();
            }

            if($request->file('criminal_background_file')) {
                $member->criminal_background_file = $this->uploadDocument();
                $member->save();
            }
            $message = "Team Member has been updated.";
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return redirect()->route('team.teamManagement');
        }
        else {
            $message = "Sorry! Something went wrong. Please try again.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }
    }

    public function deleteTeamManagementMember(Request $request)
    {
        $member = TeamManagementMember::find(Crypt::decrypt($request->id));

        if(!$member) {
            $message = "Invalid Member";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }

        $member->updated_by = auth()->id();
        $member->save();

        if($member->delete($member->id)) {
            $message = "Member has been removed successfully";
            $status = 'success';
        }
        else {
            $message = "Something went wrong. The member cannot be removed. Try again later.";
            $status = 'error';
        }
        session()->flash('status', $status);
        session()->flash('message', $message);
        return back();
    }

}
