<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use OwenIt\Auditing\Contracts\Auditable;

class Applicant extends Model implements Auditable
{
    use \OwenIt\Auditing\Auditable;
    use HasFactory;

    protected $fillable = [
        'regID','village_details_id',
        'surnames',
        'first_names',
        'other_names',
        'dobs',
        'genders',
        'state_of_births',
        'place_of_births',
        'country_of_births',
        'country_id', 'state_id', 'city_id',
        'nationalities',
        'added_by',
        'pmtScore',
        'is_orphan',
        'highest_education_level',
        'learning_status',
        'occupation',
        'father_alive',
        'mother_alive',
        'nsnp_benefits',
        'village_id'
    ];


    protected $casts = [
        'impairments'   =>  'json'
    ];


    public function history(){
        return $this->hasMany(RecordTracker::class,'applicant_number','regID');
    }

    public function idDetails() {
        return $this->hasMany(IdentificationDetail::class,'applicant_number','regID');
    }

    public function contactDetails() {
        return $this->hasMany(ContactDetail::class,'applicant_number','regID');
    }

    public function maritalDetails() {
        return $this->hasMany(MaritalDetail::class,'applicant_number','regID');
    }

    public function employmentDetails() {
        return $this->hasMany(EmploymentDetail::class,'applicant_number','regID');
    }

    public function accountDetails() {
        return $this->hasMany(AccountDetail::class,'regID','regID');
    }

    public function familyDetails() {
        return $this->hasMany(Family::class,'applicant_number','regID');
    }

    public function healthDetails() {
        return $this->hasMany(HealthDetail::class,'regID','regID');
    }

    public function documents() {
        return $this->hasMany(ApplicantDocument::class,'applicant_id');
    }

    public function applications() {
        return $this->hasMany(Application::class,'applicant_number','regID');
    }

    public function scopeActive($query) {
        return $query->where('status',1);
    }

    public function getFullnameAttribute() {
        return "{$this->surnames} {$this->first_names}";
    }

    public function getAgeAttribute() {
        return getAgeByDate($this->dobs);
    }

    public function getProfileAvatarAttribute() {
        if (!is_null($this->avatar)) {
            if (\File::exists(public_path($this->avatar))) {
                return asset($this->avatar);
            }
        }
//        else {
        return asset('images/profile-picture-default.png');
//        }
    }

    public function approvedApplications()
    {
        return $this->applications()->where('status','=','Approved');
    }

    public function totalAmountAccrued()
    {
        $approved_applications = $this->approvedApplications()->pluck('id')->toArray();
        return ApplicationApproval::whereIn('application_id',$approved_applications)->sum('amount');
    }

    public function deployments()
    {
        return $this->belongsToMany(Deployment::class)->withTimestamps();
    }

    public function verifiedContact()
    {
        return $this->contactDetails->where('status','Verified')
            ->where('default',1)
            ->where('type_name','Mobile Number')
            ->pluck('mobile')
            ->first();
    }

    public function owner()
    {
        return $this->hasOne(User::class,'applicant_number','regID');
    }

    public function village()
    {
        return $this->belongsTo(Village::class);
    }
    


    public function verifiedEmail()
    {
        return $this->contactDetails->where('status','Verified')
            ->where('default',1)
            ->where('type_name','Email')
            ->pluck('email')
            ->first();
    }

    public function ageGroup()
    {
        $age_group = null;
        $age = getRawAgeByDate($this->dobs);
        $age_groups = config('settings.age_group');
        foreach ($age_groups as $group=>$ages) {
            if ($age <= $ages[0] && $age >= $ages[1])  {
                $age_group = $group; break;
            }
        }
        return $age_group;
    }

}
