<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use MattDaneshvar\Survey\Contracts\Question as QuestionAlias;
use MattDaneshvar\Survey\Exceptions\GuestEntriesNotAllowedException;

class Entry extends \MattDaneshvar\Survey\Models\Entry
{
    use HasFactory;
    protected $fillable = ['survey_id', 'participant_id','survey_form_id'];

    /**
     * The survey the entry belongs to.
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function surveyForm()
    {
        return $this->belongsTo(get_class(app()->make(SurveyForm::class)));
    }

    /**
     * The survey the entry belongs to.
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function survey()
    {
        return $this->belongsTo(get_class(app()->make(Survey::class)));
    }

    /**
     * The participant that the entry belongs to.
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function participant()
    {
        return $this->belongsTo(User::class, 'participant_id');
    }

    /**
     * The answers within the entry.
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function answers()
    {
        return $this->hasMany(Answer::class);
    }

    /**
     * Set the survey the entry belongs to.
     *
     * @param  Survey  $survey
     * @return $this
     */
    public function onSurvey(Survey $survey)
    {
        $this->survey()->associate($survey);
        return $this;
    }

    /**
     * Set the survey the entry belongs to.
     *
     * @param  SurveyForm  $survey_form
     * @return $this
     */
    public function for(\MattDaneshvar\Survey\Contracts\Survey $survey_form)
    {
        $this->surveyForm()->associate($survey_form);
        return $this;
    }

    /**
     * Set the participant who the entry belongs to.
     *
     * @param  Model|null  $model
     * @return $this
     */
    public function by(Model $model = null)
    {
        $this->participant()->associate($model);

        return $this;
    }

    /**
     * Create an entry from an array.
     *
     * @param  array  $values
     * @return $this
     */
    public function fromArray(array $values)
    {
        foreach ($values as $key => $value) {
            if ($value === null) {
                continue;
            }

            $question_id = substr($key, 1);
            $question = Question::findOrFail($question_id);
            if ($question->type == 'multiselect') {
//                dd($value);
            }

            $answer_class = get_class(app()->make(Answer::class));

            $this->answers->add($answer_class::make([
                'question_id' => $question_id,
                'entry_id' => $this->id,
                'value' => $value,
            ]));
        }

        return $this;
    }

    /**
     * The answer for a given question.
     *
     * @param  Question  $question
     * @return mixed|null
     */
    public function answerFor(QuestionAlias $question)
    {
        $answer = $this->answers()->where('question_id', $question->id)->first();

        return isset($answer) ? $answer->value : null;
    }

    /**
     * Save the model and all of its relationships.
     * Ensure the answers are automatically linked to the entry.
     *
     * @return bool
     */
    public function push()
    {
        $this->save();

        foreach ($this->answers as $answer) {
            $answer->entry_id = $this->id;
        }

        return parent::push();
    }

    /**
     * Validate participant's legibility.
     *
     * @throws GuestEntriesNotAllowedException
     */
    public function validateParticipant()
    {
        if ($this->surveyForm->acceptsGuestEntries()) {
            return;
        }

        if ($this->participant_id !== null) {
            return;
        }

        throw new GuestEntriesNotAllowedException();
    }

    /**
     * Validate if entry exceeds the survey's
     * max entry per participant limit.
     *
     * @throws MaxEntriesPerUserLimitExceeded
     */
    public function validateMaxEntryPerUserRequirement()
    {
        $limit = $this->surveyForm->limitPerParticipant();

        if ($limit === null) {
            return;
        }

        $count = static::where('participant_id', $this->participant_id)
            ->where('survey_form_id', $this->surveyForm->id)
//            ->where('survey_id', $this->survey->id)
            ->count();

        if ($count >= $limit) {
            throw new MaxEntriesPerUserLimitExceeded();
        }
    }
}
