<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class Survey extends Model
{
    use HasFactory;

    protected $fillable = ['title', 'purpose', 'type','program_id','duration','settings','team_respondents','instance_id',
    'start_date','end_date','created_by','status','updated_by','frequency','survey_form_id','mode','self_respondents'];

    /**
     * The attributes that should be cast.
     *
     * @var array
     */
    protected $casts = [
        'settings' => 'array',
        'team_respondents' => 'array',
    ];

    public function regions()
    {
        return $this->belongsToMany(Lookup::class,'region_survey','survey_id','region_id');
    }

    public function createdBy()
    {
        return $this->belongsTo(User::class,'created_by');
    }

    public function updatedBy()
    {
        return $this->belongsTo(User::class,'updated_by');
    }

    public function _status()
    {
        return $this->belongsTo(Lookup::class,'status');
    }

    public function _type()
    {
        return $this->belongsTo(Lookup::class,'type');
    }

    public function program()
    {
        return $this->belongsTo(Program::class);
    }

    public function form()
    {
        return $this->belongsTo(SurveyForm::class,'survey_form_id');
    }

    public function teams()
    {
        return $this->belongsToMany(Team::class)->withPivot('status', 'id');
    }

    public function surveyEntries()
    {
        return $this->hasMany(Entry::class);
    }

    public function scopeActive($query)
    {
        return $query->whereDate('start_date','<=',date('Y-m-d'))->whereDate('end_date','>=',date('Y-m-d'));
    }


    /**
     * The survey entries.
     *
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function entries()
    {
        return $this->hasMany(Entry::class);
    }

    /**
     * Check if survey accepts guest entries.
     *
     * @return bool
     */
    public function acceptsGuestEntries()
    {
        return $this->settings['accept-guest-entries'] ?? false;
    }

    /**
     * The maximum number of entries a participant may submit.
     *
     * @return int|null
     */
    public function limitPerParticipant()
    {
        if ($this->acceptsGuestEntries()) {
            return;
        }

        $limit = $this->settings['limit-per-participant'] ?? 1;

        return $limit !== -1 ? $limit : null;
    }

    /**
     * Survey entries by a participant.
     *
     * @param  User  $participant
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function entriesFrom(User $participant)
    {
        return $this->entries()->where('participant_id', $participant->id);
    }

    /**
     * Last survey entry by a participant.
     *
     * @param  User  $participant
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function lastEntry(User $participant = null)
    {
        return $participant === null ? null : $this->entriesFrom($participant)->first();
    }

    /**
     * Check if a participant is eligible to submit the survey.
     *
     * @param  User|null  $model
     * @return bool
     */
    public function isEligible(User $participant = null)
    {
        if ($this->mode == 'assisted') {
            $team = $this->teams->where('user_id','=',$participant->id)->count();

            if ($team > 0) {
                return true;
            }

            return false;
        }
        elseif ($this->mode == 'self') {

            if ($participant === null) {
                return $this->acceptsGuestEntries();
            }
            else {
                if ($this->limitPerParticipant() > $this->entriesFrom($participant)->count()) {
                    if ($this->self_respondents == 'all-users') return true;
                    if ($this->self_respondents == 'administrators' && $participant->is_admin) return true;
                    if ($this->self_respondents == 'applicant' && !$participant->is_admin) return true;
                }
                else {
                    return false;
                }

            }
        }


//        //PREVIOUS
//        if ($participant === null) {
//            return $this->acceptsGuestEntries();
//        }
//
//        if ($this->limitPerParticipant() === null) {
//            return true;
//        }
//
//        return $this->limitPerParticipant() > $this->entriesFrom($participant)->count();
    }

}
